<?php
 /**
 * CoolCache: Gestion de cache pages dynamiques
 *
 * Utilise l'output buffering: fonctions type ob_xxx de PHP
 * Sample usage-1:
 *   $cc = new VWT_coolCache('cacheFileName',300);
 *   if($cc->processCache()) {
 *      .... // Ce que vous voulez cacher
 *      $cc->endCache();
 *   }
 * Sample usage-2:
 *   $cc = new VWT_coolCache('');
 *   if($cc->processSegment('cacheName1',200)) {
 *      .... // Ce que vous voulez cacher
 *      $cc->endCache();
 *   }
 *   .... // Suite non cache
 *   if($cc->processSegment('cacheName2',600)) {
 *      .... // Autre partie  cacher
 *      $cc->endCache();
 *   }
 *
 * @author KDO
 * @version 0.4
 * @since 2003-02-15
 * @copyright KDO / VWT (The Virtual Web Team)
 * @package      io
 * @subpackage   cache
 */
class CoolCache {

	/**
	* Les donnes  mettre ou contenue en cache
	*
	* @vartype string
	* @access private
	*/
	var $data;

	/**
	* le rpertoire o enregistrer les caches
	* Vous devez vrifier que vous possdez bien
	* les droits en lecture/criture sur ce rpertoire
	*
	* @vartype string
	* @access private
	*/
	var $cacheDir = '/tmp/';

	/**
	* Le chemin complet du fichier cache
	*
	* @vartype string
	* @access private
	*/
	var $file;

	/**
	* Nom du cache
	*
	* @vartype string
	* @access private
	*/
	var $Fname;

	/**
	* Timestamp de validit du cache
	*
	* @vartype int
	* @access private
	*/
	var $refreshTime;

	/**
	* indique si le cache a besoin d'tre rafrachi (1 si oui, 0 sinon)
	*
	* @vartype int
	* @access private
	*/
	var $needUpdate = 0;

	/**
	* indique si coolCache est en cours de traitement
	*
	* @vartype boolean
	* @access private
	*/
	var $isActive = false;

	/**
	* indique si coolCache traite un segment de page
	*
	* @vartype boolean
	* @access private
	*/
	var $isSegment = false;
	
	/**
	* la dure de validit d'une page
	*
	* @vartype integer
	* @access private
	*/
	var $timeout;
	
	/**
	* headers  envoyer en plus, par dfaut rien de particulier
	*
	* @vartype array
	* @access private
	*/
	var $headers = array();

	/**
	* Constructeur de la classe
	*
	* Cre le cache  partir du nom du fichier et de son temps de timeout
	*
	* @param string $fname nom du fichier
	* @param int $timeout dure en secondes de validit du cache
	* @access public
	*/
	function CoolCache($fname, $timeout=180) {
		$this->Fname = $fname;
		$this->file = $this->cacheDir.$fname;
		$this->refreshTime = time() - $timeout;
		$this->timeout = $timeout;
	}

	/**
	* Dfinit le rpertoire o sont stocks les fichiers de cache
	*
	* @param string $dir rpertoire o sont stocks les fichiers de cache
	* @access public
	*/
	function setCacheDirectory($dir) {
		$this->cacheDir = $dir;
		$this->file = $dir.$this->Fname;
	}

	/**
	* Dfinit un header.
	*/
	function setHeader($header) {
		if (0 != strlen($header)) {
			$this->headers[] = $header;
		}
	}
	
	/**
	* Affiche les headers
	*/
	function sendHeaders() {
		foreach ($this->headers as $header) {
			header($header);
		}
	}

	/**
	* Ralise le cache
	*
	* @return boolean FALSE si le contenu  du cache est valide
	* @access public
	*/
	function processCache() {
		if(!$this->isActive ) {
			$this->isActive = TRUE;
			ob_start();
			ob_implicit_flush (0);
			if (file_exists($this->file) && (($this->timeout == -1) || (filemtime( $this->file ) > $this->refreshTime)) ) {
				if (!$this->isSegment) {
					if (isset($_SERVER["HTTP_IF_MODIFIED_SINCE"])) {
						$gmtime = gmdate('D, d M Y H:i:s', filemtime( $this->file )) . ' GMT';
                        $usertime = substr($_SERVER["HTTP_IF_MODIFIED_SINCE"], 0, strpos($_SERVER["HTTP_IF_MODIFIED_SINCE"], 'GMT') + 3);
						if ($usertime == $gmtime) {
							header("HTTP/1.1 304 Not Modified");
							ob_end_flush();
							exit;
						}
					}
				}
            	$this->read();
                if (empty($gmtime)) $gmtime = gmdate('D, d M Y H:i:s', filemtime( $this->file )) . ' GMT';
                header("Last-Modified: ".$gmtime);
            	echo $this->data;
            	ob_end_flush();
            	return FALSE;

			} else {
				$this->needUpdate = 1;
				return TRUE;
			}
		}
	}

	/**
	* Ralise le cache pour un segment donn
	*
	* @since  2002-02-21
	* @return boolean FALSE si le contenu  du cache est valide
	* @access public
	*/
	function processSegment($fname, $timeout=180) {
		if(!$this->isActive ) {
			$this->isSegment = TRUE;
			$this->Fname = $fname;
			$this->file = $this->cacheDir.$fname;
			$this->refreshTime = time() - $timeout;
            $this->timeout = $timeout;
			return $this->processCache();
		}
	}

	/**
	* Termine le cache
	*
	* @access public
	*/
	function endCache() {
		$this->isActive = FALSE;
		if($this->needUpdate) {
			$this->data = ob_get_contents();
			$this->write();
			if (!$this->isSegment) {
				clearstatcache();
				$FT = filemtime( $this->file );
				$gnow = gmdate('D, d M Y H:i:s', $FT+$this->timeout) . ' GMT';
                if ($this->timeout != -1)
    				header('Expires: '.$gnow);
				$gnow = gmdate('D, d M Y H:i:s', $FT ) . ' GMT';
				header('Last-Modified: '.$gnow);
			}
			ob_end_flush();
			$this->isSegment = FALSE;
		}
	}

	/**
	* Ecrit les donnes dans le fichier de cache
	*
	* @return TRUE si l'on a pu crire dans le fichier, FALSE sinon
	* @access private
	*/
	function write() {
		$fp=@fopen($this->file, 'w'); /***** GANF : chang le mode "w" en "a" pour ne pas tronquer avant d'avoir le lock *****/
		if ($fp) {
			flock($fp, LOCK_EX); /***** GANF : on demande un lock exclusif pour l'criture *****/
			ftruncate($fp,0); /***** GANF : maintenant qu'on a le seul pointeur on tronque *****/
			rewind($fp); /***** GANF : on se replace, comme si on avait ouvert en +w *****/
			fwrite($fp, $this->data, strlen($this->data));
			flock($fp, LOCK_UN);  /***** GANF : lock relach, on peut acceder en lecture *****/
			fclose($fp);
			return TRUE;
		}
		return FALSE;
	}

	/**
	* Lit les donnes dans le cache
	*
	* @return boolean - TRUE si l'on a pu lire le cache, FALSE sinon
	* @access private
	*/
	function read() {
		$fp=@fopen($this->file, 'r');
		if ($fp) {
			flock($fp, LOCK_SH);
			$this->data = '';
			while (($tmp=fread($fp, 4096))) {
				$this->data .= $tmp;
			}
			flock($fp, LOCK_UN); /***** GANF : ajout un ralachement du lock, mme si normalement ca marche sans *****/
			fclose($fp);
			return TRUE;
		}
		return FALSE;
	}
}
?>
