<?php
/**
 * lments de configuration pour WikiRenderer 2.0RC1, pour tre compatible avec wiki2xhtml
 * @author Laurent Jouanneau <jouanneau@netcourrier.com>
 * @copyright 2003 Laurent Jouanneau
 * @module Wiki Renderer
 * @version 2.0RC1
 * @since 20/12/2003
 *
 * le code de la fonction w2x_buildlink provient de la class wiki2xhtml sous licence MPL
 * http://www.neokraft.net/docs/wiki2xhtml/, copyright Olivier Meunier.
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


class Wiki2XhtmlConfig {
  /**
 	* @var array	liste des tags inline
   */
	var $inlinetags= array(
      'strong' =>array('__','__',      null,null),
      'em'     =>array('\'\'','\'\'',  null,null),
      'code'   =>array('@@','@@',      null,null),
      'q'      =>array('^^','^^',      array('lang','cite'),null),
	  'cite'   =>array('{{','}}',      array('title'),null),
      'acronym'=>array('??','??',      array('title'),null),
      'link'   =>array('[',']',        array('href','lang','title'),'w2x_buildlink'),
	  'image'  =>array('((','))', 	   array('src','alt','align','longdesc'),'w2x_wikibuildimage'),
      'anchor' =>array('~~','~~',      array('name'),'wikibuildanchor')
	);

   /**
   * liste des balises de type bloc autorises.
   * Attention, ordre important (p en dernier, car c'est le bloc par defaut..)
   */
   var $bloctags = array('title'=>true, 'list'=>true,
   'pre'=>true,'hr'=>true, 'blockquote'=>true,'definition'=>true, 'tableth'=>true, 'p'=>true);


   var $simpletags = array('%%%'=>'<br />');

   /**
    * @var	integer	niveau minimum pour les balises titres
    */
   var $minHeaderLevel=2;
   var $headerOrder=false;
   var $blocAttributeTag='';
   var $inlineTagSeparator='|';
   var $escapeSpecialChars=true;
   var $checkWikiWord = false;
   var $checkWikiWordFunction = null;
}

// ===================================== fonctions de gnrateur de code HTML spcifiques  certaines balises inlines


function w2x_wikibuildimage($contents, $attr){
	global $_PX_context;
	$baseurl = '';
	if (!empty($_PX_context['out'])) {
        $baseurl = $_PX_context['out'];
    }
   $cnt=count($contents);
   $attribut='';
	if($cnt > 4) $cnt=4;
	switch($cnt){
		case 4:
      	$attribut.=' longdesc="'.$contents[3].'"';
		case 3:
      	if($contents[2]=='l' ||$contents[2]=='L' || $contents[2]=='g' || $contents[2]=='G')
      		$attribut.=' style="float:left;"';
         elseif($contents[2]=='r' ||$contents[2]=='R' || $contents[2]=='d' || $contents[2]=='D')
      		$attribut.=' style="float:right;"';
		case 2:
      	$attribut.=' alt="'.$contents[1].'"';
		case 1:
      default:
	  	  if (ereg('^/',$contents[0]))
	  	 	$contents[0] = $baseurl.$contents[0];
      	$attribut.=' src="'.$contents[0].'"';
         if($cnt == 1) $attribut.=' alt=""';
   }
   return '<img'.$attribut.' />';

}

/**
 * traite les signes de types table
 */
class WRB_tableth extends WikiRendererBloc {
   var $type='table';
   var $regexp="/^\| ?(.*)/";
   var $_openTag='<table border="1">';
   var $_closeTag='</table>';

   var $_colcount=0;

   function open(){
      $this->_colcount=0;
      return $this->_openTag;
   }


   function getRenderedLine(){

      $result=explode(' | ',trim($this->_detectMatch[1]));
      $str='';
      $t='';

      if((count($result) != $this->_colcount) && ($this->_colcount!=0))
         $t='</table><table border="1">';
      $this->_colcount=count($result);

      for($i=0; $i < $this->_colcount; $i++){
         if (preg_match('/\s*!(.*)!\s*$/', $result[$i], $match)) {
             $str .= '<th>'. $this->_renderInlineTag($match[1]).'</th>';
         } else {
             $str.='<td>'. $this->_renderInlineTag($result[$i]).'</td>';
         }
      }
      $str=$t.'<tr>'.$str.'</tr>';

      return $str;
   }

}


// une partie du code de cette fonction provient de la class wiki2xhtml http://www.neokraft.net/docs/wiki2xhtml/
function w2x_buildlink($contents, $attr){
	global $_PX_context;
	$baseurl = '';
	if (!empty($_PX_context['out'])) {
        $baseurl = $_PX_context['out'];
    }
   $cnt=count($contents);
   $attribut='';
   if($cnt>1) {
   	if (preg_match('#^xlink://(.+)#',$contents[1],$preg)) {
        $data = explode('/',$preg[1]);
        if(count($data) == 1) {
            $id   = $data[0];
            $type = '';
        } elseif(count($data) > 1) {
            $id   = $data[0];
            $type = $data[1];
        }
        return textXlinkCreate($id,$type,$contents[0]);
    } 
      if($cnt> count($attr))
         $cnt=count($attr)+1;
      if(strpos($contents[1],'javascript:')!==false) // for security reason
         $contents[1]='#';
	  if (ereg('^/',$contents[1]))
	  	 $contents[1] = $baseurl.$contents[1];
	  if (ereg('^mailto:',$contents[1])) {
	     $contents[1] = 'mailto:'.textHexEncode(substr($contents[1],7));
         $contents[0] = textHexEncode($contents[0], true);
	  }
      for($i=1;$i<$cnt;$i++){
         $attribut.=' '.$attr[$i-1].'="'.$contents[$i].'"';
      }
   }elseif ($cnt == 1) {
   	if (preg_match('#^xlink://(.+)#',$contents[0],$preg)) {
        $data = explode('/',$preg[1]);
        if(count($data) == 1) {
            $id   = $data[0];
            $type = '';
        } elseif(count($data) > 1) {
            $id   = $data[0];
            $type = $data[1];
        }
        return textXlinkCreate($id,$type,'');
    }
      if(strpos($contents[0],'javascript:')!==false) // for security reason
         $contents[0]='#';
	  if (ereg('^/',$contents[0]))
	  	 $contents[0] = $baseurl.$contents[0];

      $attribut=' href="'.$contents[0].'"';
      if(strlen($contents[0]) > 40)
         $contents[0]=substr($contents[0],0,40).'(..)';
   }
   if ($cnt >= 1) 
	   return '<a'.$attribut.'>'.$contents[0].'</a>';
	return '';


}

?>
