<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

/**
@class Template loads the templates.

*/
//require_once dirname(__FILE__).'/../inc/class.l10n.php';


class Template
{
	var $query      = ''; /**< Query, not always important. */
	var $parsed_query = array();
	var $script     = ''; /**< PHP_SELF .*/

	var $_cache_dir = ''; /**< Cache directory. */
	var $template   = ''; /**< Template to be loaded. */

	var $resource   = array(); /**< Data of the resource from the db or cache file. */

	var $con        = null; /**< DB connection object. */

	function Template($script='', $query='')
	{
		$this->parsed_query = $this->parseQuery('/'.trim($query));
		$this->script = basename($script);
	}

	function setParam($key, $value)
	{
		$GLOBALS['_PX_render']['params'][$key] = $value;
	}

	function getParam($key)
	{
		if (!empty($GLOBALS['_PX_render']['params'][$key])) {
			return $GLOBALS['_PX_render']['params'][$key];
		}
		return '';
	}

	/**
	Render the page (display).
	*/
	function render()
	{
		global $_PX_config, $_PX_website_config, $_PX_queries;

		$fromindex = false;
		if (preg_match('/^index/', $this->script)) {
			//The user is requesting a resource, need to find it
			$this->getResource();
			$fromindex = true;
		}

		$tmp = $this->findTemplate();
		if ('' == $tmp or '404.php' == $tmp) {
			$lookup = (!empty($_SERVER['QUERY_STRING'])) ? $_SERVER['QUERY_STRING'] : '';
			$lookup = (!empty($lookup)) ? $lookup : $_SERVER['REQUEST_URI'];
			$this->getPossibleMatch($lookup);
			$this->script = '404.php';
			$this->template = '404.php';
			$this->parsed_query = array();
			$this->setParam('q', $lookup);
			$fromindex = false;
		}

		$lifetime = $this->getLifeTime();
		if ($fromindex) {
			$cachetime = (!empty($this->resource['category_cachetime'])) ? $this->resource['category_cachetime'] : $this->resource['subtype_cachetime'];
			$cachetime = ($lifetime < $cachetime) ? $lifetime : $cachetime;
			$resid = (!empty($this->resource['resource_id'])) ? $this->resource['resource_id'] : $this->resource['category_id'];
			$id = $this->getCacheId($this->parsed_query);
			$mode = $this->parsed_query['mode'];
			$cachefile = '/cachedata_'.$mode.'_'.$resid.'_'.$id;
		} else {
			$cachetime = $lifetime;
			$mode = substr($this->script, 0, -4); // remove the '.php' or '.rdf'
			$cachefile = '/cachedata_'.$mode.'_';
		}

		include_once dirname(__FILE__).'/../extinc/class.coolcache.php';

		if ($GLOBALS['_PX_config']['debug']) $cachetime = 0; //do not cache in debug mode
		$cache = new coolcache('default', $cachetime);
		$cache->setCacheDirectory($this->getCacheDir().$cachefile);
		if (preg_match('/^404/', $this->script)) {
			$cache->setHeader('Status: 404 Not Found');
		}
		$cache->setHeader($this->getHeader($this->template));

		// Initialization of variables for access in the template functions
		// and in the template
		$GLOBALS['_PX_render']['m'] = null; //render manager object
		$GLOBALS['_PX_render']['mode'] = $mode; //current mode
		$GLOBALS['_PX_render']['parsed_query'] = $this->parsed_query;
		$GLOBALS['_PX_render']['resource'] = $this->resource;
		$GLOBALS['_PX_render']['resource']['page'] = (!empty($this->parsed_query['page'])) ? $this->parsed_query['page'] : 1;


		$GLOBALS['_PX_render']['last'] = '';
		$GLOBALS['_PX_render']['res'] = '';
		$res =& $GLOBALS['_PX_render']['res']; //for easy access
		$last =& $GLOBALS['_PX_render']['last']; //for easy access

		$cache->sendHeaders(); //this send the general headers, the cache system will send also the modified time headers
		include $GLOBALS['_PX_config']['manager_path'].'/templates/'.$GLOBALS['_PX_website_config']['website_id'].'/'.$this->template;

		if (!empty($this->con) && is_object($this->con)) {
			$this->con->close();
		}
	}

	/**
	Get the cache id of the resource from the website configuration and the
	parsed query string array.

	@param array parsed query string
	@param array website configuration (use global if empty)
	@return string id
	*/
	function getCacheId($res, $website=array())
	{
		$websiteid = (!empty($website['website_id'])) ? $website['website_id'] : $GLOBALS['_PX_website_config']['website_id'];
		return md5($websiteid.'%cat='.$res['category'].'%res='.$res['resource'].'%p='.$res['page'].'%mode='.$res['mode']);
	}

	/**
	Get the diff between update time and now from the MASS_UPDATE file
	if no MASS_UPDATE return 0. No clearstatscache is run before.

	@param string website id (use global if empty)
	@return int lifetime of the cache
	*/
	function getLifeTime($website='')
	{
		$website = (!empty($website)) ? $website : $GLOBALS['_PX_website_config']['website_id'];
		$file =  $GLOBALS['_PX_config']['manager_path'].'/cache/'.$website.'/MASS_UPDATE';
		if (!file_exists($file)) @touch($file, time());
		if (false !== ($time = @filemtime($file)))
			return time() - $time;
		return 0;
	}

	/**
	Get extra header from the template extension
	*/
	function getHeader($template)
	{
		$ext = strtolower(substr(strrchr($template, '.'), 1));
		$encoding = strtolower($GLOBALS['_PX_config']['encoding']);
		$headers = array(
		  'htm'   => 'text/html; charset='.$encoding,
		  'html'  => 'text/html; charset='.$encoding,
		  'php'   => 'text/html; charset='.$encoding,
		  'xhtml' => 'application/xhtml+xml; charset='.$encoding,
		  'txt'   => 'text/plain; charset='.$encoding,
		  'rss'   => 'text/xml; charset='.$encoding,
		  'rdf'   => 'text/xml; charset='.$encoding,
		  'xml'   => 'text/xml; charset='.$encoding,
		);
		if (!empty($headers[$ext])) {
			return sprintf('Content-Type: %s', $headers[$ext]);
		}
		return '';
	}

	/**
	Find the right template to be used. Return the template basename.

	*/
	function findTemplate()
	{
		// case of a call from index.php
		$this->template = '';
		if (preg_match('/^index/', $this->script)) {
			if (!empty($this->resource['subtype_template'])) {
				$this->template = $this->resource['subtype_template'];
			} else {
				$this->template = (!empty($this->resource['category_template'])) ? $this->resource['category_template'] : '';
			}
		} elseif (preg_match('/^rss/', $this->script)) {
			$this->template = 'rss.rdf';
		} elseif (preg_match('/^search/', $this->script)) {
			$this->template = 'search.php';
		} elseif (preg_match('/^404/', $this->script)) {
			$this->template = '404.php';
		}
		return $this->template;
	}
	
	/**
	Get possible matching resource before sending an error 404.
	If the match is done, the redirection is done and the script
	exists. A move permanently header is issued.
	
	TODO: This function should be cleaned and maybe moved in a class.
	*/
	function getPossibleMatch($query)
	{
		include_once dirname(__FILE__).'/../inc/lib.sqlutils.php';
		$this->con =& pxDBConnect();
		if ('/' != substr($query, 0, 1)) $query = '/'.$query;
		$r = 'SELECT * FROM '.$this->con->pfx.'categories
			    LEFT JOIN '.$this->con->pfx.'websites ON '.$this->con->pfx.'websites.website_id = '.$this->con->pfx.'categories.website_id
			    WHERE '.$this->con->pfx.'categories.website_id=\''.$this->con->escapeStr($GLOBALS['_PX_website_config']['website_id']).'\'
			    AND '.$this->con->pfx.'categories.category_path=\''.$this->con->escapeStr($query.'/').'\'';
		if (($rs = $this->con->select($r)) !== false) {
			if ($rs->f('category_id')) {
				//bingo the error was a missing / at the end for a category
				$this->con->close();
				header('HTTP/1.1 301 Moved Permanently');
				header('Location: '.$GLOBALS['_PX_website_config']['rel_url'].$query.'/');
				header('Connection: close');
				exit;
			}
		} else {
			header('Content-Type: text/plain');
			echo 'MySQL : '.$this->con->error();
			exit;
		}
		//need to see if a redirection is available.
		$r = 'SELECT * FROM '.$this->con->pfx.'smart404 WHERE
		      website_id=\''.$this->con->escapeStr($GLOBALS['_PX_website_config']['website_id']).'\' AND
		      oldpage=\''.$this->con->escapeStr($query).'\'';
		if (($rs = $this->con->select($r)) !== false) {
			if ($rs->f('newpage')) {
				//bingo
				$page = $rs->f('newpage');
				//try to update the count
				$r = 'UPDATE '.$this->con->pfx.'smart404 SET
				lastroutingdate=\''.timestamp().'\', total=total+1
				WHERE website_id=\''.$this->con->escapeStr($GLOBALS['_PX_website_config']['website_id']).'\' AND
		        oldpage=\''.$this->con->escapeStr($query).'\' LIMIT 1';
				$this->con->execute($r); //we do not check if errors as we already know the page
				@$this->con->close();
				header('HTTP/1.1 301 Moved Permanently');
				header('Location: '.$page);
				header('Connection: close');
				exit;
			}
		} else {
			header('Content-Type: text/plain');
			echo 'MySQL : '.$this->con->error();
			exit;
		}
		//log the error if log requested
		if ($GLOBALS['_PX_config']['log404errors']) {
				$ref = (!empty($_SERVER['HTTP_REFERER'])) ? $_SERVER['HTTP_REFERER'] : '';
				$agent = (!empty($_SERVER['HTTP_USER_AGENT'])) ? substr($_SERVER['HTTP_USER_AGENT'],0, 250) : '';
				$r = 'INSERT INTO '.$this->con->pfx.'smart404logs SET
                website_id=\''.$this->con->escapeStr($GLOBALS['_PX_website_config']['website_id']).'\',
                page=\''.$this->con->escapeStr($query).'\', fromurl=\''.$this->con->escapeStr($ref).'\',
                date=\''.timestamp().'\', useragent=\''.$this->con->escapeStr($agent).'\'';
				$this->con->execute($r); //we do not check if errors as it is not critical
		}
	}



	/**
	Get the resource data from the db or a cached file.
	*/
	function getResource()
	{
		global $_PX_config;
		$id = $this->getCacheId($this->parsed_query);
		$this->resource = array();

		include_once $_PX_config['manager_path'].'/extinc/class.cachelite.php';
		$cache = new CacheLite(array('cacheDir' => $this->getCacheDir().'/',
		                             'lifeTime' => $this->getLifeTime(),
									 'caching' => !$GLOBALS['_PX_config']['debug']));
		if (false !== ($data = $cache->get($id))) {
			$this->resource = unserialize($data);
			if (!empty($this->resource['resource_id'])) {
				if ($this->resource['publicationdate'] <= timestamp()
				   && $this->resource['enddate'] >= timestamp()) {
					return $this->resource;
				} else {
					$this->resource = array();
					return $this->resource;
				}
			}
			return $this->resource;
		}

		// here need to load all the bloat of the db to get the resource if available
		if (false === $this->getResourceFromDB()) {
			return false;
		}
		// save in the cache for next time
		if (is_array($this->resource) && 0 < count($this->resource)) {
			$cache->save(serialize($this->resource));
		}
		return $this->resource;
	}

	/**
	Get the resource data from the DB.
	If an error occure during the query, it simply stop the script and display
	the SQL error.
	*/
	function getResourceFromDB()
	{
		include_once dirname(__FILE__).'/../inc/lib.sqlutils.php';
		$this->con =& pxDBConnect();

		if ($this->parsed_query['mode'] == 'cat') {
			// get cat info
			$r = 'SELECT * FROM '.$this->con->pfx.'categories
			    LEFT JOIN '.$this->con->pfx.'websites     ON '.$this->con->pfx.'websites.website_id      = '.$this->con->pfx.'categories.website_id
			    WHERE '.$this->con->pfx.'categories.website_id=\''.$this->con->escapeStr($GLOBALS['_PX_website_config']['website_id']).'\'
			    AND '.$this->con->pfx.'categories.category_path=\''.$this->con->escapeStr($this->parsed_query['category']).'\'';
			if (($rs = $this->con->select($r)) !== false) {
				if ($rs->f('category_id')) {
					$this->resource = $rs->arry_data[0];
					return true;
				}
				$this->resource = false;
				return false;
			} else {
				header('Content-Type: text/plain');
				echo 'MySQL : '.$this->con->error();
				exit;
			}
		} else {
			// get resource info

			/* extra LEFT JOIN or WHERE depending of the type of resource (mostly for the articles as
			 * they have this 'page number' information)
			 */
			$leftjoin = '';
			$where    = '';
			if (0 < $this->parsed_query['page'] && $this->parsed_query['mode'] == 'art') {
				$leftjoin = 'LEFT JOIN '.$this->con->pfx.'articles ON '.$this->con->pfx.'resources.resource_id = '.$this->con->pfx.'articles.resource_id';
				$where    = 'AND path LIKE \''.$this->con->escapeStr($this->parsed_query['resource']).'\' AND page_number = \''.$this->con->escapeStr($this->parsed_query['page']).'\'';
			} elseif ($this->parsed_query['mode'] == 'news'){
				$where    = 'AND resource_id=\''.$this->con->escapeStr($this->parsed_query['resource']).'\'';
			}

			$r = 'SELECT * FROM '.$this->con->pfx.'resources
			     LEFT JOIN '.$this->con->pfx.'categoryasso ON '.$this->con->pfx.'categoryasso.identifier  = '.$this->con->pfx.'resources.identifier
			     LEFT JOIN '.$this->con->pfx.'categories   ON '.$this->con->pfx.'categoryasso.category_id = '.$this->con->pfx.'categories.category_id
			     LEFT JOIN '.$this->con->pfx.'websites     ON '.$this->con->pfx.'websites.website_id      = '.$this->con->pfx.'resources.website_id
			     LEFT JOIN '.$this->con->pfx.'subtypes     ON '.$this->con->pfx.'subtypes.subtype_id      = '.$this->con->pfx.'resources.subtype_id
			     '.$leftjoin.'
			     WHERE
			     '.$this->con->pfx.'resources.website_id=\''.$this->con->escapeStr($GLOBALS['_PX_website_config']['website_id']).'\'
			     AND category_path LIKE \''.$this->con->escapeStr($this->parsed_query['category']).'\'
			     AND '.$this->con->pfx.'resources.publicationdate <= '.timestamp().'
			     AND '.$this->con->pfx.'resources.enddate >= '.timestamp().'
			     AND '.$this->con->pfx.'resources.status=1
			     '.$where.' LIMIT 1';

			if (($rs = $this->con->select($r)) !== false) {
				if ($rs->f('resource_id')) {
					$this->resource = $rs->arry_data[0];
					return true;
				}
				$this->resource = false;
				return false;
			} else {
				header('Content-Type: text/plain');
				echo 'MySQL : '.$this->con->error();
				exit;
			}
		}
	}


	/**
	Get the desired resource by the user from the query string.
	This function does not validate that the resource is valid.
	This function should not be used for the pages: rss.php, 404.php
	and search.php as there is no resource to get.

	@return array
	*/
	function parseQuery($query)
	{
		$query = preg_replace( '#^(/)+#', '/', $query);
		$category = $resource = $page = $mode = '';

		if (preg_match('#^(.*/)(index)([0-9]*)$#i', $query, $match)) {
			$category = $match[1];
			$page     = ($match[3]) ? (int) $match[3] : 1;
			$mode     = 'cat';
		} elseif (preg_match('#^(.*/)$#i', $query, $match)) {
			$category = $match[1];
			$page     = 1;
			$mode     = 'cat';
		} elseif (preg_match('#^(.*/)([a-z]|[a-z][a-z0-9\_\-]*[a-z])(\d*)$#i', $query, $match)) {
			$category = $match[1];
			$resource = $match[2];
			$page     = ($match[3]) ? (int)$match[3] : 1;
			$mode     = 'art';
		} elseif (preg_match('#^(.*/)(\d+)\-([^\.]*)$#i', $query, $match)) {
			$category = $match[1];
			$resource = $match[2];
			$mode     = 'news';
		} else {
			$category = '/';
			$mode     = 'cat';
			$page     = 1;
		}
		return array('category' => $category, 'resource' => $resource, 'page' => $page, 'mode' => $mode);
	}

	/**
	Get the cache directory. Create it if necessary.
	*/
	function getCacheDir($website='')
	{
		if (!empty($this->_cache_dir)) return $this->_cache_dir;

		$website = (!empty($website)) ? $website : $GLOBALS['_PX_website_config']['website_id'];
		if (file_exists($GLOBALS['_PX_config']['manager_path'].'/cache/'.$website)) {
			$this->_cache_dir = $GLOBALS['_PX_config']['manager_path'].'/cache/'.$website;
			return $this->_cache_dir;
		}

		//we make the cache dir
		@mkdir($GLOBALS['_PX_config']['manager_path'].'/cache/'.$website, 0777);
		@chmod($GLOBALS['_PX_config']['manager_path'].'/cache/'.$website, 0777);
		$this->_cache_dir = $GLOBALS['_PX_config']['manager_path'].'/cache/'.$website;
		return $this->_cache_dir;
	}

	/**
	Get the database connection.
	*/
	function getDB()
	{
		include_once dirname(__FILE__).'/../inc/lib.sqlutils.php';
		$this->con =& pxDBConnect();
	}

}


/**
 Initialisation of the template, it loads all the other librairies. Needed only
 if the page is not cached, else we try to load the minimum.

*/
function pxTemplateInit($params = '')
{
	global $_PX_config, $_PX_website_config;
	include_once $_PX_config['manager_path'] . '/frontinc/prepend.php';

	if (is_null($GLOBALS['_PX_render']['m'])) {
		$GLOBALS['_PX_render']['m'] = new RenderManager();
		$GLOBALS['_PX_render']['m']->res =& $GLOBALS['_PX_render']['res']; //for easy access
		$GLOBALS['_PX_render']['m']->lres =& $GLOBALS['_PX_render']['last']; //for easy access
	}

	// Parameters
	$params = '|'.$params;

	$GLOBALS['_PX_render']['m']->setParam('order_res_manual', strpos($params, 'order_res_manual'));
	$GLOBALS['_PX_render']['m']->setParam('order_cat_manual', strpos($params, 'order_cat_manual'));
	$GLOBALS['_PX_render']['m']->setParam('remove_numbers', strpos($params, 'remove_numbers'));
	$GLOBALS['_PX_render']['m']->setParam('full_paths', strpos($params, 'full_paths'));

	$order = 'DESC';
	if ($GLOBALS['_PX_render']['m']->getParam('order_res_manual')) {
		$order = 'ORDER BY %sresources.title ASC';
	}
	if ($GLOBALS['_PX_render']['m']->getParam('full_paths')) {
		$GLOBALS['_PX_context']['out'] = getCurrentHostUrl();
	}
	if (preg_match('/res_per_page:(\d+)/',$params,$match)) {
		$GLOBALS['_PX_render']['m']->NbResPerPage = (int) $match[1];
	}
	if (preg_match('/set_lang:([a-z]{2})/i',$params,$match)) {
		setlocale(LC_ALL, strtolower($match[1]));
	} elseif (!empty($_PX_website_config['lang'])) {
		 setlocale(LC_ALL, $_PX_website_config['lang']);
	}

	// depending on the mode, different actions are taken.
	switch ($GLOBALS['_PX_render']['mode']) {
	case 'cat':
		px_getCat();
		px_getResources($GLOBALS['_PX_render']['resource']['category_id'], '', '',  $order);
		px_getSubCat();
		break;
	case 'art':
		px_getCat();
		px_getResources($GLOBALS['_PX_render']['resource']['category_id'], '', '',  $order);
		px_getSubCat();
		$GLOBALS['_PX_render']['m']->art = $GLOBALS['_PX_render']['m']->getArticle($GLOBALS['_PX_render']['resource']['resource_id'], $GLOBALS['_PX_render']['resource']['page']);
		break;
	case 'news':
		px_getCat();
		px_getResources($GLOBALS['_PX_render']['resource']['category_id'], '', '',  $order);
		px_getSubCat();
		$GLOBALS['_PX_render']['m']->news = $GLOBALS['_PX_render']['m']->getNews($GLOBALS['_PX_render']['resource']['resource_id']);
		break;
	case 'search':
		$GLOBALS['_PX_render']['m']->res = $GLOBALS['_PX_render']['m']->searchResources($GLOBALS['_PX_render']['m']->getParam('q'), true);
		break;
	case '404':
		$GLOBALS['_PX_render']['m']->setParam('q', textUrl2words($GLOBALS['_PX_render']['m']->getParam('q')));
		$GLOBALS['_PX_render']['m']->res = $GLOBALS['_PX_render']['m']->searchResources($GLOBALS['_PX_render']['m']->getParam('q'), true);
		break;
	case 'rss':
	default:
	}
}

?>
