<?php
/*
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Credits:
#    Lot of the logic here comes from DotClear Weblog by Olivier Meunier. 
#    More details: http://www.dotclear.net/
#    DotClear Weblog is distributed under the License MPL 1.1/GPL 2.0/LGPL 2.1
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK ***** */

/**
 @proto doc
 
 !!! Introduction
 
 The PLUME CMS template system is not the simplest, but if read the documenation and
 study a little the current templates, you will quickly figure out how to use it.
 
 The system simply use PHP functions to display the data. Sometimes the use of
 the [PHP|http://www.php.net] [sprintf|http://www.php.net/sprintf] function to
 format data.

*/ 

/**
 @proto doc
 
 !!! Informative functions
 
 These are functions to display information like the name of the website
 but not within a loop.
*/ 
  
 
/**
 Display some information about the website.
 The __name__ parameter can take the following values:
 
* name : name of the website
* url : relative url of the website
* fullurl : full url of the website
* filesurl : relative url to the files and images of the website  
* lang : the lang of the website
* namexml : name utf-8 encoded

Example:

|<a href="<?php pxInfo('url'); ?>"><?php pxInfo('name'); ?></a>
 
 @proto function pxInfo 
 @param string name Property to display ('name')
*/
function pxInfo($name = 'name')
{
	global $_PX_website_config, $_PX_config;
	if (null === $GLOBALS['_PX_render']['m']->site) {
		$GLOBALS['_PX_render']['m']->site = $GLOBALS['_PX_render']['m']->getSite($_PX_website_config['website_id']);
	}
	switch ($name) {
	case 'fullurl':
		echo 'http://'.$_PX_website_config['domain'].$_PX_website_config['rel_url'].'/'; break;
	case 'url':
		echo $_PX_website_config['rel_url'].'/'; break;
	case 'filesurl':
		echo $_PX_website_config['rel_url_files'].'/'; break;
	case 'lang':
		echo str_replace('_', '-', $_PX_config['lang']); break;
	case 'description':
		echo $GLOBALS['_PX_render']['m']->site->f('website_description'); break;
	case 'namexml':
		echo textToXML($GLOBALS['_PX_render']['m']->site->f('website_name')); break;
	case 'encoding':
		echo strtolower($_PX_config['encoding']); break;
	default:
		echo $GLOBALS['_PX_render']['m']->site->f('website_name');
	}
}

/**
 Generate the 'link' links just need to put it in the head of the
 template.

|<head>
|<?php pxHeadLinks(); ?>
|</head>

 @proto function pxHeadLinks

*/
function pxHeadLinks()
{
	global $_PX_website_config, $_PX_config;

	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$link_sep = ($_PX_config['url_format'] == 'simple') ? '/?' : '';

	if ($GLOBALS['_PX_render']['mode'] == 'cat') {
		// in a category
		if (null === $GLOBALS['_PX_render']['m']->cat) {
			$GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
		}
		if (null === $GLOBALS['_PX_render']['m']->res) {
			$GLOBALS['_PX_render']['m']->res = $GLOBALS['_PX_render']['m']->getResources($GLOBALS['_PX_render']['resource']['category_id'], '');
		}
		$GLOBALS['_PX_render']['m']->res->move(($GLOBALS['_PX_render']['resource']['page'] - 1) * $GLOBALS['_PX_render']['m']->NbResPerPage);
		while (!$GLOBALS['_PX_render']['m']->res->EOF()) {
			$title = $GLOBALS['_PX_render']['m']->res->f('title');
			if ($remove_numbers) $title = px_removeNumbers($title);
			echo '<link rel="section" href="'.$_PX_website_config['rel_url'].$link_sep
			     .$GLOBALS['_PX_render']['m']->res->getPath().'" title="'.$title.'" />'."\n";
			$GLOBALS['_PX_render']['m']->res->moveNext();
		}
		$GLOBALS['_PX_render']['m']->res->move(($GLOBALS['_PX_render']['resource']['page'] - 1) * $GLOBALS['_PX_render']['m']->NbResPerPage);
		echo '<meta name="keywords" content="'.htmlspecialchars($GLOBALS['_PX_render']['m']->cat->f('category_keywords')).'" />'."\n";

	} elseif ($GLOBALS['_PX_render']['mode'] == 'art') {
		echo '<meta name="keywords" content="'.htmlspecialchars($GLOBALS['_PX_render']['m']->art->f('subject').', '.$GLOBALS['_PX_render']['m']->art->f('category_keywords')).'" />'."\n";
	} elseif ($GLOBALS['_PX_render']['mode'] == 'news') {
		echo '<meta name="keywords" content="'.htmlspecialchars($GLOBALS['_PX_render']['m']->news->f('subject').', '.$GLOBALS['_PX_render']['m']->news->f('category_keywords')).'" />'."\n";
	}

}



/**
 Generate the RSS sequence description

 @proto function pxRssSeq
*/
function pxRssSeq()
{
	global $_PX_website_config, $_PX_config;
	$index = $GLOBALS['_PX_render']['m']->lres->getIndex();
	while (!$GLOBALS['_PX_render']['m']->lres->EOF()) {
		echo '<rdf:li rdf:ressource="'.$GLOBALS['_PX_render']['m']->lres->f('identifier').'" />'."\n";
		$GLOBALS['_PX_render']['m']->lres->moveNext();
	}
	$GLOBALS['_PX_render']['m']->lres->move($index);
}

/**
 Generate the RSS item sequence

 @proto function pxRssItems
*/
function pxRssItems()
{
	global $_PX_website_config, $_PX_config;
	$index = $GLOBALS['_PX_render']['m']->lres->getIndex();
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	while (!$GLOBALS['_PX_render']['m']->lres->EOF()) {
		$content = htmlspecialchars(textRemoveEntities(textParseContent($GLOBALS['_PX_render']['m']->lres->f('description'))));
		$title = htmlspecialchars(textRemoveEntities($GLOBALS['_PX_render']['m']->lres->f('title')));
		if ($remove_numbers) $title = px_removeNumbers($title);
		echo '<item rdf:ID="'.$GLOBALS['_PX_render']['m']->lres->f('identifier').'">'."\n".
		'  <title>'.$title."</title>\n".
		'  <link>'; pxLastResPath('full');  echo "</link>\n".
		'  <dc:date>'; pxLastResDateModification(); echo "</dc:date>\n".
		'  <description>'.$content."</description>\n".
		'</item>'."\n";

		echo '<rdf:li rdf:ressource="p'.$GLOBALS['_PX_render']['m']->lres->f('identifier').'" />'."\n";
		$GLOBALS['_PX_render']['m']->lres->moveNext();
	}
	$GLOBALS['_PX_render']['m']->lres->move($index);
}

/**
 Display the title of the current category.

 @proto function pxSingleCatTitle
 @param string s Substitution string ('%s - ')
 */
function pxSingleCatTitle($s = '%s - ')
{
	global $_PX_website_config, $_PX_config;
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	if (null === $GLOBALS['_PX_render']['m']->cat) {
		$GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
	}
	$title = $GLOBALS['_PX_render']['m']->cat->f('category_name');
	if ($remove_numbers) $title = px_removeNumbers($title);
	echo sprintf($s, $title);
}

/**
 Display the description of the current category.

 @proto function pxSingleCatDescription
 */
function pxSingleCatDescription()
{
    global $_PX_website_config, $_PX_config;
    if (null === $GLOBALS['_PX_render']['m']->cat) {
        $GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
    }
    echo textParseContent($GLOBALS['_PX_render']['m']->cat->f('category_description'));
}

/**
 Display the category path

 @proto function pxSingleCatPath
 @param string s Substitution string ('%s')

 */
function pxSingleCatPath($s = '%s')
{
    global $_PX_website_config, $_PX_config;
    $path = $_PX_website_config['rel_url'];
    if ($_PX_config['url_format'] == 'simple') {
        $path .= '/?';
    }
    $path .= $GLOBALS['_PX_render']['m']->cat->f('category_path');
    echo sprintf($s, $path);

}

/**
 Give the number of resources in the current category, when displaying a category page
 
 @proto function pxSingleCatNbResources
 @param string no String for no resources ('no resources')
 @param string one String for 1 resource ('1 resource')
 @param string more Substitution string for 2 or more resources ('%s resources')

 */
function pxSingleCatNbResources($no = 'no resources', $one = '1 resource', $more = '%s resources')
{
	if (null === $GLOBALS['_PX_render']['m']->cat) {
		$GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
	}
	if (null === $GLOBALS['_PX_render']['m']->res) {
		px_getResources($GLOBALS['_PX_render']['resource']['category_id'], '', '', 'DESC');
	}
	switch ($GLOBALS['_PX_render']['m']->res->NbRowTotal()) {
	case 0: echo $no; break;
	case 1: echo $one; break;
	default: echo sprintf($more, $GLOBALS['_PX_render']['m']->res->nbRowTotal()); break;
	}
}

/**
 Get the number of resources in the current category, when displaying a category page.
 If displaying a resource page, give the number of resources in the current category of the
 resource. Usefull in __if__ statements.

 @proto function pxSingleCatGetNbResources
 @return integer Number of resources
 */
function pxSingleCatGetNbResources()
{
	if (null === $GLOBALS['_PX_render']['m']->cat) {
		$GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
	}
	if (null === $GLOBALS['_PX_render']['m']->res) {
		px_getResources($GLOBALS['_PX_render']['resource']['category_id'], '', '', 'DESC');
	}
	return $GLOBALS['_PX_render']['m']->res->NbRowTotal();
}

/**
 Give a link to the previous/next page if available
  
 @proto function pxSingleCatNextPage
 @param int dir Direction -1 previous, (1) next
 @param string s substitution string ('%s')
 
 */
function pxSingleCatNextPage($dir = 1,$s='%s')
{
	if (null === $GLOBALS['_PX_render']['m']->res) {
		px_getResources($GLOBALS['_PX_render']['resource']['category_id'], '', '', 'DESC');
	}
	$total = $GLOBALS['_PX_render']['m']->res->NbRowTotal();
	$page  = $GLOBALS['_PX_render']['resource']['page'];
	$nbres = $GLOBALS['_PX_render']['m']->NbResPerPage;
	$path = $GLOBALS['_PX_website_config']['rel_url'];
	if ($GLOBALS['_PX_config']['url_format'] == 'simple') {
		$path .= '/?';
	}
	$path .= $GLOBALS['_PX_render']['m']->cat->f('category_path');
	if ($dir == -1 && $page > 1) {
		$page--;
		if (1 == $page) {
			echo sprintf($s, $path);
		} else {
			echo sprintf($s, $path.'index'.$page);
		}
	} elseif ($dir == 1 && ($total > ($page * $nbres))) {
		$page++;
		echo sprintf($s, $path.'index'.$page);
	}
}

/**
 Give the list of pages of a category with links like
 search engines. Looks like ''Pages: Prev 1, 2, 3, 4, 5 Next''.

 @proto function pxSingleCatListPages
 @param string s Substitution string ('<p>Pages: %s</p>')
 @param string prev Previous page string ('Prev')
 @param string next Next page string ('Next')
 @param string sep Separator between page numbers (', ')
*/
function pxSingleCatListPages($s='<p>Pages: %s</p>', $prev='Prev', $next='Next', $sep=', ')
{
	if (null === $GLOBALS['_PX_render']['m']->res) {
		px_getResources($GLOBALS['_PX_render']['resource']['category_id'], '', '', 'DESC');
	}
	$total = $GLOBALS['_PX_render']['m']->res->NbRowTotal();
	$activepage  = $GLOBALS['_PX_render']['resource']['page'];
	$nbres = $GLOBALS['_PX_render']['m']->NbResPerPage;
	$path = $GLOBALS['_PX_website_config']['rel_url'];
	if ($GLOBALS['_PX_config']['url_format'] == 'simple') {
		$path .= '/?';
	}
	$path .= $GLOBALS['_PX_render']['m']->cat->f('category_path');
	$pages = array();
	$list = '';
	$previouspage = '';
	$nextpage = '';
	$nbpages = (int) ($total/$nbres) + 1;
	if ($nbpages > 1) {
		for ($i=1;$i<=$nbpages;$i++) {
			$class = '';
			if ($i == $activepage) $class = 'class="current"';
			$url = $path;
			if ($i > 1) $url .= 'index'.$i;
			$pages[]='<a '.$class.'href="'.$url.'">'.$i.'</a>';
		}
		$list = join($sep, $pages);
		//Previous page
		if ($activepage > 1) {
			if (2 == $activepage) {
				$previouspage = '<a href="'.$path.'">'.$prev.'</a>';
			} else {
				$temp = $activepage - 1;
				$previouspage = '<a href="'.$path.'index'.$temp.'">'.$prev.'</a>';
			}
		}
		//Next page
		if ($total > ($activepage * $nbres)) {
			$temp = $activepage + 1;
			$nextpage = '<a href="'.$path.'index'.$temp.'">'.$next.'</a>';
		}
		echo sprintf($s, $previouspage.' '.$list.' '.$nextpage);

	}

}


/**
 Display the list of categories for a breadcrumb. Like
 Home >> Subcategory >> Subsubcategory

 @proto function pxSingleCatTree
 @param string s substitution string ('<ol>%s</ol>')
*/
function pxSingleCatTree($s = '<ol>%s</ol>')
{
	global $_PX_website_config, $_PX_config;
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$res = '';
	if (null === $GLOBALS['_PX_render']['m']->cat) {
		$GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
	}
	$categories = array();
	$i=0;
	$categories[$i] = $GLOBALS['_PX_render']['m']->cat;
	$parentcat    = $GLOBALS['_PX_render']['m']->cat->f('category_parentid');
	$currentcatid = $GLOBALS['_PX_render']['m']->cat->f('category_id');
	$i++;
	while ($parentcat !=  $currentcatid) {
		$categories[$i] = $GLOBALS['_PX_render']['m']->getCategory($parentcat);
		$parentcat    = $categories[$i]->f('category_parentid');
		$currentcatid = $categories[$i]->f('category_id');
		$i++;
	}
    $path = $_PX_website_config['rel_url'];
    if ($_PX_config['url_format'] == 'simple') {
        $path .= '/?';
    }
    reset($categories);
	foreach ($categories as $cat) {
		$title = $cat->f('category_name');
       	if ($remove_numbers) $title = px_removeNumbers($title);
		$res = '<li><a href="'.$path.$cat->f('category_path').'">'.$title.'</a></li>'."\n".$res;
	}
	echo sprintf($s, $res);
}

/**
 Display the parent name cat, only if not in homepage

 @proto function pxParentCatTitle
 @param string s Substitution string ('%s - ')

 */
function pxParentCatTitle($s = '%s - ')
{
    global $_PX_website_config, $_PX_config;
    $remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
    if (null === $GLOBALS['_PX_render']['m']->pcat) {
        $GLOBALS['_PX_render']['m']->pcat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['m']->cat->f('category_parentid'));
    }    
    if ($GLOBALS['_PX_render']['m']->cat->f('category_parentid') != $GLOBALS['_PX_render']['m']->cat->f('category_id')) {
		$title = $GLOBALS['_PX_render']['m']->pcat->f('category_name');
    	if ($remove_numbers) $title = px_removeNumbers($title);
        echo sprintf($s, $title);
    }
}

/**
 Display the parent description cat, only if not in homepage

 @proto function pxParentCatDescription

 */
function pxParentCatDescription()
{
    global $_PX_website_config, $_PX_config;
    if (null === $GLOBALS['_PX_render']['m']->pcat) {
        $GLOBALS['_PX_render']['m']->pcat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['m']->cat->f('category_parentid'));
    }
    if ($GLOBALS['_PX_render']['m']->cat->f('category_parentid') != $GLOBALS['_PX_render']['m']->cat->f('category_id')) {
        echo textParseContent($GLOBALS['_PX_render']['m']->pcat->f('category_description'));
    }
}

/**
 Display the parent category path, only if not in homepage

 @proto function pxParentCatPath
 @param string s Substitution string ('%s')

 */
function pxParentCatPath($s = '%s')
{
    global $_PX_website_config, $_PX_config;
    if (null === $GLOBALS['_PX_render']['m']->pcat) {
        $GLOBALS['_PX_render']['m']->pcat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['m']->cat->f('category_parentid'));
    }
    if ($GLOBALS['_PX_render']['m']->cat->f('category_parentid') != $GLOBALS['_PX_render']['m']->cat->f('category_id')) {
        $path = $_PX_website_config['rel_url'];
        if ($_PX_config['url_format'] == 'simple') {
            $path .= '/?';
        }
        $path .= $GLOBALS['_PX_render']['m']->pcat->f('category_path');
        echo sprintf($s, $path);
    }
}

/**
 Display the list of primary categories

 @proto function pxPrimaryCategories
 @param string s Substitution string ('<ul>%s</ul>')

*/
function pxPrimaryCategories($s = '<ul>%s</ul>')
{
	global $_PX_website_config, $_PX_config;
	$ordermanual = $GLOBALS['_PX_render']['m']->getParam('order_cat_manual');
    $remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$order = 'ORDER BY category_path';
	if ($ordermanual) {
		$order = 'ORDER BY category_name';
	}
	$rootcat = $GLOBALS['_PX_render']['m']->getCategory('/');
	$prim    = $GLOBALS['_PX_render']['m']->getCategories($rootcat->f('category_id'), $order);

	$cats = '';
	while (!$prim->EOF()) {
		if ($prim->f('category_path') != '/') {
			$path = $_PX_website_config['rel_url'];
			if ($_PX_config['url_format'] == 'simple') {
				$path .= '/?';
			}
			$path .= $prim->f('category_path');
			$name = $prim->f('category_name');
			if ($remove_numbers) {
				$name = px_removeNumbers($name);
			}
			$cats .= '<li><a href="'.$path.'">'.$name.'</a></li>'."\n";
		}
		$prim->moveNext();
	}
	echo sprintf($s, $cats);
}

/**
 Display the list of subcategories in the form of an unordered list.
 
 @proto function pxSubCategories
 @param string s Substitution string ('%s')
*/ 
function pxSubCategories($s = '%s')
{
    global $_PX_website_config, $_PX_config;
	
    if ($GLOBALS['_PX_render']['mode'] == 'cat' or $GLOBALS['_PX_render']['mode'] == 'art' 
	or $GLOBALS['_PX_render']['mode'] == 'news')  {
	
	$ordermanual = $GLOBALS['_PX_render']['m']->getParam('order_cat_manual');
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$order = 'ORDER BY category_path';
	if ($ordermanual) {
	    $order = 'ORDER BY category_name';
	}
	// if (null === $GLOBALS['_PX_render']['m']->subcat) {
	    $GLOBALS['_PX_render']['m']->subcat = $GLOBALS['_PX_render']['m']->getCategories($GLOBALS['_PX_render']['m']->cat->f('category_id'), $order);
	    //}
	$path = $_PX_website_config['rel_url'];
	if ($_PX_config['url_format'] == 'simple') {
	    $path .= '/?';
	}
	if ($GLOBALS['_PX_render']['m']->subcat->NbRow() > 0) {
	    $GLOBALS['_PX_render']['m']->subcat->MoveStart();
	    $ls = "<ul>\n";
	    while (!$GLOBALS['_PX_render']['m']->subcat->EOF()) {
		$name = $GLOBALS['_PX_render']['m']->subcat->f('category_name');
		if ($remove_numbers) {
		    $name = px_removeNumbers($name);
		}
		$ls .= '<li><a href="'.$path.$GLOBALS['_PX_render']['m']->subcat->f('category_path').'">'.$name.'</a></li>'."\n";
		$GLOBALS['_PX_render']['m']->subcat->moveNext();
	    }
	    $ls .= "</ul>\n";
	    echo sprintf($s, $ls);
	}
    }
}

/**
 Display the query string.
 
 To be used only in the search template.

 @proto function pxSearchQuery
 @param string s Substitution string ('%s')
*/
function pxSearchQuery($s= '%s')
{
	global $_PX_website_config, $_PX_config;
	echo sprintf($s, htmlspecialchars($GLOBALS['_PX_render']['m']->getParam('q')));
}



/**
 @proto doc
 
 !!! Functions in loops
 
 It is possible to run functions within loops to get access to specific data from
 the elements in the loops. 
 
 !! The $res loop

 This loop is available in the categories. It is used to display the last 
 resources of a category. For example:

|<?php while (!$res->EOF()): ?>
|	<div class="resource">
|	<h2><a href="<?php pxResPath(); ?>"><?php pxResTitle(); ?></a></h2>
|	<?php pxResDescription(); ?>
|	</div>
|<?php 
|$res->moveNext(); 
|endwhile; ?>
 
*/ 


/**
 Display the title of the resource. 

 @proto function pxResTitle
 @param string s Substitution string ('%s')

*/
function pxResTitle($s='%s')
{
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
    $title = $GLOBALS['_PX_render']['m']->res->f('title');
    if ($remove_numbers) {
    	$title = px_removeNumbers($title);
	}
    echo sprintf($s, $title);
}

/**
 Display the path to the resource.

 @proto function pxResPath
 @param string type 'full' give path with http:// ('relative')
*/
function pxResPath($type = 'relative')
{
    global $_PX_website_config, $_PX_config;
    switch ($type) {
        case 'full' : $path = 'http://'.$_PX_website_config['domain'].$_PX_website_config['rel_url']; break;
        default : $path = $_PX_website_config['rel_url']; break;
    }    
    if ($_PX_config['url_format'] == 'simple') {
        $path .= '/?';
    }
    $path .= $GLOBALS['_PX_render']['m']->res->getPath();
    echo $path;
    
}

/**
 Display the description of the resource.

 If a limit is given, the description is converted into raw text and then the limit is applied.

 @proto function pxResDescription
 @param string s Substitution string ('%s')
 @param int limit Number of words (characters for the moment) to limit the description
*/
function pxResDescription($s='%s', $limit=0)
{
    if ($limit) {
    	$text = substr(textParseContent($GLOBALS['_PX_render']['m']->res->f('description'), 'text'), 0, $limit);
    	echo sprintf($s, $text);
	} else {
		echo textParseContent($GLOBALS['_PX_render']['m']->res->f('description'));
	}
}

/**
 Display the list of categories in which the ressource is.

 The list is not an HTML list, it is to be used as sentence like
 "Category one, category two and category tree" The category names
 are linked to the category pages.

 @proto function pxResCategories
 @param string s Substitution string ('%s')
 @param string p1 First delimiters (', ')
 @param string p2 Last delimiter (' and ')
*/
function pxResCategories($s='%s', $p1=', ', $p2=' and ')
{
    global $_PX_website_config, $_PX_config;
    $link_sep = ($_PX_config['url_format'] == 'simple') ? '/?' : '';
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');

	$cat = $GLOBALS['_PX_render']['m']->res->getCategories();
	$nr = $cat->nbRow();
	$i = 1;
	$link = '<a href="%s">%s</a>';
	$res = '';
	while (!$cat->EOF()) {
		$title = $cat->f('category_name');
		if ($remove_numbers) $title = px_removeNumbers($title);
		$res .= sprintf($link, $_PX_website_config['rel_url'].$link_sep.$cat->f('category_path'), $title);
		if ($nr >= 2 && ($i < ($nr - 1))) {
			$res .= $p1;
		}
		if ($nr >= 2 && ($i == ($nr - 1))) {
			$res .= $p2;
		}
		$i++;
		$cat->moveNext();
	}
	echo sprintf($s, $res);
}


/**
 Display the name of the author

 @proto function pxResAuthor
*/
function pxResAuthor()
{
	$GLOBALS['_PX_render']['m']->res->getAuthors();
	echo $GLOBALS['_PX_render']['m']->res->authors->f('user_realname');
}

/**
 Display the author public email if available.

@proto function pxResAuthorEmail
@param string s Substitution string ('%s')
@param string encoding Encoding for a mailto ('link') or for display 'text'
*/
function pxResAuthorEmail($s = '%s', $encoding = 'link')
{
	$text = ($encoding == 'link') ? false : true;
	$GLOBALS['_PX_render']['m']->res->getAuthors();

	if (strlen($GLOBALS['_PX_render']['m']->res->authors->f('user_pubemail')) > 0) {
		echo sprintf($s, textHexEncode($GLOBALS['_PX_render']['m']->res->authors->f('user_pubemail'), $text));
	}
}

/**
 Display the modification date of the resource. Only if newer than the publication date.

 @proto function pxResDateModification
 @param string dateformat Format of the date ('%Y-%d-%mT%T+00:00')
 @param string s Substitution string ('%s')
 @param mixed ifmodified Time in minutes between publication date and modification to display, false to always display it (false)
*/
function pxResDateModification($dateformat = '%Y-%m-%dT%H:%M:%S+00:00', $s = '%s', $ifmodified = false)
{
	if (false !== $ifmodified) {
		$ifmodified = $ifmodified * 60;
		$md = makeTime($GLOBALS['_PX_render']['m']->res->f('modifdate'));
		$pd = makeTime($GLOBALS['_PX_render']['m']->res->f('publicationdate'));
		if ($md >  ($pd + $ifmodified))
			echo sprintf($s, strftime($dateformat, $md));
	} else {
		echo sprintf($s, strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->res->f('modifdate'))));
	}
}

/**
 Display the publication date of the resource.

 @proto function pxResDatePublication
 @param string dateformat Format of the date ('%Y-%d-%mT%T+00:00')
 */
function pxResDatePublication($dateformat = '%Y-%m-%dT%H:%M:%S+00:00')
{
	echo strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->res->f('publicationdate')));
}

/**
 Display the score of the resource for this research.

 To be used only in the search template.

 @proto function pxResSearchScore
 @param string s Substitution string ('%01.2f%%')
*/
function pxResSearchScore($s= '%01.2f%%')
{
	global $_PX_website_config, $_PX_config;
	if (empty($GLOBALS['_PX_render']['resource']['search_max_occ'])) {
		$GLOBALS['_PX_render']['resource']['search_max_occ'] = (float) $GLOBALS['_PX_render']['m']->res->f('score');
	}
	$score = (float) $GLOBALS['_PX_render']['m']->res->f('score') / $GLOBALS['_PX_render']['resource']['search_max_occ'] * 100.0;
	echo sprintf($s, $score);
}
/**
 @proto doc

 !! The $last loop

 This loop is available in all the templates, you just have to initialize it
 before use:

|<?php pxGetLastResources(); ?>
|<?php while (!$last->EOF()): ?>
|    <p><a href="<?php pxLastResPath(); ?>"><?php pxLastResTitle(); ?></a></p>
|<?php
|$last->moveNext();
|endwhile; ?>

*/

/**
 Get the list of last resources and put them in $last
 Must be run before using the $last loop

 @proto function pxGetLastResources
 @param int limit Number of last resources (5)
 @param string type Type of resources ('') for all, 'news' or 'articles'
 @param int category Category id ('')
*/
function pxGetLastResources($limit = 5, $type = '', $category = '') {
	$GLOBALS['_PX_render']['m']->lres = $GLOBALS['_PX_render']['m']->getResources($category, $limit, $type);
}

/**
 Display the title of the resource.

 @proto function pxLastResTitle
 @param string s Substitution string ('%s')

*/
function pxLastResTitle($s='%s')
{
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$title = $GLOBALS['_PX_render']['m']->lres->f('title');
	if ($remove_numbers) $title = px_removeNumbers($title);
	echo sprintf($s, $title);
}

/**
 Display the path to the resource.

 @proto function pxLastResPath
 @param string type If 'full' give path with http:// ('relative')

*/
function pxLastResPath($type = 'relative')
{
	global $_PX_website_config, $_PX_config;
    $s = ($_PX_website_config['secure']) ? 's' : '';
	switch ($type) {
		case 'full' : $path = 'http'.$s.'://'.$_PX_website_config['domain'].$_PX_website_config['rel_url']; break;
		default : $path = $_PX_website_config['rel_url']; break;
	}
	if ($_PX_config['url_format'] == 'simple') {
		$path .= '/?';
	}
	$path .= $GLOBALS['_PX_render']['m']->lres->getPath();
	echo $path;
}

/**
 Display the description of the resource.

 @proto function pxLastResDescription

*/
function pxLastResDescription()
{
	echo textParseContent($GLOBALS['_PX_render']['m']->lres->f('description'));
}

/**
 Display the name of the author

 @proto function pxLastResAuthor
*/
function pxLastResAuthor()
{
	$GLOBALS['_PX_render']['m']->lres->getAuthors();
	echo $GLOBALS['_PX_render']['m']->lres->authors->f('user_realname');
}

/**
 Display the author public email if available.

@proto function pxLastResAuthorEmail
@param string s Substitution string ('%s')
@param string encoding Encoding for a mailto ('link') or for display 'text'
*/
function pxLastResAuthorEmail($s = '%s', $encoding = 'link')
{
	$text = ($encoding == 'link') ? false : true;
	$GLOBALS['_PX_render']['m']->lres->getAuthors();
	if (strlen($GLOBALS['_PX_render']['m']->lres->authors->f('user_pubemail')) > 0) {
		echo sprintf($s, textHexEncode($GLOBALS['_PX_render']['m']->lres->authors->f('user_pubemail'), $text));
	}
}

/**
 Display the modification date of the resource.

 @proto function pxLastResDateModification
 @param string dateformat Format of the date ('%Y-%d-%mT%T+00:00')
 @param string s Substitution string ('%s')
 @param mixed ifmodified Time in minutes between publication date and modification to display, false to always display it (false)
 */
function pxLastResDateModification($dateformat = '%Y-%m-%dT%H:%M:%S+00:00', $s = '%s', $ifmodified = false)
{
	if ($ifmodified !== false) {
		$pd = makeTime($GLOBALS['_PX_render']['m']->lres->f('publicationdate'));
		$md = makeTime($GLOBALS['_PX_render']['m']->lres->f('modifdate'));
		$ifmodified = 60 * (int) $ifmodified;
		if ($md > ($pd + $ifmodified)) {
			echo sprintf($s, strftime($dateformat, $md));
		}
	} else {
		echo sprintf($s, strftime($dateformat, makeTime($GLOBALS['_PX_render']['m']->lres->f('modifdate'))));
	}
}

/**
 Display the associated link and title if available. When getting the last
 resources, if the 'news' type is chose as link can be associated to
 news, this is the way to get it back if available.

 @proto function pxLastResAssociatedLink
 @param string s Substitution string ('<a href="%1$s">%2$s</a>')
*/
function pxLastResAssociatedLink($s = '<a href="%1$s">%2$s</a>')
{
	if (strlen($GLOBALS['_PX_render']['m']->lres->f('news_titlewebsite')) > 0) {
		echo sprintf($s, $GLOBALS['_PX_render']['m']->lres->f('news_linkwebsite'), $GLOBALS['_PX_render']['m']->lres->f('news_titlewebsite'));
	}
}

/**
 Display the list of categories in which the last ressource is.

 The list is not an HTML list, it is to be used as sentence like
 "Category one, category two and category tree" The category names
 are linked to the category pages.

 @proto function pxLastResCategories
 @param string s Substitution string ('%s')
 @param string p1 First delimiters (', ')
 @param string p2 Last delimiter (' and ')
*/
function pxLastResCategories($s='%s', $p1=', ', $p2=' and ')
{
    global $_PX_website_config, $_PX_config;
    $link_sep = ($_PX_config['url_format'] == 'simple') ? '/?' : '';
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$cat = $GLOBALS['_PX_render']['m']->lres->getCategories();
	$nr = $cat->nbRow();
	$i = 1;
	$link = '<a href="%s">%s</a>';
	$res = '';
	while (!$cat->EOF()) {
		$title = $cat->f('category_name');
		if ($remove_numbers) $title = px_removeNumbers($title);
		$res .= sprintf($link, $_PX_website_config['rel_url'].$link_sep.$cat->f('category_path'), $title);
		if ($nr >= 2 && ($i < ($nr - 1))) {
			$res .= $p1;
		}
		if ($nr >= 2 && ($i == ($nr - 1))) {
			$res .= $p2;
		}
		$i++;
		$cat->moveNext();
	}
	echo sprintf($s, $res);
}


/**
 @proto doc

 !!! The resource functions

 These are functions to display information about the current resource in the
 page. For the moment the resource can be either an __article__ or a __news__.

 !! The article functions

 These functions are to be used in the ''article'' templates.

*/


/**
 Display the title of the article.

 @proto function pxArtTitle
 @param string s Substitution string ('%s')

*/
function pxArtTitle($s='%s')
{
    $title = $GLOBALS['_PX_render']['m']->art->f('title');
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	if ($remove_numbers) $title = px_removeNumbers($title);
    echo sprintf($s, $title);
}

/**
 Display the description of the article.

 @proto function pxArtDescription
*/
function pxArtDescription()
{
    echo textParseContent($GLOBALS['_PX_render']['m']->art->f('description'));
}

/**
 Display the name of the author

 @proto function pxArtAuthor
*/
function pxArtAuthor()
{
    if (null === $GLOBALS['_PX_render']['m']->art->authors) {
        $GLOBALS['_PX_render']['m']->art->getAuthors();
    }
    echo $GLOBALS['_PX_render']['m']->art->authors->f('user_realname');
}

/**
 Display the path to the article.

 @proto function pxArtPath
 @param string type 'full' give path with http:// ('relative')
*/
function pxArtPath($type = 'relative')
{
    global $_PX_website_config, $_PX_config;
    $s = ($_PX_website_config['secure']) ? 's' : '';
    switch ($type) {
        case 'full' : $path = 'http'.$s.'://'.$_PX_website_config['domain'].$_PX_website_config['rel_url']; break;
        default : $path = $_PX_website_config['rel_url']; break;
    }
    if ($_PX_config['url_format'] == 'simple') {
        $path .= '/?';
    }
    $path .= $GLOBALS['_PX_render']['m']->art->getPath();
    echo $path;

}

/**
 Display the author public email if available.

@proto function pxArtAuthorEmail
@param string s Substitution string ('%s')
@param string encoding Encoding for a mailto ('link') or for display 'text'
*/
function pxArtAuthorEmail($s = '%s', $encoding = 'link')
{
    $text = ($encoding == 'link') ? false : true;
    if (null === $GLOBALS['_PX_render']['m']->art->authors) {
        $GLOBALS['_PX_render']['m']->art->getAuthors();
    }
    if (strlen($GLOBALS['_PX_render']['m']->art->authors->f('user_pubemail')) > 0) {
        echo sprintf($s, textHexEncode($GLOBALS['_PX_render']['m']->art->authors->f('user_pubemail'), $text));
    }
}

/**
 Display the creation date of the article.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxArtDateCreation
 @param string dateformat Format of the date ('%A %e %B %Y')

 */
function pxArtDateCreation($dateformat = '%A %e %B %Y')
{
    echo strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->art->f('creationdate')));
}

/**
 Display the publication date of the article.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxArtDatePublication
 @param string dateformat Format of the date ('%A %e %B %Y')

 */
function pxArtDatePublication($dateformat = '%A %e %B %Y')
{
	echo strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->art->f('publicationdate')));
}

/**
 Display the modification date of the article. Only if newer than the
 publication date.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxArtDateModification
 @param string dateformat Format of the date ('%A %e %B %Y - %T ')
 @param string s Substitution ('Modified the %s.')
 @param mixed ifmodified Time in minutes between publication date and modification to display, false to always display it (false)
 */
function pxArtDateModification($dateformat='%A %e %B %Y', $s='Modified the %s.', $ifmodified=false)
{
	if (false !== $ifmodified) {
		$ifmodified = 60 * $ifmodified;
		$md = makeTime($GLOBALS['_PX_render']['m']->art->f('modifdate'));
		$pb = makeTime($GLOBALS['_PX_render']['m']->art->f('publicationdate'));
		if ($md > ($pb + $ifmodified)) {
			echo sprintf($s, strftime($dateformat, $md));
		}
	} else {
		echo sprintf($s, strftime($dateformat, makeTime($GLOBALS['_PX_render']['m']->art->f('modifdate'))));
	}
}

/**
 Display the date of end of availaibility of the article. Only if end date.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxArtDateEnd
 @param string dateformat Format of the date ('%A %e %B %Y - %T ')
 @param string s Substitution ('End the %s.')
 */
function pxArtDateEnd($dateformat = '%A %e %B %Y', $s = 'End the %s.')
{
	$y = substr($GLOBALS['_PX_render']['m']->art->f('enddate'),0,4);
	if ((int)$y < 9999) {
		echo sprintf($s, strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->art->f('enddate'))));
	}
}

/**
 Display the keywords or subject of the article

@proto function pxArtKeywords
@param string s Substitution string ('%s')
*/
function pxArtKeywords($s = '%s')
{
	$keywords = trim($GLOBALS['_PX_render']['m']->art->f('subject'));
	if (strlen($keywords) > 0) {
		echo sprintf($s, $keywords);
	}
}

/**
 Display the list of categories in which the article is.

 The list is not an HTML list, it is to be used as sentence like
 "Category one, category two and category tree" The category names
 are linked to the category pages.

 @proto function pxArtCategories
 @param string s Substitution string ('%s')
 @param string p1 First delimiters (', ')
 @param string p2 Last delimiter (' and ')
*/
function pxArtCategories($s='%s', $p1=', ', $p2=' and ')
{
	$link_sep = ($GLOBALS['_PX_config']['url_format'] == 'simple') ? '/?' : '';
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$cat = $GLOBALS['_PX_render']['m']->art->getCategories();
	$nr = $cat->nbRow();
	$i = 1;
	$link = '<a href="%s">%s</a>';
	$res = '';
	while (!$cat->EOF()) {
		$title = $cat->f('category_name');
		if ($remove_numbers) $title = px_removeNumbers($title);
		$res .= sprintf($link, $GLOBALS['_PX_website_config']['rel_url'].$link_sep.$cat->f('category_path'), $title);
		if ($nr >= 2 && ($i < ($nr - 1))) {
			$res .= $p1;
		}
		if ($nr >= 2 && ($i == ($nr - 1))) {
			$res .= $p2;
		}
		$i++;
		$cat->moveNext();
	}
	echo sprintf($s, $res);
}


/**
 * Create a link for a wiki or another system
 *
 * @proto function pxWikiLink
 * @param string base Base link ('')
 *
 */
function pxArtWikiLink($base='')
{
    echo $base.$GLOBALS['_PX_render']['m']->art->f('path');
}


/**
 Return true if the current page is the first page of the article

 @proto function pxArtPageIsFirst
 @return bool __true__ if the current page is the first, else __false__
*/
function pxArtPageIsFirst()
{
	return ((int)$GLOBALS['_PX_render']['resource']['page'] == 1);
}

/**
 Display the title of the current article page.

 @proto function pxArtPageTitle
 @param string s Substitution string ('%s')

*/
function pxArtPageTitle($s = '%s')
{
	echo sprintf($s, $GLOBALS['_PX_render']['m']->art->currentpage['page_title']);
}

/**
 Display the content of the current article page.

 @proto function pxArtPageContent

*/
function pxArtPageContent()
{
	echo textParseContent($GLOBALS['_PX_render']['m']->art->currentpage['page_content']);
}

/**
 Display an ordered list of pages in the article with link to the pages.
 The current page is set as ''active'' with the corresponding <li> element
 being from the __current__ class.

 An output example is:

|<ol>
|<li><a href="/cat/my-article">Page 1</a></li>
|<li class="current"><a href="/cat/my-article2">Page 2</a></li>
|<li><a href="/cat/my-article3">Page 3</a></li>
|</ol>

 @proto function pxArtListPages
 @param string s Substitution string ('%s')

*/
function pxArtListPages($s = '%s')
{
	if ($GLOBALS['_PX_render']['m']->art->pages->nbRow() > 1) {
		$GLOBALS['_PX_render']['m']->art->pages->moveStart();
		$path = $GLOBALS['_PX_website_config']['rel_url'];
		if ($GLOBALS['_PX_config']['url_format'] == 'simple') {
			$path .= '/?';
		}
		$lp = '<ol>'."\n";
		while (!$GLOBALS['_PX_render']['m']->art->pages->EOF()) {
			$active = ($GLOBALS['_PX_render']['resource']['page'] == $GLOBALS['_PX_render']['m']->art->pages->f('page_number')) ? ' class="current"' : '';
			$page = ($GLOBALS['_PX_render']['m']->art->pages->f('page_number') == 1) ? '' : $GLOBALS['_PX_render']['m']->art->pages->f('page_number');
			$lp .= '<li'.$active.'><a href="'.$path.$GLOBALS['_PX_render']['m']->art->f('category_path').$GLOBALS['_PX_render']['m']->art->f('path').$page.'">'.$GLOBALS['_PX_render']['m']->art->pages->f('page_title').'</a></li>'."\n";
			$GLOBALS['_PX_render']['m']->art->pages->moveNext();
		}
		$lp .= '</ol>'."\n";
		echo sprintf($s, $lp);
	}
}

/**
 @proto doc

 !! The news functions

 These functions are to be used in the ''news'' templates.

*/

/**
 Display the title of a news.

 @proto function pxNewsTitle
 @param string s Substitution string ('%s')

*/
function pxNewsTitle($s='%s')
{
	echo sprintf($s, $GLOBALS['_PX_render']['m']->news->f('title'));
}

/**
 Display the content of a news.

 @proto function pxNewsContent
*/
function pxNewsContent()
{
	echo textParseContent($GLOBALS['_PX_render']['m']->news->f('description'));
}

/**
 Display the keywords or subject of the news.

@proto function pxNewsKeywords
@param string s Substitution string ('%s')
*/
function pxNewsKeywords($s = '%s')
{
	$keywords = trim($GLOBALS['_PX_render']['m']->news->f('subject'));
	if (strlen($keywords) > 0) {
		echo sprintf($s, $keywords);
	}
}

/**
 Display the creation date of the news.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxNewsDateCreation
 @param string dateformat Format of the date ('%A %e %B %Y')

 */
function pxNewsDateCreation($dateformat = '%A %e %B %Y')
{
	echo strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->news->f('creationdate')));
}

/**
 Display the publication date of the news.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxNewsDatePublication
 @param string dateformat Format of the date ('%A %e %B %Y')

 */
function pxNewsDatePublication($dateformat = '%A %e %B %Y')
{
	echo strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->news->f('publicationdate')));
}

/**
 Display the modification date of the news. Only if newer than the
 publication date.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxNewsDateModification
 @param string dateformat Format of the date ('%A %e %B %Y - %T ')
 @param string s Substitution ('Modified the %s.')
 @param mixed ifmodified Time in minutes between publication date and modification to display, false to always display it (false)
 */
function pxNewsDateModification($dateformat='%A %e %B %Y', $s='Modified the %s.', $ifmodified=false)
{
	if (false !== $ifmodified) {
		$ifmodified = $ifmodified * 60;
		$md = makeTime($GLOBALS['_PX_render']['m']->news->f('modifdate'));
		$pd = makeTime($GLOBALS['_PX_render']['m']->news->f('publicationdate'));
		if ($md > ($ifmodified + $pd)) {
			echo sprintf($s, strftime($dateformat, $md));
		}
	} else {
		echo sprintf($s, strftime($dateformat, makeTime($GLOBALS['_PX_render']['m']->news->f('modifdate'))));
	}
}

/**
 Display the date of end of availaibility of the news. Only if end date.

 The substitution string for the date is directly given to
 [strftime|http://www.php.net/strftime]

 @proto function pxNewsDateEnd
 @param string dateformat Format of the date ('%A %e %B %Y - %T ')
 @param string s Substitution ('End the %s.')
 */
function pxNewsDateEnd($dateformat = '%A %e %B %Y', $s = 'End the %s.')
{
	$y = substr($GLOBALS['_PX_render']['m']->news->f('enddate'),0,4);
	if ((int)$y < 9999) {
		echo sprintf($s, strftime($dateformat , makeTime($GLOBALS['_PX_render']['m']->news->f('enddate'))));
	}
}

/**
 Display the name of the author

 @proto function pxNewsAuthor
*/
function pxNewsAuthor()
{
	if (null === $GLOBALS['_PX_render']['m']->news->authors) {
		$GLOBALS['_PX_render']['m']->news->getAuthors();
	}
	echo $GLOBALS['_PX_render']['m']->news->authors->f('user_realname');
}

/**
 Display the author public email if available.

@proto function pxNewsAuthorEmail
@param string s Substitution string ('%s')
@param string encoding Encoding for a mailto ('link') or for display 'text'
*/
function pxNewsAuthorEmail($s = '%s', $encoding = 'link')
{
	$text = ($encoding == 'link') ? false : true;
	if (null === $GLOBALS['_PX_render']['m']->news->authors) {
		$GLOBALS['_PX_render']['m']->news->getAuthors();
	}
	if (strlen($GLOBALS['_PX_render']['m']->news->authors->f('user_pubemail')) > 0) {
		echo sprintf($s, textHexEncode($GLOBALS['_PX_render']['m']->news->authors->f('user_pubemail'), $text));
	}

}

/**
 Display the path to the news.

 @proto function pxNewsPath
 @param string type 'full' give path with http:// ('relative')
*/
function pxNewsPath($type = 'relative')
{
    global $_PX_website_config, $_PX_config;
    $s = ($_PX_website_config['secure']) ? 's' : '';
    switch ($type) {
        case 'full' : $path = 'http'.$s.'://'.$_PX_website_config['domain'].$_PX_website_config['rel_url']; break;
        default : $path = $_PX_website_config['rel_url']; break;
    }
    if ($_PX_config['url_format'] == 'simple') {
        $path .= '/?';
    }
    $path .= $GLOBALS['_PX_render']['m']->news->getPath();
    echo $path;

}

/**
 Display the list of categories in which the news is.

 The list is not an HTML list, it is to be used as sentence like
 "Category one, category two and category tree" The category names
 are linked to the category pages.

 @proto function pxNewsCategories
 @param string s Substitution string ('%s')
 @param string p1 First delimiters (', ')
 @param string p2 Last delimiter (' and ')
*/
function pxNewsCategories($s='%s', $p1=', ', $p2=' and ')
{
    global $_PX_website_config, $_PX_config;
    $link_sep = ($_PX_config['url_format'] == 'simple') ? '/?' : '';
	$remove_numbers = $GLOBALS['_PX_render']['m']->getParam('remove_numbers');
	$cat = $GLOBALS['_PX_render']['m']->news->getCategories();
	$nr = $cat->nbRow();
	$i = 1;
	$link = '<a href="%s">%s</a>';
	$res = '';
	while (!$cat->EOF()) {
		$title = $cat->f('category_name');
		if ($remove_numbers) $title = px_removeNumbers($title);
		$res .= sprintf($link, $_PX_website_config['rel_url'].$link_sep.$cat->f('category_path'), $title);
		if ($nr >= 2 && ($i < ($nr - 1))) {
			$res .= $p1;
		}
		if ($nr >= 2 && ($i == ($nr - 1))) {
			$res .= $p2;
		}
		$i++;
		$cat->moveNext();
	}
	echo sprintf($s, $res);
}


/**
 Get the resources for this category, with correct position with respect
 to the number of resources in one page

 @private
 */
function px_getResources($category='', $limit = 10, $type = '',  $order = 'DESC')
{
    global $_PX_website_config, $_PX_config;
    if (empty($GLOBALS['_PX_render']['m']->res)) {
        $GLOBALS['_PX_render']['m']->res = $GLOBALS['_PX_render']['m']->getResources($category, $limit, $type, $order);
    }
    $GLOBALS['_PX_render']['m']->res->move(($GLOBALS['_PX_render']['resource']['page'] - 1) * $GLOBALS['_PX_render']['m']->NbResPerPage);
    if ($GLOBALS['_PX_render']['m']->res->int_row_count > ($GLOBALS['_PX_render']['resource']['page'] * $GLOBALS['_PX_render']['m']->NbResPerPage)) {
        $GLOBALS['_PX_render']['m']->res->int_row_count = $GLOBALS['_PX_render']['resource']['page'] * $GLOBALS['_PX_render']['m']->NbResPerPage;
    }
}

/**
 Get the the current category

 @private
 */
function px_getCat()
{
    global $_PX_website_config, $_PX_config;
    if (empty($GLOBALS['_PX_render']['m']->cat)) {
        $GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
    }
}

/**
 Get the subcategories

  @private
*/
function px_getSubCat()
{
    global $_PX_website_config, $_PX_config;
    if (empty($GLOBALS['_PX_render']['m']->cat)) {
        $GLOBALS['_PX_render']['m']->cat = $GLOBALS['_PX_render']['m']->getCategory($GLOBALS['_PX_render']['resource']['category_id']);
    }
    if (empty($GLOBALS['_PX_render']['m']->subcat)) {
        $GLOBALS['_PX_render']['m']->subcat = $GLOBALS['_PX_render']['m']->getCategories($GLOBALS['_PX_render']['resource']['category_id']);
    }
}

/**
 Remove the numbers at the start of a string

 @param string string
 @return string
 @private
*/
function px_removeNumbers($string)
{
	return preg_replace('/^\s*\d+\.\s*/', '', $string);
}
?>
