<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

require_once dirname(__FILE__).'/class.resource.php';

define('PX_RESOURCE_MANAGER_ARTICLE','articles');

class Article extends Resource
{
	var $pages       = null;
	var $currentpage = null;

	function getPages()
	{
		$this->getConnection();
		$r = 'SELECT * FROM '.$this->con->pfx.'articles WHERE resource_id=\''.$this->f('resource_id').'\'
 		      ORDER BY page_number ASC';
		if (($rs = $this->con->select($r)) !== false) {
			$this->pages = $rs;
			return $rs;
		} else {
			$this->con->setError();
			return false;
		}
	}

	/**
	Set info for the article
	*/
	function set($title, $subject, $description, $format, $status, $path,
	             $datestart, $dateend, $useenddate, $subtype, $categoryid='', $website='', $user='')
	{
		$this->setField('title', $title);
		$this->setField('subject', $subject);
		$this->setField('description', '='.$format."\n".$description);
		$this->setField('status', $status);
		$this->setField('publicationdate', $datestart);
		$this->setField('path', $path);
		if (empty($useenddate)) $dateend = '99991231235959';
		$this->setField('enddate', $dateend);
		$this->setField('subtype_id', $subtype);

		if (!empty($categoryid)) $this->setField('website_id', $website);
		if (!empty($user)) $this->setField('user_id', $user);

		if (!empty($categoryid)) {
			$this->categories = new RecordSet();
			$this->categories->setField('category_id', $categoryid);
			$this->categories->setField('categoryasso_type', PX_RESOURCE_CATEGORY_MAIN);
		}
		return true;
	}

	/**
	Go to the page with a specific id
	
	@param int id of the page
	@return bool success to go to the page
	*/
	function goToPage($id)
	{
		$this->pages->moveStart();
    	$page_id = $this->pages->f('page_id');
    	$found = true;
		// get the requested page from the list of pages
    	while ($id != $page_id) {
        	if (false == $this->pages->moveNext()) {
            	$found = false; //went through all the pages without finding it
				break;
        	} else {
            	$page_id = $this->pages->f('page_id');
			}
		}
		return $found;
    }

	/**
	Set info for the page
	*/
	function setPage($id, $title, $content, $format, $number, $index='')
	{
		$oldnumber = '';
		if ($this->pages === null || $this->pages === false) {
			$this->pages = new recordset();
		}
		if (empty($id)) {
			// new page
			$this->pages->int_index = $this->pages->nbRow();
			$this->pages->int_row_count++;
		} else {
		    // modification of a page, we move to it
			if (false === $this->goToPage($id))
				return false;
			$oldnumber = $this->pages->f('page_number');
		}

		$this->pages->setField('old_page_number', $oldnumber);
		$this->pages->setField('page_number', $number);
		$this->pages->setField('page_title', $title);
		$this->pages->setField('page_content', '='.$format."\n".$content);
		$this->pages->setField('page_id', $id);
		return true;
}

	/**
	Return the content of the news as a string ready for indexation.

	@param string Format of the string (html, wiki, text)
	@return string The content of the news as a string
	*/
	function getAsString($format = 'html')
	{
		if (null === $this->pages) {
			$this->getPages();
		}
		$this->pages->moveStart();

		$string = str_repeat($this->f('title').' ', 5);
		$string .= ' '.str_repeat($this->f('subject').' ', 3);
		$string .= ' '.str_repeat($this->f('path').' ', 3);
		$string .= ' '.$this->f('description');
		while (!$this->pages->EOF()) {
			$string .= ' '.str_repeat($this->pages->f('page_title').' ', 3);
			$string .= ' '.$this->pages->f('page_content');
			$this->pages->moveNext();
		}
		return $string;
	}


}



?>
