<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****


/**
@class checklist

Class to run series of tests/checks.
A result of a test can be good, bad or warning.

*/

class checklist
{
	var $tests = array();
	/**
	A new checklist is created with a test.

	@see addtest
	*/
	function checklist($id='', $res=0, $good='', $bad='', $warning='')
	{
		$this->addTest($id, $res, $good, $bad, $warning);
	}

	/**
	Add a new test

	@param string id of the test
	@param int result of the test (0=bad, 1=good, 2=warning)
	@param string String in case of good result
	@param sring String in case of bad result
	@param string String in case of warning
	@return bool valid or invalid test
	*/
	function addTest($id='', $res=0, $good='', $bad='', $warning='')
	{
		if (empty($id)) {
			return false;
		}
		$string = $good;
		if (2 === $res) {
			$string = $warning;
		} elseif (false === $res or 0 === $res) {
			$string = $bad;
			$res = 0;
		} else {
			$res = 1;
		}
		$this->tests[] = array(
			'id' => $id,
			'res' => $res,
			'mess' => $string
		);
		return true;
	}
	
	/** Compatibility with the dotclear checklist */
	function addItem($id='', $res=0, $good='', $bad='') 
	{
		if (NULL === $res) $res = 1;
		return $this->addTest($id, $res, $good, $bad);
	}

	/**
	Check the tests, 2 levels all good or good and warnings.

	@param int Level 1=all good, 2=warnings ok (default)
	@return bool global result of the tests
	*/
	function checkAll($level=2)
	{
		foreach ($this->tests as $test) {
			if (0 === $test['res']) return false;
			if (1 === $level && 2 === $test['res']) return false;
		}
		return true;
	}

	/**
	Get the an html list with the results of the tests.

	@param string Path to the images (no trailing slash)
	@param string Good image ('check_on.png')
	@param string Bad image ('check_off.png')
	@param string Warning image ('check_wait.png')
	@return string HTML code
	*/
	function getHtml($path, $gi='check_on.png', $bi='check_off.png', $wi='check_wait.png')
	{
		reset($this->tests);
		$o = '<ul class="checklist">'."\n";
		foreach ($this->tests as $test) {
			$img = $path.'/'.$gi;
			$string = $test['mess'];
			if (2 === $test['res']) {
				$img = $path.'/'.$wi;
			} elseif (false == $test['res']) {
				$img = $path.'/'.$bi;
			}
			$o .= '<li><img src="'.$img.'" /> '.$string.'</li>'."\n";
		}
		$o .= '</ul>'."\n\n";
		return $o;
	}
}



?>
