<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

// The l10n has been rewritten with a "free" inspiration from the work
// by Olivier Meunier, I especially discover the ability to create a
// function called "__" :o) Big thanks to him.

// 2 letter ISO codes from http://www.oasis-open.org/cover/iso639a.html
// The list of languages supported by ISO-8859-1 is coming from Wikipedia

function __($str)
{
	$t = trim($str);
	if (isset($GLOBALS['_PX_locale'][$t])) {
		return $GLOBALS['_PX_locale'][$t];
	} elseif (!empty($GLOBALS['_PX_locale_lang']) && 'en' != $GLOBALS['_PX_locale_lang'] && !empty($GLOBALS['_PX_config']['debug'])) {
		$GLOBALS['_PX_debug_data']['untranslated'][] = $t;
	}
	return $t;
}

class l10n
{

	/** constructor
	@param string lang
	@param string domain (the lang file in the main locale folder)
	*/
	function l10n($lang = 'en', $domain = 'plume')
	{
		$this->loadDomain($lang, $domain);
	}
	
	function loadDomain($lang = 'en', $domain = 'plume')
	{
		global $_PX_config;
		$GLOBALS['_PX_locale_lang'] = $lang;
		if ('en' == $lang) return true;
		return $this->loadFile( $_PX_config['manager_path'].'/locale/'.$lang.'/'.$domain.'.lang');
	}

	/**
	Load the locale of a plugin in the translation array
	*/
	function loadPlugin($lang = 'en', $plugin)
	{
		global $_PX_config;
		if ('en' == $lang) return true;
		return $this->loadFile( $_PX_config['manager_path'].'/tools/'.$plugin.'/locale/'.$lang.'/'.$plugin.'.lang');
	}

	/**
	Load a locale file

	@param file the complete path to the locale file
	*/
	function loadFile($file, $getencodingonly = false)
	{
		global $_PX_config;
		if (empty($_PX_config['encoding'])) $_PX_config['encoding'] = 'utf-8'; //safe default value

		if (!empty( $GLOBALS['_PX_locale_files'][$file])) {
			return true;
		}
		if (!file_exists($file)) {
			return false;
		}
		$phpfile = substr($file, 0, -5).'.php';
		if (file_exists($phpfile) && (@filemtime($file) < @filemtime($phpfile))) {
			include $phpfile;
			$GLOBALS['_PX_locale_files'][$file] = 'optimized';
			return true;
		}

		$lines = file($file);
		// the first line is the encoding of the file a la Python
		// # -*- coding: <encoding name> -*-
		$encoding = '';
		$convert = false;
		if (preg_match('/^#\s-\*-\scoding:\s(.*)\s-\*-/', $lines[0], $match)) {
			$encoding = strtolower($match[1]);
			if ($getencodingonly) return $encoding;
			if ('iso-8859-1' == $encoding && 'utf-8' == strtolower($_PX_config['encoding']))
				$convert = true;
		}

		$count = count($lines);
		for ($i=1; $i<$count; $i++) {
			$tmp = (!empty($lines[$i+1])) ? trim($lines[$i+1]) : '';
			if (!empty($tmp) && ';' == substr($lines[$i],0,1)) {
				if ($convert) $tmp = utf8_encode($tmp);
				$GLOBALS['_PX_locale'][trim(substr($lines[$i],1))] = $tmp;
				$i++;
			}
		}
		$GLOBALS['_PX_locale_files'][$file] = (!empty($encoding)) ? $encoding : true;
		return true;
	}
	
	/**
	Optimize a locale. Convert the .lang in a .php file ready to be included.
	The optimized file is encoded with the current encoding.
	
	@param string locale file to optimize
	@return bool optimization ok
	*/
	function optimizeLocale($file)
	{
		global $_PX_config;
		if (empty($_PX_config['encoding'])) $_PX_config['encoding'] = 'utf-8'; //safe default value

		if (!file_exists($file)) {
			return false;
		}
		$phpfile = substr($file, 0, -5).'.php';

		$lines = file($file);
		// the first line is the encoding of the file a la Python
		// # -*- coding: <encoding name> -*-
		$encoding = '';
		$convert = false;
		if (preg_match('/^#\s-\*-\scoding:\s(.*)\s-\*-/', $lines[0], $match)) {
			$encoding = strtolower($match[1]);
			if ('iso-8859-1' == $encoding && 'utf-8' == strtolower($_PX_config['encoding']))
				$convert = true;
		}

		if (false === ($fp = @fopen($phpfile,'w'))) return false;
		fputs($fp, '<?php '."\n".'/* automatically generated file from: '.$file.'  */'."\n\n");
		$count = count($lines);
		for ($i=1; $i<$count; $i++) {
			$tmp = (!empty($lines[$i+1])) ? trim($lines[$i+1]) : '';
			if (!empty($tmp) && ';' == substr($lines[$i],0,1)) {
				if ($convert) $tmp = utf8_encode($tmp);
				$string = '$GLOBALS[\'_PX_locale\'][\''.str_replace("'", "\\'", trim(substr($lines[$i],1))).'\'] = \''.str_replace("'", "\\'", $tmp).'\';'."\n";
				fputs($fp, $string);
				$i++;
			}
		}
		fputs($fp, "\n".'?>');
		@fclose($fp);
		return true;
	}

	function getAvailableLocales($plugin = '', $domain='')
	{
		global $_PX_config;
		if ('' == $plugin) {
			$rootdir = $_PX_config['manager_path'].'/locale';
		} else {
			$rootdir = $_PX_config['manager_path'].'/tools/'.$plugin.'/locale';
		}
		$locales = array();
		$locales[] = 'en'; //English is always available
		$current_dir = opendir($rootdir);
		if (!empty($domain)) $domain .= '.lang';
		while($entryname = readdir($current_dir)) {
			if (is_dir($rootdir.'/'.$entryname.'/') and ($entryname != '.' and $entryname!='..') and (2 == strlen($entryname))) {
				$entryname = strtolower($entryname);
				if (empty($domain))
					$locales[] = $entryname;
				elseif (is_file($rootdir.'/'.$entryname.'/'.$domain))
					$locales[] = $entryname;
			}
		}
		closedir($current_dir);
		sort($locales);
		reset($locales);
		return $locales;
	}

	/**
	Return the "best" accepted language from the list of available 
	languages.

	Note: if $accepted is empty and $_SERVER['HTTP_ACCEPT_LANGUAGE'] returns 'en'

	@param array Available languages in the system
	@param string String of accepted languages, use $_SERVER['HTTP_ACCEPT_LANGUAGE'] if empty
	@return string Language (2 letter lower case iso code) for the user
	*/
	function getAcceptedLanguage($available, $accepted ='')
	{
		if (empty($accepted)) {
			if (!empty($_SERVER['HTTP_ACCEPT_LANGUAGE'])) $accepted = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
			else return 'en';
		}
		$acceptedlist = explode(',', $accepted);
		foreach ($acceptedlist as $lang) {
			$lang = strtolower(substr($lang, 0, 2)); //for the fr-FR en-US cases
			if (in_array($lang, $available)) {
				return $lang;
			}
		}
		//no match found, English
		return 'en';
	}

	function getIsoCodes($lang = false)
	{
		$res = array('aa' => 'Afar',
				'ab' => 'Abkhazian',
				'af' => 'Afrikaans',
				'am' => 'Amharic',
				'ar' => 'Arabic',
				'as' => 'Assamese',
				'ay' => 'Aymara',
				'az' => 'Azerbaijani',
				'ba' => 'Bashkir',
				'be' => 'Byelorussian',
				'bg' => 'Bulgarian',
				'bh' => 'Bihari',
				'bi' => 'Bislama',
				'bn' => 'Bengali',
				'bo' => 'Tibetan',
				'br' => 'Breton',
				'ca' => 'Catalan',
				'co' => 'Corsican',
				'cs' => 'Czech',
				'cy' => 'Welsh',
				'da' => 'Danish',
				'de' => 'German',
				'dz' => 'Bhutani',
				'el' => 'Greek',
				'en' => 'English',
				'eo' => 'Esperanto',
				'es' => 'Spanish',
				'et' => 'Estonian',
				'eu' => 'Basque',
				'fa' => 'Persian',
				'fi' => 'Finnish',
				'fj' => 'Fiji',
				'fo' => 'Faroese',
				'fr' => 'French',
				'fy' => 'Frisian',
				'ga' => 'Irish',
				'gd' => 'Scots gaelic',
				'gl' => 'Galician',
				'gn' => 'Guarani',
				'gu' => 'Gujarati',
				'ha' => 'Hausa',
				'he' => 'Hebrew',
				'hi' => 'Hindi',
				'hr' => 'Croatian',
				'hu' => 'Hungarian',
				'hy' => 'Armenian',
				'ia' => 'Interlingua',
				'ie' => 'Interlingue',
				'ik' => 'Inupiak',
				'id' => 'Indonesian',
				'is' => 'Icelandic',
				'it' => 'Italian',
				'iu' => 'Inuktitut',
				'ja' => 'Japanese',
				'jv' => 'Javanese',
				'ka' => 'Georgian',
				'kk' => 'Kazakh',
				'kl' => 'Greenlandic',
				'km' => 'Cambodian',
				'kn' => 'Kannada',
				'ko' => 'Korean',
				'ks' => 'Kashmiri',
				'ku' => 'Kurdish',
				'ky' => 'Kirghiz',
				'la' => 'Latin',
				'ln' => 'Lingala',
				'lo' => 'Laothian',
				'lt' => 'Lithuanian',
				'lv' => 'Latvian;lettish',
				'mg' => 'Malagasy',
				'mi' => 'Maori',
				'mk' => 'Macedonian',
				'ml' => 'Malayalam',
				'mn' => 'Mongolian',
				'mo' => 'Moldavian',
				'mr' => 'Marathi',
				'ms' => 'Malay',
				'mt' => 'Maltese',
				'my' => 'Burmese',
				'na' => 'Nauru',
				'ne' => 'Nepali',
				'nl' => 'Dutch',
				'no' => 'Norwegian',
				'oc' => 'Occitan',
				'om' => 'Afan (oromo)',
				'or' => 'Oriya',
				'pa' => 'Punjabi',
				'pl' => 'Polish',
				'ps' => 'Pashto;pushto',
				'pt' => 'Portuguese',
				'qu' => 'Quechua',
				'rm' => 'Rhaeto-romance',
				'rn' => 'Kurundi',
				'ro' => 'Romanian',
				'ru' => 'Russian',
				'rw' => 'Kinyarwanda',
				'sa' => 'Sanskrit',
				'sd' => 'Sindhi',
				'sg' => 'Sangho',
				'sh' => 'Serbo-croatian',
				'si' => 'Singhalese',
				'sk' => 'Slovak',
				'sl' => 'Slovenian',
				'sm' => 'Samoan',
				'sn' => 'Shona',
				'so' => 'Somali',
				'sq' => 'Albanian',
				'sr' => 'Serbian',
				'ss' => 'Siswati',
				'st' => 'Sesotho',
				'su' => 'Sundanese',
				'sv' => 'Swedish',
				'sw' => 'Swahili',
				'ta' => 'Tamil',
				'te' => 'Telugu',
				'tg' => 'Tajik',
				'th' => 'Thai',
				'ti' => 'Tigrinya',
				'tk' => 'Turkmen',
				'tl' => 'Tagalog',
				'tn' => 'Setswana',
				'to' => 'Tonga',
				'tr' => 'Turkish',
				'ts' => 'Tsonga',
				'tt' => 'Tatar',
				'tw' => 'Twi',
				'ug' => 'Uigur',
				'uk' => 'Ukrainian',
				'ur' => 'Urdu',
				'uz' => 'Uzbek',
				'vi' => 'Vietnamese',
				'vo' => 'Volapuk',
				'wo' => 'Wolof',
				'xh' => 'Xhosa',
				'yi' => 'Yiddish',
				'yo' => 'Yoruba',
				'za' => 'Zhuang',
				'zh' => 'Chinese',
				'zu' => 'Zulu');
		if ($lang) {
			$res = array_flip($res);
			ksort($res); //order by lang
		}
		return $res;
	}

	function getIsoWestern()
	{
		$res = array('af', 'sq', 'eu', 'ca', 'da', 'nl', 'en', 'fo', 'fi', 'fr',
		     'de', 'is', 'ga', 'it', 'no', 'pt', 'rm', 'gd',
		     'es', 'sv', 'sw');
		return $res;
	}

}


class CError
{
	var $error = array();


//------------------------------------------------------------------------------
// Error functions resetError(), setError() and error() from Olivier Meunier
// See CREDITS for details
//------------------------------------------------------------------------------
	function resetError()
	{
		$this->error = array();
	}

	function setError($msg,$no=0)
	{
		$this->error[] = array($no,$msg);
	}

	function error($html=false,$with_nb=true)
	{
		if (count($this->error) > 0) {
			if (!$html) {
				return $this->error;
			} else {
				$res = '<ul>'."\n";
				foreach($this->error as $v) {
					$res .= '<li>'.
					( ($with_nb) ? '<span class="errno">'.$v[0].'</span> - ' : '').
					'<span class="errmsg">'.$v[1].'</span></li>'."\n";
				}
				return $res."</ul>\n";
			}
		} else {
			return false;
		}
	}

	function bulkSetError($errors)
	{
		if (!is_array($errors)) return false;
		foreach ($errors as $e) {
			$this->setError($e[1], $e[0]);
		}
		return true;
	}

} // end class CError

?>
