<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

require_once dirname(__FILE__).'/class.l10n.php';
require_once dirname(__FILE__).'/class.resource.php';
require_once dirname(__FILE__).'/class.basicmanager.php';
require_once dirname(__FILE__).'/class.user.php';
require_once dirname(__FILE__).'/class.article.php';
require_once dirname(__FILE__).'/class.news.php';

class Manager extends BasicManager
{

	/* Functions used in the manager to populate data for the 
	 * forms etc...
	 * ------------------------------------------------------------ */

	/**
	If $addallcat set to true a special "All the categories" is also
	given. Used for the listing of the resources in the manager.
	*/
	function getArrayCategories($addallcat=false)
	{
		$cats = $this->getCategories();
		$arry_cat = array();
		if ($addallcat)
			$arry_cat[ __('All the categories')]='allcat';
		while (!$cats->EOF()) {
			$name  = $cats->f('category_name');
			$name .= ' ('.$cats->f('category_path').')';
			if (isGhostCat($cats->f('category_path'))) $name .= ' ['. __('Hidden category').']';
			$arry_cat[$name] = $cats->f('category_id');
			$cats->moveNext();
		}
		return $arry_cat;
	}
	
	/**
	@param string type of resource
	@param int category id
	@param bool (true) add the all dates choice (true by default!!!)
	@return array($first, $last, array of months + year)
	*/
	function getArrayMonths($type='', $cat_id='', $addalldates=true)
	{
		$arry_months = array();
		if ($addalldates)
			$arry_months[__('All the dates')] = 'alldate';
		$last = '';
		$k = '';
		//getAllDates returns the values in time DESC order
		foreach ($this->getAllDates('m', $type, $cat_id) as $k => $v) {
			if (empty($last)) $last = $k;
			$arry_months[if_utf8(strftime('%B %Y',makeTime($k)))] = $k;
		}
		return array($k, $last, $arry_months);
	}


	/* Management of the resources
	 * ------------------------------------------------------------ */

    /** Add a resource in a category

    @return bool
    @param  int resource id
    @param  int category id
    @param  int type of category (main or normal)
    */
    function addResourceInCategory($resid, $catid, $type = PX_RESOURCE_CATEGORY_OTHER)
    {
        if (false === ($res = $this->getResourceByIdentifier($resid))) return false;
        if (!preg_match('/^\d+$/', $catid)) {
            $this->setError( __('The proposed category is invalid.'), 400); 
            return false;
        }            
        if (false === ($cats = $res->getCategories())) {
            $this->setError($res->con->error, $res->con->errno);
            return false;
        }
        
        $update = false;
        $ids = $cats->getIDs('', 'category_id');
        if (in_array($catid, $ids)) $update = true;

       
        if ($type == PX_RESOURCE_CATEGORY_MAIN) {
            $insReq = 'UPDATE '.$this->con->pfx.'categoryasso SET
                 categoryasso_type=\''.PX_RESOURCE_CATEGORY_OTHER.'\'
                 WHERE identifier=\''.$this->con->escapeStr($res->f('identifier')).'\'';
            if (!$this->con->execute($insReq)) {
                $this->setError('MySQL : '.$this->con->error(), 500);
                return false;
            }
        } elseif ($update && $cats->f('category_id') == $catid) {
            // the first category in $cats is the main category
            // try to set the cat as not main, but it is currently the main... skip
            return true;
        }

        if ($update) {
            $insReq = 'UPDATE '.$this->con->pfx.'categoryasso SET
                 categoryasso_type=\''.$this->con->escapeStr($type).'\'
                 WHERE category_id=\''.$this->con->escapeStr($catid).'\'
                 AND identifier=\''.$this->con->escapeStr($res->f('identifier')).'\'';
        } else {
            $insReq = 'INSERT INTO '.$this->con->pfx.'categoryasso SET
                 category_id=\''.$this->con->escapeStr($catid).'\',
                 identifier=\''.$this->con->escapeStr($res->f('identifier')).'\',
                 categoryasso_type=\''.$this->con->escapeStr($type).'\'';
        }
        
        if (!$this->con->execute($insReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
        $this->triggerMassUpdate();
        return true;
    }
    
    /** Remove a resource from a category
    
    @return bool
    @param  int resource id
    @param  int category id 
    */
    function removeResourceFromCategory($resid, $catid)
    {
        if (false === ($res = $this->getResourceByIdentifier($resid))) return false;
        if (!preg_match('/^\d+$/', $catid)) {
            $this->setError( __('The proposed category is invalid.'), 400); 
            return false;
        }            
        if (false === ($cats = $res->getCategories())) {
            $this->setError($res->con->error, $res->con->errno);
            return false;
        }

        $ids = $cats->getIDs('', 'category_id');
        if (!in_array($catid, $ids)) return false;
        
        if ($cats->f('category_id') == $catid) {
            $this->setError('resource_error_remove_from_main_category', 400);
            return false;
        }
        $insReq = 'DELETE FROM '.$this->con->pfx.'categoryasso 
                WHERE category_id=\''.$this->con->escapeStr($catid).'\'
                AND identifier=\''.$this->con->escapeStr($res->f('identifier')).'\'';
        if (!$this->con->execute($insReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }        
        $this->triggerMassUpdate();
        return true;    
    }
    
    /**
    Trigger mass update for a given website. If now website
    use the currently managed by the user
    */
    function triggerMassUpdate($website = '')
    {
    	if (empty($website)) $website = $this->user->website;
		$mufile = dirname(__FILE__).'/../cache/'.$website.'/MASS_UPDATE';
		return @touch($mufile, time());
	}


    /**
    Get the list of subtypes, ordered by type
    @return object RecordSet
    @param int Optional, limit the search to one subtype
    @param string Optional, limit the search to a type of resource
    */
    function getSubTypes($id = '', $type='')
    {
        $r = 'SELECT * FROM '.$this->con->pfx.'subtypes WHERE website_id=\''.$this->user->website.'\'';
        
        if (!empty($id)) $r .= ' AND subtype_id=\''.$this->con->escapeStr($id).'\'';
        if (!empty($type)) $r .= ' AND type_id=\''.$this->con->escapeStr($type).'\'';
        
        $r .= ' ORDER BY type_id'; 
        if (($rs = $this->con->select($r)) !== false) {
            return $rs;
        } else {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }        
    }

	/**
	Index a resource.

	@param int Resource id
	@param object Resource object
	@return bool Success or not
	*/
	function indexResource($id, &$res)
	{
		include_once dirname(__FILE__).'/class.search.php';
		$s = new Search($this->con, $this->user->website);
		$string = $res->getAsString();
		$s->index($string , $id);
		if (false !== $s->error()) {
			return false;
		}
		return true;
	}
	
	/**
	Remove a resource from the index.
	
	@param int Resource id
	@return bool Success or not
	*/
	function indexRemove($id)
	{
		include_once dirname(__FILE__).'/class.search.php';
		$s = new Search($this->con, $this->user->website);
		$s->remove_from_index($id);
		if (false !== $s->error()) {
			return false;
		} 
		return true;
	}


    
    /**
    Check if a subtype is used
    @return bool True if in use
    @param int Subtype id
    */
    function isSubTypeUsed($id)
    {
        $req = 'SELECT COUNT(*) AS total FROM '.$this->con->pfx.'resources WHERE subtype_id=\''.$this->con->escapeStr($id).'\'';
        if (($rs = $this->con->select($req)) === false) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return true; //by security set as used
        }
        if (0 == (int) $rs->f('total')) {
            return false;
        }                                
        return true;        
    }
    
    /**
    Delete a subtype
    @return bool Success or not
    @param int Subtype id
    */
    function deleteType($id)
    {
        if ($this->isSubTypeUsed($id)) {
            $this->setError( __('Impossible to delete this type as it is in use.'), 400);            
            return false;
        }
        $delReq = 'DELETE FROM '.$this->con->pfx.'subtypes WHERE subtype_id=\''.$this->con->escapeStr($id).'\'';  
        if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        } 

        return true;
    }

    /**
    Save/add the type
    
    @return int Id of the subtype (false if error)
    @param int Id of the subtype (empty if adding a new)
    @param string Type id 'news' or 'articles'
    @param string Name of the subtype
    @param string Template file
    @param int Time for the cache
    @param string Extra information 1
    @param string Extra information 2
    @param string Website id (''), use the current website id from the user object if empty
    */
    function saveType($id, $type_id, $name, $template, $cachetime, $extra1, $extra2, $website='')
    {
        if (empty($name)) {
            $this->setError( __('You must provide a name.'), 400);
        }
        if (empty($template)) {
            $this->setError( __('You must provide a template.'), 400);
        }
        if (!preg_match('/^[-]{0,1}[0-9]+$/', $cachetime)) {
            $this->setError( __('The cachetime must be an integer.'), 400);
        }
        // if errors we go out.
        if (false !== $this->error()) {
            return false;
        }

        $new = false;
        if (empty($id)) {
            // get a new id
            $new = true;
            $req = 'INSERT INTO ';
        } else {
            $req = 'UPDATE ';
        }
		if (empty($website)) $website = $this->user->website;

		//            subtype_id=\''.$this->con->escapeStr($id).'\',
        $req .= $this->con->pfx.'subtypes SET
            type_id=\''.$this->con->escapeStr($type_id).'\',
            website_id=\''.$this->con->escapeStr($website).'\',
            subtype_name=\''.$this->con->escapeStr($name).'\',
            subtype_template=\''.$this->con->escapeStr($template).'\',
            subtype_cachetime=\''.$this->con->escapeStr($cachetime).'\',
            subtype_extra1=\''.$this->con->escapeStr($extra1).'\',
            subtype_extra2=\''.$this->con->escapeStr($extra2).'\'';

        if (!$new) {
            $req .= ' WHERE subtype_id=\''.$this->con->escapeStr($id).'\'';
        }

        if (!$this->con->execute($req)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
        if (empty($id)) $id = $this->con->getLastID();
        $this->triggerMassUpdate();

        return $id;
    }


    /** Save a category
    */
    function saveCategory($id, $parentid, $name, $description, $description_format, $keywords, $path, $template, $cachetime)
    {
        $cat = false;
        if (!empty($id) && false === ($cat = $this->getCategory($id))) {
            $this->setError( __('Error: Category not available for modification.'), 400);
            return false;
        }

        // in case of force some values not to be changed
        if ($cat !== false) {
            $id       = $cat->f('category_id');
            $parentid = ($cat->f('category_path') == '/') ? $cat->f('category_parentid') : $parentid;
            $path     = ($cat->f('category_path') == '/') ? $cat->f('category_path')     : trim($path);
        }

        // do the sanity check
        // the path
        if (   (strlen($path) == 0) || 
               (strlen($path) == 1 && $path != '/') ||
               (strlen($path) > 1 && !preg_match('/^\/[A-Za-z0-9\_\-\/]+\/$/',$path)) ) { 
            $this->setError( __('Error: Invalid path.'), 400);   
            return false;
        }
        $r = 'SELECT * FROM '.$this->con->pfx.'categories WHERE website_id=\''.$this->user->website.'\'
              AND category_path=\''.$this->con->escapeStr($path).'\' LIMIT 1'; 
        if (($rs = $this->con->select($r)) !== false) {
            if (   ($rs->nbRow() == 1) &&
                   (empty($id) || ($rs->f('category_id') != $id)) ) {
                $this->setError( __('Error: Path already used by another category.'), 400);
                return false;
            }
        } else {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }        
        // the template
        if (strlen($template) < 4) {
            $this->setError( __('Error: Invalid template.'), 400);   
            return false;            
        }
        
        //  need more sanity check
        
        if ($cat === false) {
            // insertion of a new category
            $insReq = 'INSERT INTO '.$this->con->pfx.'categories SET
                category_parentid        = \''.$this->con->escapeStr($parentid).'\',
                website_id               = \''.$this->con->escapeStr($this->user->website).'\',
                category_name            = \''.$this->con->escapeStr($name).'\',
                category_description     = \''.$this->con->escapeStr('='.trim($description_format)."\n".$description).'\',  
                category_keywords        = \''.$this->con->escapeStr($keywords).'\',
                category_path            = \''.$this->con->escapeStr($path).'\',
                category_publicationdate = \''.timestamp().'\',  	  
                category_creationdate    = \''.timestamp().'\',  
                category_enddate         = \'99991231235959\',
                category_template        = \''.$this->con->escapeStr($template).'\',
                category_type            = \'default\',
                category_cachetime       = \''.$this->con->escapeStr($cachetime).'\',
                image_id                 = \'0\',
                icon_id                  = \'0\',
                forum_id                 = \'0\'';
        } else {
            // update of the category
            $insReq = 'UPDATE '.$this->con->pfx.'categories SET
                category_parentid        = \''.$this->con->escapeStr($parentid).'\',
                category_name            = \''.$this->con->escapeStr($name).'\',
                category_description     = \''.$this->con->escapeStr('='.trim($description_format)."\n".$description).'\',  
                category_keywords        = \''.$this->con->escapeStr($keywords).'\',
                category_path            = \''.$this->con->escapeStr($path).'\',
                category_template        = \''.$this->con->escapeStr($template).'\',
                category_cachetime       = \''.$this->con->escapeStr($cachetime).'\'
                WHERE category_id = \''.$this->con->escapeStr($id).'\'';                   
        }
        if (!$this->con->execute($insReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
        $this->triggerMassUpdate();
        if ($cat === false) $id = $this->con->getLastID();
        return $id;            
    }
    
    /** Del category. 
    
    only if no resources in, and not a root category with a '/' path
    */
    function delCategory($id)
    {
        if (!empty($id) && false === ($cat = $this->getCategory($id))) {
            $this->setError( __('Error: Category not available for modification.'), 400);
            return false;
        }
        // see if resources in the category
        if (false === ($res = $this->getResources('', '', $cat->f('category_id')))) return false;
        if ($res->nbRow() > 0) {
            $this->setError( __('Error: Category not empty.'), 400);
            return false;
        }
        
        $delReq = 'DELETE FROM '.$this->con->pfx.'categories WHERE category_id = \''.$this->con->escapeStr($id).'\'';   
        if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }        

        // not necessary but can remove some orphan entries
        $delReq = 'DELETE FROM '.$this->con->pfx.'categoryasso WHERE category_id = \''.$this->con->escapeStr($id).'\'';   
        if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        } 
        $this->triggerMassUpdate();
        return $id;                             
    }  
	

	function saveUser($id, $username, $password, $realname, $email, $pubemail, $authwebs = null)
	{
		if ($id == 1 && $this->user->_userid != 1) {
			$this->setError( __('Error: You do not have the rights to modify this user.'), 400);
			return false;
		}
		if (preg_match('/[^A-Za-z0-9]/', $username)) {
			$this->setError( __('Error: The login is not valid, only letters and digits allowed.'), 400);
			return false;
		}
		// get the user with the same username if available
        if (false === ($user = $this->getUserById($username))) {
	        return false;
        }

		// add a user check
		if (empty($id)) {
			if ($user->nbRow() > 0) {
				$this->setError( __('Error: This login is already used, please use another one.'), 400);
				return false;
			}
			if (strlen($password) == 0) {
				$this->setError( __('Error: You need to give a password.'), 400);
				return false;
			}
		} else {
			if ($user->nbRow() > 0 && $user->f('user_id') != $id) {
				$this->setError( __('Error: This login is already used, please use another one.'), 400);
				return false;
			}
		}
		if (strlen($realname) == 0) {
            $this->setError( __('Error: You need to give a name.'), 400);
            return false;
		}
		if (empty($id)) {
			$insReq = 'INSERT INTO ';
		} else {
			$insReq = 'UPDATE ';
		}
		$insReq .= $this->con->pfx.'users SET
			user_username = \''.$this->con->escapeStr($username).'\',
			user_realname = \''.$this->con->escapeStr($realname).'\',
			user_email    = \''.$this->con->escapeStr($email).'\',
			user_pubemail = \''.$this->con->escapeStr($pubemail).'\'';
		if (!empty($password)) {
			$insReq .= ', user_password = \''.$this->con->escapeStr(md5($password)).'\'';
		}
		if (empty($id)) {
			$insReq .= ', user_creationdate = \''.timestamp().'\'';
		} else {
			$insReq .= ' WHERE user_id = \''.$this->con->escapeStr($id).'\'';
		}
        if (!$this->con->execute($insReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }

        if (empty($id)) $id = $this->con->getLastID();

		if (!is_null($authwebs)) {
    		// update the rights for the websites
    		$delReq = 'DELETE FROM '.$this->con->pfx.'grants WHERE user_id = \''.$this->con->escapeStr($id).'\'';
            if (!$this->con->execute($delReq)) {
                $this->setError('MySQL : '.$this->con->error(), 500);
                return false;
            }
            foreach ($authwebs as $site => $score) {
    			$insReq = 'INSERT INTO '.$this->con->pfx.'grants SET
    						user_id = \''.$this->con->escapeStr($id).'\',
    						website_id = \''.$this->con->escapeStr($site).'\',
    						level = \''.$this->con->escapeStr($score).'\'';
               if (!$this->con->execute($insReq)) {
                	$this->setError('MySQL : '.$this->con->error(), 500);
                	return false;
           		}
        	}
		}
		if (!empty($id)) {
			$this->user->set($username);
			$this->user->synchronize(PX_USER_SYNCHRO_TO_SESSION);
		}
        return $id;
	}


	function delUser($id)
	{
		if (false === ($user = $this->getUserById($id))) {
			return false;
		}
        $res = $user->getListResources();
        if ($res->nbRow() > 0 || $id == 1) {
        	$this->setError( __('Error: This user cannot be deleted.'), 400);
        	return false;
        }
		$delReq = 'DELETE FROM '.$this->con->pfx.'grants WHERE user_id = \''.$this->con->escapeStr($id).'\'';
        if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
		$delReq = 'DELETE FROM '.$this->con->pfx.'users WHERE user_id = \''.$this->con->escapeStr($id).'\'';
        if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
		return true;

	}


	/**
		Save a site or create a new one. If $id is empty, a new site is created and the
		log of the creation is set in &$log_new_site. The log is pure HTML ready for
		display.
	*/
	function saveSite($id, $name, $description, $sitelang, $website_address, $website_path, $xmedia_name, &$log_new_site, $force_new_id='')
	{
		include_once dirname(__FILE__).'/../extinc/class.configfile.php';
		include_once dirname(__FILE__).'/class.checklist.php';
		global $_PX_config;

		$update = (empty($id)) ? false : true;
		$xmedia_path = '';
		if (!empty($website_path) && !empty($xmedia_name))
			$xmedia_path = files::real_path($website_path).'/'.$xmedia_name;
		$parsedurl = parse_url($website_address);

		if ($update) {
			// check the data if $update
			if (0 == strlen(trim($id))) {
				$this->setError( __('Error: Internal error, please report your actions leading to this error message.'),500);
			}
			if (preg_match('/[^A-Za-z0-9]/', $id)) {
				$this->setError( __('Error: Internal error, please report your actions leading to this error message.'),500);
			}
		}
		if (empty($xmedia_path) or !file_exists($xmedia_path)) {
			$this->setError( sprintf(__('Error: File and image folder %s not available. Check the name you gave.'), $xmedia_path), 400);
		}
		if (!empty($xmedia_path) && file_exists($xmedia_path) && !is_writable($xmedia_path)) {
			$this->setError( sprintf(__('Error: No write access to the file and image folder %s. Check the name you gave.'), $xmedia_path), 400);
		}
		if (empty($website_path) or !file_exists($website_path)) {
			$this->setError( sprintf(__('Error: The document root folder %s is not available.'), $website_path), 400);
		}
		if (!empty($website_path) && file_exists($website_path) && !is_writable($website_path)) {
			$this->setError( sprintf(__('Error: No write access to the root folder %s.'), $website_path), 400);
		}
		if (2 != strlen(trim($sitelang))) {
			$sitelang = 'en';
		}
		if (0 == strlen(trim($website_address))) {
			$this->setError( __('Error: You must give a website address.'),400);
		}
		if (0 == strlen(trim($description))) {
			$this->setError( __('Error: You must give a description.'),400);
		}
		if ((0 != strlen(trim($website_address))) && (!is_array($parsedurl)
		or empty($parsedurl['scheme'])
		or !preg_match('/(http|https)/', $parsedurl['scheme'])
		or empty($parsedurl['host'])
		or !preg_match('/^([0-9a-z\-]+\.)+[a-z]{2,4}$|^[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}$/i', $parsedurl['host']))) {
			$this->setError( __('Error: You must provide a valid website address.'),400);
		}
		if (0 == strlen(trim($name))) {
			$this->setError( __('Error: You must give a name.'),400);
		}

		// if errors, break
		if (false !== $this->error(true, false)) {
			return false;
		}

		// Generate all the needed information for at least the update
		$xmedia_path = files::real_path($xmedia_path);
		$website_path = files::real_path($website_path);
		$reurl = (!empty($parsedurl['path'])) ? $parsedurl['path'] : '';

		$reurl        = preg_replace('#(/)+$#', '', $reurl);
		$website_path = preg_replace('#(/)+$#', '', $website_path);
		$xmedia_reurl = preg_replace('#(/)+$#', '', $reurl.'/'.$xmedia_name);

		$domain = trim($parsedurl['host']);
		$secure = (strtolower($parsedurl['scheme']) == 'https');

		// get the website with this id
		if (empty($id)) {
			if (!empty($force_new_id)) $id = $force_new_id; //to be able to force the first site with the "default" id.
			else $id = substr(preg_replace('/[^A-Za-z0-9]/', '', $website_address), 4); //new id from the address (without http but the s)
		}

		$site = $this->getSites($id);

		if ($update) {
			if (!file_exists(dirname(__FILE__).'/../conf/configweb_'.$id.'.php') || !is_writable(dirname(__FILE__).'/../conf/configweb_'.$id.'.php')) {
				$this->setError(sprintf( __('Error: The configuration file %s is not writeable.'), files::real_path(dirname(__FILE__).'/../conf/').'/configweb_'.$id.'.php'), 500);
				return false;
			}
			if ($site->nbRow() == 0) {
				$this->setError( __('This site is not available.') , 400);
				return false;
			}

		} else {
			// check if this website already exists
			if ($site->nbRow() >= 1) {
				$this->setError( __('Error: Id already used.') , 400);
				return false;
			}
			// need to create a new file
			// copy paste the config_default.php
			if (!is_writable(dirname(__FILE__).'/../conf/')) {
				$this->setError(sprintf( __('Error: The configuration folder %s is not writeable.'),
					files::real_path(dirname(__FILE__).'/../conf/')), 500);
				return false;
			}
			$source_file      = dirname(__FILE__).'/../conf/configweb_default.copy.php';
			$destination_file = dirname(__FILE__).'/../conf/configweb_'.$id.'.php';
			if (file_exists($destination_file)) {
				@unlink ($destination_file);
			}
			if (!copy($source_file, $destination_file)) {
				$this->setError( __('Error: Impossible to create the configuration file.') , 500);
				return false;
			}
			@chmod($destination_file, 0666);

		}
		// open file for edition of the data
		$cfg = new configfile(dirname(__FILE__).'/../conf/configweb_'.$id.'.php');
		$cfg->prefix = '_PX_website_config';
		$cfg->editVar('website_id',    (string) $id);
		$cfg->editVar('xmedia_root',   (string) $xmedia_path);
		$cfg->editVar('domain',        (string) $domain);
		$cfg->editVar('rel_url',       (string) $reurl);
		$cfg->editVar('rel_url_files', (string) $xmedia_reurl);
		$cfg->editVar('secure',        (bool)   $secure);
		$cfg->editVar('lang',          (string) $sitelang);

		if (!$cfg->saveFile()) {
			$this->setError( __('Error: Impossible to create the configuration file.') , 500);
			return false;
		}
		// no update or add in the db
		if ($update) {
			$insReq = 'UPDATE '.$this->con->pfx.'websites SET ';
		} else {
			$insReq = 'INSERT INTO '.$this->con->pfx.'websites SET
					  website_id =\''.$this->con->escapeStr($id).'\',
					  website_startdate = \''.timestamp().'\', ';
		}
		$securestring = $secure ? 's' : '';
		$insReq .= 'website_name   = \''.$this->con->escapeStr($name).'\', ';
		$insReq .= 'website_url    = \''.$this->con->escapeStr('http'.$securestring.'://'.$domain.$reurl).'\', ';
		$insReq .= 'website_reurl  = \''.$this->con->escapeStr($reurl).'\', ';
		$insReq .= 'website_path   = \''.$this->con->escapeStr('').'\', ';
		$insReq .= 'website_xmedia_reurl   = \''.$this->con->escapeStr($xmedia_reurl).'\', ';
		$insReq .= 'website_xmedia_path   = \''.$this->con->escapeStr($xmedia_path).'\', ';
		$insReq .= 'website_description  = \''.$this->con->escapeStr($description).'\' ';
		if ($update) {
			$insReq .= 'WHERE website_id =\''.$this->con->escapeStr($id).'\'';
		}

		if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
		}


		if (!$update) {
			include_once dirname(__FILE__).'/lib.auth.php';
			// As this user adds the site, give him root access to it
			$insReq = 'INSERT INTO '.$this->con->pfx.'grants SET
					  website_id =\''.$this->con->escapeStr($id).'\',
					  user_id    = \''.$this->con->escapeStr($this->user->_userid).'\',
					  level      = \''.PX_AUTH_ADMIN.'\'';

			if (!$this->con->execute($insReq)) {
				$this->setError('MySQL : '.$this->con->error(), 500);
				return false;
			}

			// Add the root category
			$hp_title = __('Home Page');
			$hp_desc = __('Root category, that plays the role of homepage.');
			$hp_kw = __('homepage, index, default');
			$insReq='INSERT INTO '.$this->con->pfx.'categories SET
				website_id=\''.$this->con->escapeStr($id).'\',
				category_name=\''.$this->con->escapeStr($hp_title).'\',
				category_description=\''.$this->con->escapeStr($hp_desc).'\',
				category_keywords=\''.$this->con->escapeStr($hp_kw).'\',
				category_path=\'/\',
				category_publicationdate=\''.timestamp().'\',
				category_creationdate=\''.timestamp().'\',
				category_enddate=99991231235959,
				category_template=\'category_homepage.php\',
				category_type=\'default\',
				category_cachetime=86400';
			if (!$this->con->execute($insReq)) {
				$this->setError('MySQL : '.$this->con->error(), 500);
				return false;
			}
			if (false == ($catid = $this->con->getLastID())) {
				$this->setError('MySQL : '.$this->con->error(), 500);
				return false;
			}
			$updReq = 'UPDATE '.$this->con->pfx.'categories SET category_parentid=\''.$this->con->escapeStr($catid).'\'
					WHERE category_id = \''.$this->con->escapeStr($catid).'\'';

			if (!$this->con->execute($updReq)) {
				$this->setError('MySQL : '.$this->con->error(), 500);
				return false;
			}
			// Add 2 default subtypes

			if (false === $this->saveType('', 'articles', __('Article'), 'resource_article.php', 3600, '', '', $id)) {
				return false;
			}
			if (false === $this->saveType('', 'news', __('News'), 'resource_news.php', 3600, '1', '', $id)) {
				return false;
			}
			
			// All the database related work is done. The creation is a success, we may have error to copy the 
			// files, but they are not erros, only "warnings".
			// 1- Create the xmedia/thumb folder
			// 2- Create the xmedia/theme folder
			// 3- Create the manager/templates/$id folder
			// 4- Copy folder manager/templates/_dist/default/theme into folder created in 2
			// 5- Copy folder manager/templates/_dist/default/templates into folder created in 3
			// 6- If id != 'default' copy from the 'default' document root config.php index.php prepend.php rss.php search.php
			//    into new document root
			// 7- Edit config.php for $_PX_config['manager_path'] and to load the good config file.

			include_once dirname(__FILE__).'/class.files.php';
			include_once dirname(__FILE__).'/class.checklist.php';
			$f = new files();
			$checklist = new checklist();
			// 1- Create the xmedia/thumb folder
			$checklist->addTest('thumb-folder', files::is_success($f->createfolder($xmedia_path.'/thumb', 0777)) ? 1 : 2,
				sprintf(__('Thumbnail folder %s created successfully.'), files::real_path($xmedia_path.'/thumb')),
				'' /* no error */,
				sprintf(__('Unable to create the thumbnail folder %s.'), $xmedia_path.'/thumb'));

			// 2- Create the xmedia/theme folder
			$checklist->addTest('theme-folder', files::is_success($f->createfolder($xmedia_path.'/theme', 0777)) ? 1 : 2,
				sprintf(__('Theme folder %s created successfully.'), files::real_path($xmedia_path.'/theme')),
				'' /* no error */,
				sprintf(__('Unable to create the theme folder %s.'), $xmedia_path.'/theme'));

			// 3- Create the manager/templates/$id folder
			$checklist->addTest('template-folder', files::is_success($f->createfolder(files::real_path(dirname(__FILE__).'/../templates/'.$id), 0777)) ? 1 : 2,
				sprintf(__('Template folder %s created successfully.'), files::real_path(dirname(__FILE__).'/../templates/'.$id)),
				'' /* no error */,
				sprintf(__('Unable to create the template folder %s.'), dirname(__FILE__).'/../templates/'.$id));

			// 4- Copy folder manager/templates/_dist/default/theme into folder created in 2
			$checklist->addTest('content-theme-folder',
				files::is_success($f->copyfolder(files::real_path(dirname(__FILE__).'/../templates/_dist/default/theme'),
				                                 files::real_path($xmedia_path.'/theme'))) ? 1 : 2,
				sprintf(__('Theme files successfully copied from %s to the theme folder.'), files::real_path(dirname(__FILE__).'/../templates/_dist/default/theme')),
				'' /* no error */,
				sprintf(__('Unable to copy the theme files from %s to the theme folder.'), files::real_path(dirname(__FILE__).'/../templates/_dist/default/theme')));

			// 5- Copy folder manager/templates/_dist/default/templates into folder created in 3
			$checklist->addTest('content-template-folder',
				files::is_success($f->copyfolder(files::real_path(dirname(__FILE__).'/../templates/_dist/default/templates'),
				                                 files::real_path(dirname(__FILE__).'/../templates/'.$id))) ? 1 : 2,
				sprintf(__('Templates files successfully copied from %s to the template folder.'), files::real_path(dirname(__FILE__).'/../templates/_dist/default/templates')),
				'' /* no error */,
				sprintf(__('Unable to copy the templates files from %s to the template folder.'), files::real_path(dirname(__FILE__).'/../templates/_dist/default/templates')));

			// 6- If id != 'default' copy from the 'default' document root config.php index.php prepend.php rss.php search.php
			//    into new document root
			if ('default' != $id) {
				$checklist->addTest('config-php',
					files::is_success($f->copyfile(files::real_path(dirname(__FILE__).'/../../config.php'), $website_path.'/config.php')) ? 1 : 2,
					sprintf(__('Config file successfully copied from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../config.php') ),
					'' /* no error */,
					sprintf(__('Unable to copy the config file from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../config.php') ));

				$checklist->addTest('index-php',
					files::is_success($f->copyfile(files::real_path(dirname(__FILE__).'/../../index.php'), $website_path.'/index.php')) ? 1 : 2,
					sprintf(__('Index file successfully copied from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../index.php') ),
					'' /* no error */,
					sprintf(__('Unable to copy the index file from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../index.php') ));

				$checklist->addTest('prepend-php',
					files::is_success($f->copyfile(files::real_path(dirname(__FILE__).'/../../prepend.php'), $website_path.'/prepend.php')) ? 1 : 2,
					sprintf(__('Prepend file successfully copied from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../prepend.php') ),
					'' /* no error */,
					sprintf(__('Unable to copy the prepend file from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../prepend.php') ));

				$checklist->addTest('rss-php',
					files::is_success($f->copyfile(files::real_path(dirname(__FILE__).'/../../rss.php'), $website_path.'/rss.php')) ? 1 : 2,
					sprintf(__('Rss file successfully copied from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../rss.php') ),
					'' /* no error */,
					sprintf(__('Unable to copy the rss file from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../rss.php') ));

				$checklist->addTest('search-php',
					files::is_success($f->copyfile(files::real_path(dirname(__FILE__).'/../../search.php'), $website_path.'/search.php')) ? 1 : 2,
					sprintf(__('Search file successfully copied from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../search.php') ),
					'' /* no error */,
					sprintf(__('Unable to copy the search file from %s to the document root folder.'), files::real_path(dirname(__FILE__).'/../../search.php') ));

				// 7- Edit config.php for $_PX_config['manager_path'] and to load the good config file.
				// open file for edition of the data
				$cfg = new configfile($website_path.'/config.php');
				$cfg->prefix = '_PX_config';
				$cfg->editVar('manager_path',    (string) files::real_path($_PX_config['manager_path']));
				$edit_config_success = 1;
				if (!$cfg->saveFile()) {
					$edit_config_success = 2;
				} else {
					if (!file_exists($website_path.'/config.php') or !is_writable($website_path.'/config.php')) {
						$edit_config_success = 2;
					} else {
						$config_file = @join('', @file($website_path.'/config.php'));
						$config_file = preg_replace('/configweb\_([A-Za-z0-9]+)\.php/', 'configweb_'.$id.'.php', $config_file);
						$open = @fopen($website_path.'/config.php', 'w');
						@fwrite($open, $config_file);
						@fclose($open);
					}
				}
				$checklist->addTest('edit-config-php', $edit_config_success,
					sprintf(__('Config file %s successfully updated.'), files::real_path($website_path.'/config.php') ),
					'' /* no error */,
					sprintf(__('Unable to update the config file %s.'), files::real_path($website_path.'/config.php') ));

			} //end of if not 'default'
			$path = ('default' != $id) ? 'themes/'.$GLOBALS['_px_theme'].'/images' : '../themes/default/images';
			$log_new_site = $checklist->getHtml($path);

		}

		return true;

	}

	function delSite($id)
	{
		if (preg_match('/[^A-Za-z0-9]/', $id)) {
			$this->setError( __('Error: Invalid id, it must contain only letters and digits.'),400);
			return false;
		}

		// get the website with this id
		$site = $this->getSites($id);
        if ($site->nbRow() == 0) {
        	$this->setError( __('This site is not available.') , 400);
        	return false;
        }
		$date = $this->getEarlierDate('m', '', '', $id);
		if (strlen($date) == 14) {
        	$this->setError( __('Error: The site can only be deleted if empty.') , 400);
        	return false;
		}


        if (	 !file_exists(dirname(__FILE__).'/../conf/configweb_'.$id.'.php')
          	|| !is_writable(dirname(__FILE__).'/../conf/configweb_'.$id.'.php')) {

        	$this->setError(sprintf( __('Error: The configuration file %s is not writeable.'),
        	files::real_path(dirname(__FILE__).'/../conf/').'/configweb_'.$id.'.php'), 500);
        return false;
        }


		$delReq = 'DELETE FROM '.$this->con->pfx.'websites WHERE website_id =\''.$this->con->escapeStr($id).'\'';
		if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
		$delReq = 'DELETE FROM '.$this->con->pfx.'grants WHERE website_id =\''.$this->con->escapeStr($id).'\'';
		if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
		$delReq = 'DELETE FROM '.$this->con->pfx.'userprefs WHERE website_id =\''.$this->con->escapeStr($id).'\'';
		if (!$this->con->execute($delReq)) {
            $this->setError('MySQL : '.$this->con->error(), 500);
            return false;
        }
		@unlink(dirname(__FILE__).'/../conf/configweb_'.$id.'.php');
		return true;
	}



    /**
    Add a news
    */
    function addNews($news)
    {
        // first check the integrity of the news
        if (true !== ($check = $this->checkNews($news, true))) {
            return false;
        }
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $news->f('website_id'))
               || ($news->f('user_id') == $this->user->user['user_id'] && $news->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this news.'), 400);
            return false;
        }

        $title      = $news->f('title');
        $subject    = $news->f('subject');
        $content    = $news->f('description');
        $status     = $news->f('status');
        $datestart  = $news->f('publicationdate');
        $dateend    = $news->f('enddate');
        $website    = $news->f('website_id');
        $categoryid   = $news->categories->f('category_id');
        $categoryasso = $news->categories->f('categoryasso_type');
        $subtype = $news->f('subtype_id');
        $titlewebsite = $news->f('news_titlewebsite');
        $linkwebsite = $news->f('news_linkwebsite');

        $insReq = 'INSERT INTO '.$this->con->pfx.'resources SET
                     website_id      = \''.$this->con->escapeStr($website).'\',
                     type_id         = \''.PX_RESOURCE_MANAGER_NEWS.'\',
                     subtype_id         = \''.$this->con->escapeStr($subtype).'\',
                     user_id         = \''.$this->user->user['user_id'].'\',
                     subject         = \''.$this->con->escapeStr($subject).'\',
                     creatorname     = \'\',
                     creatoremail    = \'\',
                     creatorwebsite  = \'\',
                     publisher       = \'\',
                     lang_id         = \''.$_SESSION['lang'].'\',
                     title           = \''.$this->con->escapeStr(textSecureString($title)).'\',
                     description     = \''.$this->con->escapeStr($content).'\',
                     creationdate    = \''.timestamp().'\',
                     publicationdate = \''.$this->con->escapeStr($datestart).'\',
                     modifdate       = \''.timestamp().'\',
                     enddate         = \''.$this->con->escapeStr($dateend).'\',
                     status          = \''.$this->con->escapeStr($status).'\',
                     size            = \'\',
                     version         = 1,
                     comment         = \'\',
                     misc            = \'\',
                     format          = \'text/html\',
                     dctype          = \'Text\',
                     dccoverage      = \'\',
                     rights          = \'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        if (false == ($id = $this->con->getLastID())) {
            $this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }

        $updReq = 'UPDATE '.$this->con->pfx.'resources SET identifier = \'news-'.$this->con->escapeStr($id).'\',
					path = \''.$this->con->escapeStr($id).'-'.$this->con->escapeStr(textStr2url($title)).'\'
        		WHERE resource_id = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($updReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        $insReq = 'INSERT INTO '.$this->con->pfx.'categoryasso SET
                    category_id       = \''.$this->con->escapeStr($categoryid).'\',
                    identifier        = \'news-'.$this->con->escapeStr($id).'\',
                    categoryasso_type = \''.PX_RESOURCE_CATEGORY_MAIN.'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'INSERT INTO '.$this->con->pfx.'authorasso SET
                    user_id         = \''.$this->user->user['user_id'].'\',
                    resource_id     = \''.$this->con->escapeStr($id).'\',
                    authorasso_type = \''.PX_RESOURCE_CREATOR.'\',
                    authorasso_date = \''.timestamp().'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'INSERT INTO '.$this->con->pfx.'news SET
                    resource_id       = \''.$this->con->escapeStr($id).'\',
                    news_serial       = \''.$this->con->escapeStr(md5($title)).'\',
                    news_titlewebsite = \''.$this->con->escapeStr($titlewebsite).'\',
                    news_linkwebsite  = \''.$this->con->escapeStr($linkwebsite).'\',
                    feed_id           = \'0\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
		$this->indexResource($id, $news);
        $this->triggerMassUpdate();
        return $id;    
    }


    /**
    Del a news
    */
    function delNews($newsid)
    {
        if (false === ($news = $this->getResourceByIdentifier($newsid))) return false;
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $news->f('website_id'))
               || ($news->f('user_id') == $this->user->user['user_id'] && $news->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this news.'), 400);
            return false;
        }
        $id         = $news->f('resource_id');
                                           
        $insReq = 'DELETE FROM '.$this->con->pfx.'resources WHERE
                     resource_id     = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
                    
        $insReq = 'DELETE FROM '.$this->con->pfx.'categoryasso WHERE
                    identifier        = \'news-'.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'DELETE FROM '.$this->con->pfx.'authorasso WHERE
                    resource_id     = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'DELETE FROM '.$this->con->pfx.'news WHERE
                    resource_id       = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        $this->indexRemove($id);
        $this->triggerMassUpdate();
            
        return $id;    
    }

    
    /**
    Update a news
    */
    function updNews($news)
    {
        // first check the integrity of the news
        if (true !== ($check = $this->checkNews($news))) {
            return false;
        }
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $news->f('website_id'))
               || ($news->f('user_id') == $this->user->user['user_id'] && $news->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this news.'), 400);
            return false;
        }                                   
         
        $title      = $news->f('title');
        $subject    = $news->f('subject');
        $content    = $news->f('description');
        $status     = $news->f('status');
        $datestart  = $news->f('publicationdate');
        $dateend    = $news->f('enddate');
        $website    = $news->f('website_id');
        $id         = $news->f('resource_id');
        $subtype    = $news->f('subtype_id');
        $titlewebsite = $news->f('news_titlewebsite');
        $linkwebsite = $news->f('news_linkwebsite');
          
   
        $insReq = 'UPDATE '.$this->con->pfx.'resources SET
                     subject         = \''.$this->con->escapeStr($subject).'\',
                     subtype_id      = \''.$this->con->escapeStr($subtype).'\',
                     title           = \''.$this->con->escapeStr(textSecureString($title)).'\',
                     description     = \''.$this->con->escapeStr($content).'\',
                     path            = \''.$this->con->escapeStr($id).'-'.$this->con->escapeStr(textStr2url($title)).'\',
                     publicationdate = \''.$this->con->escapeStr($datestart).'\',
                     modifdate       = \''.timestamp().'\',
                     enddate         = \''.$this->con->escapeStr($dateend).'\',
                     status          = \''.$this->con->escapeStr($status).'\'
                     WHERE resource_id     = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'UPDATE '.$this->con->pfx.'news SET
                    news_serial       = \''.$this->con->escapeStr(md5($title)).'\',
                    news_titlewebsite = \''.$this->con->escapeStr($titlewebsite).'\',
                    news_linkwebsite = \''.$this->con->escapeStr($linkwebsite).'\'
                    WHERE resource_id     = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }          

		$this->indexResource($id, $news);
        $this->triggerMassUpdate();
        return $id;    
    }

    
    function checkNews(&$news, $new = false)
    {
        if (strlen(textSecureString($news->f('title'))) == 0)
            $this->setError( __('You need to give a title.'), 400); 
        if (false !== ($date = checkCleanDate($news->f('publicationdate'))))
            $news->setField('publicationdate', $date);
        else $this->setError( __('The publication date is wrong.'), 400);             
        if (false !== ($date = checkCleanDate($news->f('enddate'))))
            $news->setField('enddate', $date);
        else $this->setError( __('The off-line date is wrong.'), 400);
        if ($new) {
            // need to check if the category id provided is ok
            $cat = $this->getCategories();
            $ids = $cat->getIDs('', 'category_id');
            if (!in_array($news->categories->f('category_id'), $ids)) {
                $this->setError( __('The proposed category is invalid.'), 400);
            }                   
        }  
        if (strlen($news->f('subtype_id')) == 0)
			$this->setError( __('A type of resource is needed.'), 400); 
        if (strlen($news->f('news_titlewebsite')) > 0) {
            if (!preg_match('#^(http|ftp|news|https)://#', $news->f('news_linkwebsite'))) {
                $this->setError( __('The associated link must start with http:// or a valid protocol.'), 400);
            }
        }          
        
        if (false !== ($error = $this->error()))
            return $error;            
        return true;       
    }



    /**
    Add an article
    */
    function addArticle($ar)
    {
        // first check the integrity of the article
        if (true !== ($check = $this->checkArticle($ar, true))) {
            return false;
        }
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $ar->f('website_id'))
               || ($ar->f('user_id') == $this->user->user['user_id'] && $ar->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this article.'), 400);
            return false;
        }                                   
         
        $title      = $ar->f('title');
        $subject    = $ar->f('subject');
        $content    = $ar->f('description');
        $status     = $ar->f('status');
        $datestart  = $ar->f('publicationdate');
        $dateend    = $ar->f('enddate');
        $website    = $ar->f('website_id');
        $path       = $ar->f('path');
        $subtype    = (int) $ar->f('subtype_id');
        $categoryid   = $ar->categories->f('category_id');
        $categoryasso = $ar->categories->f('categoryasso_type');
          
        $insReq = 'INSERT INTO '.$this->con->pfx.'resources SET
                     website_id      = \''.$this->con->escapeStr($website).'\',
                     type_id         = \''.PX_RESOURCE_MANAGER_ARTICLE.'\',
                     subtype_id      = \''.$this->con->escapeStr($subtype).'\',
                     user_id         = \''.$this->user->user['user_id'].'\',
                     subject         = \''.$this->con->escapeStr($subject).'\',
                     creatorname     = \'\',
                     creatoremail    = \'\',
                     creatorwebsite  = \'\',
                     publisher       = \'\',
                     lang_id         = \''.$_SESSION['lang'].'\',
                     title           = \''.$this->con->escapeStr(textSecureString($title)).'\',
                     description     = \''.$this->con->escapeStr($content).'\',
                     path            = \''.$this->con->escapeStr($path).'\',
                     creationdate    = \''.timestamp().'\',
                     publicationdate = \''.$this->con->escapeStr($datestart).'\',
                     modifdate       = \''.timestamp().'\',
                     enddate         = \''.$this->con->escapeStr($dateend).'\',
                     status          = \''.$this->con->escapeStr($status).'\',
                     size            = \'\',
                     version         = 1,
                     comment         = \'\',
                     misc            = \'\',
                     format          = \'text/html\',
                     dctype          = \'Text\',
                     dccoverage      = \'\',
                     rights          = \'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }

		if (false == ($id = $this->con->getLastID())) {
            $this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }

        $updReq = 'UPDATE '.$this->con->pfx.'resources SET identifier = \'art-'.$this->con->escapeStr($id).'\'
				WHERE resource_id = \''.$this->con->escapeStr($id).'\'';

	    if (!$this->con->execute($updReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }

        $insReq = 'INSERT INTO '.$this->con->pfx.'categoryasso SET
                    category_id       = \''.$this->con->escapeStr($categoryid).'\',
                    identifier        = \'art-'.$this->con->escapeStr($id).'\',
                    categoryasso_type = \''.PX_RESOURCE_CATEGORY_MAIN.'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'INSERT INTO '.$this->con->pfx.'authorasso SET
                    user_id         = \''.$this->user->user['user_id'].'\',
                    resource_id     = \''.$this->con->escapeStr($id).'\',
                    authorasso_type = \''.PX_RESOURCE_CREATOR.'\',
                    authorasso_date = \''.timestamp().'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        $this->indexResource($id, $ar);
        $this->triggerMassUpdate();
        return $id;    
    }
    

    /**
    Update an article
    */
    function updArticle($ar)
    {
        // first check the integrity of the article
        if (true !== ($check = $this->checkArticle($ar))) {
            return false;
        }
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $ar->f('website_id'))
               || ($ar->f('user_id') == $this->user->user['user_id'] && $ar->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this article.'), 400);
            return false;
        }                                   
         
        $title      = $ar->f('title');
        $subject    = $ar->f('subject');
        $content    = $ar->f('description');
        $status     = $ar->f('status');
        $datestart  = $ar->f('publicationdate');
        $dateend    = $ar->f('enddate');
        $path       = $ar->f('path');
        $id         = $ar->f('resource_id');
        $subtype    = (int) $ar->f('subtype_id');
        
    
        $insReq = 'UPDATE '.$this->con->pfx.'resources SET
                     title           = \''.$this->con->escapeStr(textSecureString($title)).'\',
                     subtype_id      = \''.$this->con->escapeStr($subtype).'\',
                     description     = \''.$this->con->escapeStr($content).'\',
                     path            = \''.$this->con->escapeStr($path).'\',
                     publicationdate = \''.$this->con->escapeStr($datestart).'\',
                     modifdate       = \''.timestamp().'\',
                     enddate         = \''.$this->con->escapeStr($dateend).'\',
                     subject         = \''.$this->con->escapeStr($subject).'\',
                     status          = \''.$this->con->escapeStr($status).'\'
                     WHERE resource_id     = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        $this->indexResource($id, $ar);
        $this->triggerMassUpdate();
        return $id;    
    }

    /**
    Add an article page
    */
    function addArticlePage($ar)
    {
        // first check the integrity of the article
        if (true !== ($check = $this->checkArticle($ar))) {
            return false;
        }
        // check the integrity if the page
        if (true !== ($check = $this->checkArticlePage($ar))) {
            return false;
        }            
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $ar->f('website_id'))
               || ($ar->f('user_id') == $this->user->user['user_id'] && $ar->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this article.'), 400);
            return false;
        }

        $oldnumber = $ar->pages->f('old_page_number');
        $number    = $ar->pages->f('page_number'); 
        $title     = $ar->pages->f('page_title');
        $content   = $ar->pages->f('page_content');

        if ($oldnumber === '') $oldnumber = $ar->pages->nbRow();
        if ($number === '') $number = $ar->pages->nbRow();
        
        if ($oldnumber != $number) {
            // need to update the numbers of the pages
            $insReq = 'UPDATE '.$this->con->pfx.'articles SET'.
                ' page_number=page_number+1'.
                ' WHERE page_number < '.$this->con->escapeStr($oldnumber).
                ' AND page_number >= '.$this->con->escapeStr($number).
                ' AND resource_id=\''.$this->con->escapeStr($ar->f('resource_id')).'\'';
    	    if (!$this->con->execute($insReq)) {
    			$this->setError('MySQL : '.$this->con->error(), 500);
    			return false;
            }
            $insReq = 'UPDATE '.$this->con->pfx.'articles SET'.
                ' page_number=page_number-1'.
                ' WHERE page_number > '.$this->con->escapeStr($oldnumber).
                ' AND page_number <= '.$this->con->escapeStr($number).
                ' AND resource_id=\''.$this->con->escapeStr($ar->f('resource_id')).'\'';
    	    if (!$this->con->execute($insReq)) {
    			$this->setError('MySQL : '.$this->con->error(), 500);
    			return false;
            }
        }
        if (strlen($number) == 0) $number = $ar->pages->nbRow();
        $insReq = 'INSERT INTO '.$this->con->pfx.'articles SET'.
                  ' page_number    = \''.$this->con->escapeStr($number).'\','.
                  ' page_title     = \''.$this->con->escapeStr(textSecureString($title)).'\','.
                  ' page_content   = \''.$this->con->escapeStr($content).'\','.
                  ' page_creationdate = \''.timestamp().'\','.
                  ' page_modifdate = \''.timestamp().'\','.
                  ' resource_id=\''.$this->con->escapeStr($ar->f('resource_id')).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        $id = $this->con->getLastID();

        $this->indexResource($ar->f('resource_id'), $ar);
        $this->triggerMassUpdate();
        return $id;    
    }
    
    /**
    Update an article page
    */
    function updArticlePage($ar)
    {
        // first check the integrity of the article
        if (true !== ($check = $this->checkArticle($ar))) {
            return false;
        }
        // check the integrity of the page
        if (true !== ($check = $this->checkArticlePage($ar))) {
            return false;
        }            
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $ar->f('website_id'))
               || ($ar->f('user_id') == $this->user->user['user_id'] && $ar->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this article.'), 400);
            return false;
        }                                   

        $oldnumber = $ar->pages->f('old_page_number');
        $number    = $ar->pages->f('page_number'); 
        $title     = $ar->pages->f('page_title');
        $content   = $ar->pages->f('page_content');
        $id        = $ar->pages->f('page_id'); 

        //if (strlen($oldnumber) == 0) $oldnumber = $ar->pages->nbRow() + 1;
        if ($oldnumber != $number) {
            // need to update the numbers of the pages
            $insReq = 'UPDATE '.$this->con->pfx.'articles SET'.
                ' page_number=page_number+1'.
                ' WHERE page_number < '.$this->con->escapeStr($oldnumber).
                ' AND page_number >= '.$this->con->escapeStr($number).
                ' AND resource_id=\''.$this->con->escapeStr($ar->f('resource_id')).'\'';
            //echo $insReq.'<br>';                
    	    if (!$this->con->execute($insReq)) {
    			$this->setError('MySQL : '.$this->con->error(), 500);
    			return false;
            }
            $insReq = 'UPDATE '.$this->con->pfx.'articles SET'.
                ' page_number=page_number-1'.
                ' WHERE page_number > '.$this->con->escapeStr($oldnumber).
                ' AND page_number <= '.$this->con->escapeStr($number).
                ' AND resource_id=\''.$this->con->escapeStr($ar->f('resource_id')).'\'';
            // echo $insReq.'<br>';                
    	    if (!$this->con->execute($insReq)) {
    			$this->setError('MySQL : '.$this->con->error(), 500);
    			return false;
            }
        }
        $insReq = 'UPDATE '.$this->con->pfx.'articles SET'.
                  ' page_number    = \''.$this->con->escapeStr($number).'\','.
                  ' page_title     = \''.$this->con->escapeStr(textSecureString($title)).'\','.
                  ' page_content   = \''.$this->con->escapeStr($content).'\','.
                  ' page_modifdate = \''.timestamp().'\''.
                  ' WHERE page_id = \''.$this->con->escapeStr($id).'\'';
        //echo $insReq.'<br>';                    
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        $this->indexResource($ar->f('resource_id'), $ar);
        $this->triggerMassUpdate();
        return $id;    
    }

    /**
    Del an article page
    */
    function delArticlePage($arid, $pageid)
    {
        if (false === ($ar = $this->getResourceByIdentifier($arid, 'Article'))) return false;
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $ar->f('website_id'))
               || ($ar->f('user_id') == $this->user->user['user_id'] && $ar->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this article.'), 400);
            return false;
        }
        $id         = $ar->f('resource_id');
        $ar->getPages();
        $ar->pages->moveStart();
        $page_id = $ar->pages->f('page_id');
        $is_editable = true;
        while ($pageid != $page_id) {
            if (false == $ar->pages->moveNext()) {
                $is_editable = false;
                $page_id = '';
                $this->setError( __('Error: The requested page is not available.'), 400); 
                break;               
            } else {
                $page_id = $ar->pages->f('page_id');
            }                
        }
        if ($is_editable) {
            $insReq = 'DELETE FROM '.$this->con->pfx.'articles WHERE
                        resource_id       = \''.$this->con->escapeStr($arid).'\''.
                      '  AND page_id=\''.$this->con->escapeStr($pageid).'\'';
    	    if (!$this->con->execute($insReq)) {
    			$this->setError('MySQL : '.$this->con->error(), 500);
    			return false;
            }
        }
        $this->indexResource($arid, $ar);
        $this->triggerMassUpdate();
        return true;    
    }    
    
    
    /**
    Del an article
    */
    function delArticle($arid)
    {
        if (false === ($ar = $this->getResourceByIdentifier($arid))) return false;
        if (!(    authAsLevel(PX_USER_LEVEL_ADVANCED, $ar->f('website_id'))
               || ($ar->f('user_id') == $this->user->user['user_id'] && $ar->f('status') != PX_RESOURCE_STATUS_VALIDE) )
           ) {
            $this->setError( __('Error: You do not have the rights to edit this article.'), 400);
            return false;
        }
        $id         = $ar->f('resource_id');
                                           
        $insReq = 'DELETE FROM '.$this->con->pfx.'resources WHERE
                     resource_id     = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
                    
        $insReq = 'DELETE FROM '.$this->con->pfx.'categoryasso WHERE
                    identifier        = \'news-'.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'DELETE FROM '.$this->con->pfx.'authorasso WHERE
                    resource_id     = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
        
        $insReq = 'DELETE FROM '.$this->con->pfx.'articles WHERE
                    resource_id       = \''.$this->con->escapeStr($id).'\'';
	    if (!$this->con->execute($insReq)) {
			$this->setError('MySQL : '.$this->con->error(), 500);
			return false;
        }
		$this->indexRemove($id);          
        $this->triggerMassUpdate();
        return $id;    
    }    
    

    function checkArticle(&$ar, $new = false)
    {
        if (strlen(textSecureString($ar->f('title'))) == 0)
            $this->setError( __('You need to give a title.'), 400); 
        
        if (!preg_match('/^([A-Za-z]|[A-Za-z][A-Za-z0-9\_\-]*[A-Za-z])$/', $ar->f('path')))
            $this->setError( __('Illegal caracters in the path.'), 400);            
        $r = 'SELECT resource_id FROM '.$this->con->pfx.'resources
          WHERE path=\''.$this->con->escapeStr($ar->f('path')).'\' AND
          website_id=\''.$this->con->escapeStr($this->user->website).'\'';
		if (($rs = $this->con->select($r)) !== false) {
            if ($rs->nbRow() > 0 && $rs->f('resource_id') != $ar->f('resource_id') ) {
                $this->setError( __('Path already used.'), 400);
			}
		} else {
			$this->con->setError();
            $this->setError('MySQL : '.$this->con->error(), 500);
		}         
        if (false !== ($date = checkCleanDate($ar->f('publicationdate'))))
            $ar->setField('publicationdate', $date);
        else $this->setError( __('The publication date is wrong.'), 400);             
        if (false !== ($date = checkCleanDate($ar->f('enddate'))))
            $ar->setField('enddate', $date);
        else $this->setError( __('The off-line date is wrong.'), 400);
        if ($new) {
            // need to check if the category id provided is ok
            $cat = $this->getCategories();
            $ids = $cat->getIDs('', 'category_id');
            if (!in_array($ar->categories->f('category_id'), $ids)) {
                $this->setError( __('The proposed category is invalid.'), 400);
            }                   
        }            
       if (strlen($ar->f('subtype_id')) == 0)
			$this->setError( __('A type of resource is needed.'), 400);
        if (false !== ($error = $this->error()))
            return $error;            
        return true;       
    }    

    function checkArticlePage(&$ar)
    {
        if (strlen(textSecureString($ar->pages->f('page_title'))) == 0) {
            $this->setError( __('The page must have a title.'), 400); 
        }
        if ( strlen( $ar->pages->f('page_number') ) && (!preg_match('/^\d+$/', $ar->pages->f('page_number'))))
            $this->setError( __('Error: Invalid page number.'), 400);            

        if (strlen($ar->pages->f('page_content')) == 0)
            $this->setError( __('The page must have a content.'), 400); 

        if ( strlen($ar->pages->f('page_id')) && !preg_match('/^\d+$/', $ar->pages->f('page_id')))
            $this->setError( __('Error: Invalid page id.'), 400);             

        if (false !== ($error = $this->error()))
            return $error;            
        return true;       
    }     

	/* -------------------------------------------------------------------------
	 *  Management of the help system
	 * ---------------------------------------------------------------------- */


	/**
	Get a help file. Returned as a string containing only the HTML content
	with the good locale and the encoding.
	
	@param string Id of the help to get
	@param string Id of the plugin if the help is from a plugin
	@param bool Get also the title and the id of the help
	@return string The help
	*/
	function getHelp($id, $plugin='', $getall = false)
	{
		if (preg_match('/[^a-z_\-]/i', $id) or preg_match('/[^a-z_\-]/i', $plugin))
			return '';

		$lang = $this->user->lang;


		$file = (empty($plugin)) ? $GLOBALS['_PX_config']['manager_path'] : $GLOBALS['_PX_config']['manager_path'] .'/tools/'.$plugin;
		$file .= '/help/'.$lang.'/'.$id.'.html';

		if (false !== ($help = $this->getHelpChapter($file))) {
			if ($getall)
				return $help;
			return $help[2];
		} else {
			return '';
		}

	}

	/**
	Get all the help files, sorted by file name usefull to list the 
	possible themes in the help.
	
	@param string Id of the plugin if the help is from a plugin
	@return array The help list
	*/
	function getHelpChapters($plugin='')
	{
		if (strlen($plugin) > 0 && preg_match('/[^a-z_\-]/i', $plugin))
			return false;

		$lang = $this->user->lang;

		$helpfolder = (empty($plugin)) ? $GLOBALS['_PX_config']['manager_path'] : $GLOBALS['_PX_config']['manager_path'] .'/tools/'.$plugin;
		$helpfolder .= '/help/'.$lang.'/';
		
		include_once dirname(__FILE__).'/class.files.php';
		
		$files = array();
		files::listfiles($helpfolder, $files, '/\.html$/');
		
		sort($files);
		reset($files);
		$chapters = array();
		foreach ($files as $file) {
			$chapters[] = $this->getHelpChapter($file);
		}
		return $chapters;
	}
	
	/**
	Get the id and the title of a help chapter from the file name and the 
	content encoded with the output encoding.
	
	@param string file name
	@return array 0=id 1=title 2=content
	*/
	function getHelpChapter($file)
	{

		if (!file_exists($file)) {
			$GLOBALS['_PX_debug_data']['help'][] = __('File not found: ').$file;
			return false;
		}

		$encoding = $GLOBALS['_PX_config']['encoding'];
		$id = substr(basename($file), 0, -5);
		$html = file($file);
		$convert = false;
		if (preg_match('/encoding=[\'"](.*)[\'"]\s*\?>\s*/', $html[0], $match)) {
			$codfile = strtolower($match[1]);
			if ('iso-8859-1' == $codfile && 'utf-8' == strtolower($encoding))
				$convert = true;
		}
		$html[0] = '';
		$htmlfile = implode('', $html);
		$title = substr($htmlfile, strpos($htmlfile,'<title>') + strlen('<title>'));
		$title = substr($title, 0, strpos($title, '</title>'));
		$htmlfile = substr($htmlfile, strpos($htmlfile,'<body>') + strlen('<body>'));
		$htmlfile = substr($htmlfile, 0, strpos($htmlfile, '</body>'));
		if ($convert) {
			$htmlfile = utf8_encode($htmlfile);
			$title = utf8_encode($title);
		}

		return array($id, $title, $htmlfile);
	}

	/**
	Display the contextual help link
	
	@param string chapter, correspond to the file of the help
	@param string section the part to see in the file
	@param string plugin ('')
	*/
	function HelpLink($chapter, $section, $plugin='')
	{
		$theme = $this->user->getTheme();
		
		if (strlen($plugin) > 0) $plugin = '&amp;p='.$plugin;
		$link = 'help.php?c='.$chapter.$plugin;
		$linkpop = $link.'&amp;mode=popup';
		$link .= '#'.$section;
		$linkpop .= '#'.$section;
		$img = 'themes/'.$theme.'/images/ico_help_small.png';
		$help = '<a title="'.__('Help').'" href="'.$link.'" onclick="popup(\''.$linkpop.'\'); return false;">'.
				'<img class="minihelp" src="'.$img.'" alt="'.__('Help').'" /></a>';
		echo $help;
	}
}


?>
