<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

/*==============================================================================
 Class User
==============================================================================*/
define('PX_USER_NOINIT', 0);
define('PX_USER_INIT',   1);
define('PX_USER_SYNCHRO_FROM_SESSION', 2);
define('PX_USER_SYNCHRO_TO_SESSION',   3);

require_once dirname(__FILE__).'/../extinc/class.recordset.php';

class User extends RecordSet
{
	var $prefs   = array();
	var $webs    = array();
	var $wdata   = array();
	var $user    = array();
	var $_userid = 0;
	var $website = ''; //current website
	var $lang    = '';  //current lang (can change with the website)
	var $con     = null;

	function User($user = 0, $init = PX_USER_NOINIT)
	{
		$this->con =& pxDBConnect();
		if (is_array($user)) {
			$this->recordset($user); //initialization of the parent object.
		} else {
			$this->_userid = $user;
			if (0 < (int)$user && PX_USER_INIT == $init) {
				$this->getPrefs();
				$this->getWebsites();
			}
		}
	}

	function set($user = '')
	{
		if (strlen($user) == 0) return false;
		if (!$this->con->con_id) {
			return false;
		}
		$r = 'SELECT * FROM '.$this->con->pfx.'users WHERE user_username
			LIKE \''.$this->con->escapeStr($user).'\' LIMIT 1';
		//echo $r;
		if (($rs = $this->con->select($r)) !== false) {
			//print_r($rs);
			$this->user = array_pop($rs->getData());
			if (isset($this->user['user_id'])) $this->_userid = $this->user['user_id'];
			if (false === $this->getPrefs() ||
				false === $this->getWebsites() ||
				null  === $this->user) {
				$this->clear();
				return false;
			} else {
				$this->_userid = $this->user['user_id'];
				return true;
			}
		} else {
			$this->setError('MySQL: '. $this->con->error(), 500);
			return false;
		}
	}

	function getListResources($website = '')
	{
		if ($this->_userid > 0) {
			$userid = $this->_userid;
		} else {
			$userid = $this->f('user_id');
		}
		if (!$this->con->con_id) {
			return false;
		}
		$r = 'SELECT * FROM '.$this->con->pfx.'resources
			WHERE  user_id = \''.$this->con->escapeStr($userid).'\'';
		if (!empty($website))
			$r .= ' AND website_id = \''.$this->con->escapeStr($website).'\'';
		if (($rs = $this->con->select($r)) !== false) {
			return $rs;
		} else {
			$this->setError('MySQL: ' . $this->con->error(), 500);
			return false;
		}
	}

	function clear()
	{
		$this->prefs   = array();
		$this->webs    = array();
		$this->user    = array();
		$this->_userid = 0;
	}
	
	function logout()
	{
	    $this->clear();
		$_SESSION = array();
		$_SESSION['user_id'] = '';
		$_SESSION['webs'] = array();
	}

	function checkUser($user, $pswd, $level = 1, $website = 'default')
	{
		if (0 == strlen($user) || 0 == strlen($pswd)) return false;
		if (!$this->con->con_id) {
			return false;
		}
		$r = 'SELECT * FROM '.$this->con->pfx.'users WHERE user_username
		      LIKE \''.$this->con->escapeStr($user).'\'
		      AND  user_password=\''.md5($pswd).'\'  LIMIT 1';
		if (($rs = $this->con->select($r)) !== false) {
			return !$rs->isEmpty();
		} else {
			$this->setError('MySQL: ' . $this->con->error(), 500);
			return false;
		}
	}
	
	/**
	Get the current "main" theme. It can be used for the path to
	the images and css.
	*/
	function getTheme()
	{
		global $_PX_config;
		$theme = $this->getPref('theme');
		// check if the path exists else reset to 'default'
		if (strlen($theme) > 0 && file_exists($_PX_config['manager_path'].'/themes/'.$theme.'/')) {
			return $theme;
		} else {
			return 'default';
		}
	}

	/**
	Get the current "plugin" theme. It can be used for the path to
	the images and css of the current plugin, as the "main" theme
	may not be available for the plugin.

	@param string id of the plugin (plugin folder)

	*/
	function getPluginTheme($plugin)
	{
		global $_PX_config;
		$theme = $this->getPref('theme');
		// check if the path exists else reset to 'default'
		if (strlen($theme) > 0 && file_exists($_PX_config['manager_path'].'/tools/'.$plugin.'/themes/'.$theme.'/')) {
			return $theme;
		} else {
			return 'default';
		}
	}

	/** get user preferences. If no userid given, use the internal
	one. If no internal, return false. Return array of prefs if
	succes. Set the internal prefs to the retrieved prefs.

	@author loic d'Anterroches
	@return false or array
	@param  int optional userid
	*/
	function getPrefs($user = '')
	{
		$this->prefs = array();
		if (strlen($user) == 0 && $this->_userid) $user = $this->_userid;
		if (!$this->con->con_id) {
			return false;
		}
		if ((int)$user > 0) {
			$r = 'SELECT * FROM '.$this->con->pfx.'userprefs WHERE user_id
				  LIKE \''.$this->con->escapeStr($user).'\'';
			if (($rs = $this->con->select($r)) !== false) {
				$d = $rs->getData();
				while (list( , $val) = each($d)) {
					$this->prefs[$val['keyname']][$val['website_id']] = $val['data'];
				}
				$this->_userid = $user;
				$this->lang = $this->getPref('lang', $this->website);
				return $this->prefs;
			} else {
				$this->setError('MySQL: ' . $this->con->error(), 500);
				return false;
			}
		} else {
			return false;
		}
	}

	/** get one preference.

	@author loic d'Anterroches
	@return string preference or empty string if no pref
	@param string key
	@param string website id
	*/
	function getPref($key, $websiteid = '')
	{
		global $_PX_config;
		if (strlen($key) == 0)
			return '';
		if (strlen($websiteid) == 0)
			$websiteid = $this->website;
		if (!empty($this->prefs[$key][$websiteid]))
			return $this->prefs[$key][$websiteid];
		if (!empty($this->prefs[$key]['#all#']))
			return $this->prefs[$key]['#all#'];
		if (!empty($_PX_config[$key])) return $_PX_config[$key];
		return '';
	}
	/** get user websites. Auto set the user id and save internally the websites.

	@author loic d'Anterroches
	@return false or array (key websites => value level)
	@param  int user id optional
	*/
	function getWebsites($user = '')
	{
		if (!$this->con->con_id) {
			return false;
		}
	   $this->webs = array();
		if (strlen($user) == 0 && $this->_userid) $user = $this->_userid;
		if ((int) $user > 0) {
			$r = 'SELECT * FROM '.$this->con->pfx.'grants
			      LEFT JOIN '.$this->con->pfx.'websites ON '.$this->con->pfx.'websites.website_id='.$this->con->pfx.'grants.website_id
			      WHERE user_id LIKE \''.$this->con->escapeStr($user).'\'';
			if (($rs = $this->con->select($r)) !== false) {
				while (!$rs->EOF()) {
					$this->webs[$rs->f('website_id')]  = $rs->f('level');
					$this->wdata[$rs->f('website_id')]['website_name']  = $rs->f('website_name');
					$this->wdata[$rs->f('website_id')]['website_url']   = $rs->f('website_url');
					$this->wdata[$rs->f('website_id')]['website_reurl'] = $rs->f('website_reurl');
					$this->wdata[$rs->f('website_id')]['website_xmedia_path'] = $rs->f('website_xmedia_path');
					$rs->moveNext();
				}
				$this->_userid = $user;
				return $this->webs;
			} else {
				$this->setError('MySQL: ' . $this->con->error(), 500);
				return false;
			}
		} else {
			return false;
		}
	}

	/** remove a pref for a user in the database, and in the
	user object.

	@author loic d'Anterroches
	@return bool success
	@param string key
	@param string website id #all# by default
	@param int (1 for all websites include the #all#, 0 else default)
	*/
	function removePref($key, $website = '#all#', $all = 0, $sessiononly = false)
	{
		if (strlen($key) == 0) return false;

		if (!empty($_SESSION['prefs'][$key][$website]))
			unset($_SESSION['prefs'][$key][$website]);
		if (!empty($this->prefs[$key][$website]))
			unset($this->prefs[$key][$website]);

		if (false == $sessiononly) {
			if (!$this->con->con_id) {
				return false;
			}
			$extra = '';
			if (0 == $all) $extra = 'website_id=\''.$this->con->escapeStr($website).'\' AND ';
			$r = 'DELETE FROM '.$this->con->pfx.'userprefs
			      WHERE user_id=\''.$this->_userid.'\' AND
			      '. $extra .' keyname=\''.$this->con->escapeStr($key).'\'';
			return $this->con->execute($r);
		}
		return true;
	}

	function savePref($key, $value, $website = '#all#', $sessiononly = false)
	{

		if (false === ($error = $this->removePref($key, $website, 0, $sessiononly))) {
			return $this->con->error();
		}

		if (strlen($value) > 0) {
	  		// the call to removePref() already makes some check on $key and $website
	  		if (false === $sessiononly) {
				$r = 'INSERT INTO '.$this->con->pfx.'userprefs  SET
				      user_id=\''.$this->_userid.'\',
				      website_id=\''.$this->con->escapeStr($website).'\',
				      keyname =\''.$this->con->escapeStr($key).'\',
				      data =\''.$this->con->escapeStr($value).'\'';

				if (false === $this->con->execute($r)) {
					return $this->con->error();
				}
     		}
			$_SESSION['prefs'][$key][$website] = $value;
			$this->prefs[$key][$website] = $value;
			$this->synchronize();
		}
		return true;
	}
	
	/* for the particular case of the textarea, we can increase or decrease them */
	function increase($area)
	{
		$p = $this->getPref($area);
		$p = $p + 5;
		if ($p > 100) $p = 100;
		$this->savePref($area, $p);
		return $p;
	}

	/* for the particular case of the textarea, we can increase or decrease them */
	function decrease($area)
	{
		$p = $this->getPref($area);
		$p = $p - 5;
		if ($p < 5) $p = 5;
		$this->savePref($area, $p);
		return $p;
	}


	function setWebsite($website)
	{
		$this->website = $website;
		$this->lang	= $this->getPref('lang', $website);
		if (false === @setlocale(LC_ALL, $this->lang)) {
			@setlocale(LC_ALL, $this->lang.'_'.strtoupper($this->lang));
		}
	}

	function synchronize($from = PX_USER_SYNCHRO_FROM_SESSION)
	{
		if (PX_USER_SYNCHRO_FROM_SESSION == $from) {
			$this->user['user_id']       = $_SESSION['user_id'];
			$this->_userid               = $_SESSION['user_id'];
			$this->user['user_username'] = $_SESSION['user_username'];
			$this->user['user_realname'] = $_SESSION['user_realname'];
			$this->prefs                 = $_SESSION['prefs'];
			$this->webs                  = $_SESSION['webs'];
			$this->wdata                 = $_SESSION['wdata'];
			$this->website               = $_SESSION['website_id'];
			$this->user                  = $_SESSION['user'];
			$this->lang                  = $this->getPref('lang', $this->website);

			// set the locale
			if (false === @setlocale(LC_ALL, $this->lang)) {
				@setlocale(LC_ALL, $this->lang.'_'.strtoupper($this->lang));
			}
		} else {
			$_SESSION['user_id']       = $this->user['user_id'];
			$_SESSION['user_username'] = $this->user['user_username'];
			$_SESSION['user_realname'] = $this->user['user_realname'];
			$_SESSION['prefs']         = $this->prefs;
			$_SESSION['webs']          = $this->webs;
			$_SESSION['wdata']         = $this->wdata;
			$_SESSION['website_id']    = $this->website;
			$_SESSION['user']          = $this->user;
			$_SESSION['lang']          = $this->getPref('lang', $this->website);
			setcookie('lang',       $_SESSION['lang'],       time()+31536000);
			setcookie('website_id', $_SESSION['website_id'], time()+31536000);
		}
	}

} // end class User
?>
