<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

define('PX_AUTH_ROOT',     10);
define('PX_AUTH_ADMIN',     9);
define('PX_AUTH_ADVANCED',  5);
define('PX_AUTH_NORMAL',    1);
define('PX_AUTH_DISABLE',   0);

/**
 Check the authentification level and website.
 If the authentification is not ok, the user is sent to the login page

 @return void
 @param  int level
 @param  string website id
*/
function auth($level = PX_AUTH_NORMAL, $website = 'default')
{
	global $_PX_context, $_PX_config;
	if (empty($_SESSION['webs'])    ||
		empty($_SESSION['user_id']))
		authGoToLogin();
	if ($_SESSION['webs'][$website] < $level && $_SESSION['user_id'] != 1)
		authGoToLogin();
	if (strlen($_SESSION['wdata'][$website]['website_reurl'])) {
		$_PX_context['out'] = substr($_SESSION['wdata'][$website]['website_url'], 0, -strlen($_SESSION['wdata'][$website]['website_reurl']));
	} else {
		$_PX_context['out'] = $_SESSION['wdata'][$website]['website_url'];
	}
	//remember the website
	setcookie('website_id', $website, time()+31536000);

}

function authAsLevel($level = PX_AUTH_NORMAL, $website = 'default', $user ='')
{
	if (empty($user)) {
		if ($level == PX_AUTH_ROOT)
			return ($_SESSION['user_id'] == 1);

		if (!isset( $_SESSION['webs'][$website])) return false;
		return  ($_SESSION['webs'][$website] >= $level);

	} else {
		// direct check of a user
		if ($user->f('user_id') > 0) {
			$user->getWebsites($user->f('user_id'));
		}
		if ($level == PX_AUTH_ROOT)
			return ($user->_userid == 1);
		if (!isset( $user->webs[$website])) return false;
		return  ($user->webs[$website] >= $level);

	}
}

function authGoToLogin()
{
	$l = getCurrentRelativeUrl().'login.php';
	header('Location: '.$l);
	exit;
}

/** Log a user to the system. If the user has the rights to access the system
user data are saved in the session. Some cookies are also set like the
language, the website. To be used if the session timeout, so the user is
sent back with the right language in the last website.

The login scheme is:

- check if login/password ok, if not return false
- get the authorized websites if no websites return false
- get the default website for the user, if not set to the first of the
authorized websites and return true
- check if the default website is in the list of authorized, if not, remove it
from the user prefs and set the default as the first authorized website and
return true.

@author loic d'Anterroches
@return bool success
@param string username
@param string password
@param int level [1]
@param string website []
*/

function authLogin($user = '', $pswd = '', $level = PX_AUTH_NORMAL, $website = '')
{
	global $_PX_config;

	if (0 == strlen($user) || 0 == strlen($pswd)) return false;
	if (preg_match('/[^A-Za-z0-9]/', $user)) return false;
	if (preg_match('/ /', $pswd)) return false;
	$ok = false;
	$u = new User();
	if ($u->checkUser($user, $pswd, $level)) {
		$u->set($user);
		if (!is_array($u->webs)) {
			// no authorized web
			return false;
		}
		if (strlen($website) > 0 && !isset($u->webs[$website])) {
			// the one provided is not good, it can be a fake cookie so skip
			$website = '';
		}
		if (strlen($website) == 0) {
			$website = $u->getPref('default_website', '#all#'); //default website for the user
			if (strlen($website) > 0 && !isset($u->webs[$website])) {
				// current default is not authorized! remove from prefs
				$u->removePref('default_website','#all#');
				$website = '';
			}
		}
		if (strlen($website) == 0) {
			// get a website for the user
			$website = array_pop(array_keys($u->webs));
		}
		$u->setWebsite($website);
		$u->synchronize(PX_USER_SYNCHRO_TO_SESSION, $website);
		$ok = true;
	}
	return $ok;
}


?>
