<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

require_once dirname(__FILE__).'/lib.path.php';

/** call the wiki parser depending of the content of the string

@author loic d'Anterroches
@param string unparsed content
@param string output format, default html or text
@return  string parsed content

*/
function textParseContent($text='', $output='html')
{
    if (0 == strlen($text)) return '';
    if ($output == 'html') {
    	$output = '';
	} else {
		$output = 'To'.$output;
	}
    $format = 'wiki';
    if (preg_match('/^=([A-Za-z]{4})+\s*/', $text, $match)) {
        $text = substr($text, 5);
        $format  = trim($match[1]);
    }
    if (function_exists('textParse'.$format.$output)) $parsefunc = 'textParse'.$format.$output;
    else $parsefunc = 'textParseWiki';
    return $parsefunc($text);
}

/** return type of text (wiki, html, etc.) or false if type
not found
*/
function textGetType($text = '')
{
    if (0 == strlen($text)) return false;
    $format = false;
    if (preg_match('/^=([A-Za-z]{4})\s*/', $text, $match)) {
        $format  = trim($match[1]);
    }
    return $format;
}

function textGetRawContent($text = '')
{
    if (0 == strlen($text)) return '';
    if (preg_match('/^=[A-Za-z]{4}\s*/', $text)) {
        $text = substr($text, 5);
    }
    return trim($text);
}

function textParseWiki($text)
{
    global $_PX_config, $_PX_context;
    include_once $_PX_config['manager_path'].'/extinc/wikirender/WikiRenderer.lib.php';
	include_once $_PX_config['manager_path'].'/extinc/wikirender/WikiRenderer_w2x.conf.php';

 	$wkr = new WikiRenderer(new Wiki2XhtmlConfig());
    return $wkr->render($text);
}

function textParseWikiToText($text)
{
    global $_PX_config, $_PX_context;
    include_once $_PX_config['manager_path'].'/extinc/wikirender/WikiRenderer.lib.php';
	include_once $_PX_config['manager_path'].'/extinc/wikirender/WikiRenderer_w2text.conf.php';

 	$wkr = new WikiRenderer(new WikiRenderer_w2text());
    return $wkr->render($text);
}

function textParseHtml($text)
{
    while (preg_match('#<xlink\s+id="(.*)"\s+type="(.*)"\s*/>#i',            $text, $match) or
           preg_match('#<xlink\s+id="(.*)"\s+type="(.*)"\s*>(.*)</xlink>#i', $text, $match)
          ) {
        $all   = quotemeta($match[0]);
        $id    = $match[1];
        $type  = $match[2];
        $title = '';
        if (isset($match[3])) $title = $match[3];
        $replace = textXlinkCreate($id, $type, $title);
        $title   = '';
        $text    = preg_replace('#'.$all.'#', $replace, $text);
    }
    return $text;
}

function textParseHtmlToText($text)
{
	include_once $_PX_config['manager_path'].'/extinc/class.html2text.php';
	$h2t = new html2text($text);
	return $h2t->get_text();
}

/** create a link to a resource in the CMS

@return string the HTML tag with the link
@param  string id of the resource (art-12, file-2, news-1, etc.)
@param  string format of the link 'textlink', 'img' or 'icon'
@param  string title for the link
*/
 
function textXlinkCreate($id, $type = 'textlink', $title = '')
{
	include_once dirname(__FILE__).'/class.manager.php';
    $m = new Manager();
    $r = $m->getResourceByIdentifier($id);
    if ($r->isEmpty()) {
        return '';
    } else {        
        if (strlen($title)) $r->setField('title',$title);
        
        if (function_exists('textXlinkCreate'.$type)) $createfunc = 'textXlinkCreate'.$type;
        else $createfunc = 'textXlinkCreateTextLink';
        
        return $createfunc($r);
    }
}

function textXlinkCreateIcon($res)
{
    global $_PX_context;
    if (preg_match('/\.([A-Za-z0-9]{1,4})$/', $res->f('path'), $preg)) $ext = $preg[1];
    else $ext = 'default';

    $path               = pathToResource($res);
    $description        = htmlspecialchars($res->f('description'));
    $cleanpath          = htmlspecialchars($res->f('path'));

    if (isset($_PX_context['out'])) $base = $res->f('website_url');
    else $base = $res->f('website_reurl');
    
    $r =  '<div class="pxIcon"><a href="'.$path.'"><img src="'.$base.$res->f('website_xmedia_reurl').'/icons/'.$ext.'-dist.png"';
    $r .= ' title="'.$description.' ('.$cleanpath.')" alt="'.$description.' ('.$cleanpath.')" /></a><br /> ';
    $r .= '<p>'.$res->f('title').'</p></div>';
    return $r;
}

function textXlinkCreateImg($res)
{
    $path               = pathToResource($res);
    $description        = htmlspecialchars($res->f('description'));
    $cleanpath          = htmlspecialchars($res->f('path'));

    return '<img src="'.$path.'" title="'.$res->f('title').'" alt="'.$res->f('title').'" />';
}

function textXlinkCreateTextLink($res)
{
    $path = pathToResource($res);
    return '<a href="'.$path.'">'.$res->f('title').'</a>';
}

/** hex encode a string not to be 'seen' by a spam system

  @return string encoded
  @param  string not encoded
  @param  bool   is the string for outside a &lt;a ...&gt; link?
*/  
function textHexEncode($str, $text = false)
{
    $encoded = '';
    if ($text) {
        for ($i=0; $i < strlen($str); $i++) {
            $encoded .= '&#x' . bin2hex($str[$i]).';';
        }
    } else {
        $encoded = bin2hex($str);
        $encoded = chunk_split($encoded, 2, '%');
        $encoded = '%'.substr($encoded, 0, strlen($encoded) - 1);
    }
    return $encoded;
} 

/** remove entities of a string

@note Available under MPL 1.1/GPL 2.0/LGPL 2.1 with DotClear Weblog
More details: http://www.dotclear.net/
@author Olivier Meunier
@return string with entities
@param  string without
*/

function textRemoveEntities($string)
{
	if (strtolower($GLOBALS['_PX_config']['encoding']) == 'utf-8') 
		$string = utf8_decode($string);

    # Tableau des codes de 130  140 et 145  156
    $tags = array('' => '&sbquo;','' => '&fnof;','' => '&bdquo;',
        '' => '&hellip;','' => '&dagger;','' => '&Dagger;','' => '&circ;',
        '' => '&permil;','' => '&Scaron;','' => '&lsaquo;','' => '&OElig;',
        '' => '&lsquo;','' => '&rsquo;','' => '&ldquo;','' => '&rdquo;',
        '' => '&bull;','' => '&ndash;','' => '&mdash;','' => '&tilde;',
        '' => '&trade;','' => '&scaron;','' => '&rsaquo;','' => '&oelig;',
        '' => '&Yuml','' => '&euro;');
    
    $vtags = array(
        '' => '&#8218;','' => '&#402;','' => '&#8222;','' => '&#8230;',
        '' => '&#8224;','' => '&#8225;','' => '&#710;','' => '&#8240;',
        '' => '&#352;','' => '&#8249;','' => '&#338;','' => '&#8216;',
        '' => '&#8217;','' => '&#8220;','' => '&#8221;','' => '&#8226;',
        '' => '&#8211;','' => '&#8212;','' => '&#732;','' => '&#8482;',
        '' => '&#353;','' => '&#8250;','' => '&#339;','' => '&#376;',
        '' => '&#8364;');
    
    $tags = array_merge($tags,get_html_translation_table(HTML_ENTITIES));
    foreach($tags as $k => $v) {
        $ASCIItags[$k] = '&#'.ord($k).';';
    }

    $string = str_replace($tags,array_flip($tags),$string);
    $string = str_replace($ASCIItags,array_flip($ASCIItags),$string);
    $string = str_replace(array_values($vtags),array_keys($vtags),$string);
    
    return if_utf8($string);
}

/** 
@note Available under MPL 1.1/GPL 2.0/LGPL 2.1 with DotClear Weblog
More details: http://www.dotclear.net/
@author Olivier Meunier
*/
function textToXML($string,$utf8=true)
{
	$string = htmlspecialchars(textRemoveEntities($string),ENT_NOQUOTES);

	if($utf8) {
		$string = utf8_encode($string);
	}
	return $string;
}

/** Convert an URL into words (space delimited) for a query
*/
function textUrl2words($url)
{
	$url = ' '.removeFileExtension($url);
	$url = preg_replace('/[^a-z0-9]/i',' ', $url);
	$url = preg_replace('/[a-z]([A-Z])([A-Z])([a-z]|$)/', " \\1 \\2\\3", $url);
	$url = preg_replace('/([A-Z][a-z0-9]+)/', " \\1", $url);
	$url = preg_replace('/( [0-9]+ )/', ' ', $url);
	return trim(preg_replace('/([A-Z][A-Z]+)/', " \\1", $url));
}


/** Convert a string into an URL string.

Je n'aime pas crire du code becomes Je-n-aime-pas-ecrire-du-code

@note Available under MPL 1.1/GPL 2.0/LGPL 2.1 with DotClear Weblog
More details: http://www.dotclear.net/
@author Olivier Meunier
@return string url
@param string
*/
function textStr2url($str, $sep='-')
{
    $str = textRemoveEntities($str);
	if (strtolower($GLOBALS['_PX_config']['encoding']) == 'utf-8') 
		$str = utf8_decode($str);

    $str = strtr($str,"","AAAAAAaaaaaaOOOOOOooooooEEEEeeeeIIIIiiiiUUUUuuuuyNn");
    $str = preg_replace('/-/',' ',$str);    
    $str = preg_replace('/[^a-z0-9_\s]/','',strtolower($str));

    $str = preg_replace('/[\s]+/',' ',trim($str));
    
    $T = explode(' ',$str);
    array_walk($T,create_function('&$v,$k','$v=ucfirst($v);'));
    $res = implode($sep,$T);
    
    return $res;
}

/** Clean a string.

@note Available under MPL 1.1/GPL 2.0/LGPL 2.1 with DotClear Weblog
More details: http://www.dotclear.net/
@author Olivier Meunier
@return
@param
*/
function textSecureString($str)
{
    $str = trim($str);
    $str = stripslashes($str);
    $str = strip_tags($str);
    $str = htmlspecialchars($str);
    return $str;
}


?>
