<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

/** Create a timestamp with an offset with respect to time or today.

@author loic d'Anterroches
@return int timestamp now by default
@param  int day offset 
@param  int month offset
@param  int year offset
@param  int time
*/
function timestamp($day = 0, $month = 0, $year = 0, $time = '')
{
    $time = (strlen($time)) ? $time : time();
    return date('YmdHis', mktime(date('H',$time), date('i',$time), date('s',$time), date('m',$time) + $month, date('d',$time) + $day, date('Y',$time) + $year));
}

/** Round a timestamp to the day, month or year.

For example 20031202123212 => 20030101000000, 20031201000000, 20031203000000

@return int timestamp
@param  int timestamp
@param  string 'd', 'm' [default] or 'y'            
*/
function roundStamp($time, $f = 'm')
{
    $n['y'] = 4;
    $n['m'] = 6;
    $n['d'] = 8;
    return str_pad(str_pad(substr($time, 0, $n[$f]),8,'01'),14,'0');
}

function dateMonth($offset = 0)
{
    return roundStamp(timestamp(0,$offset),'m'); 
}

function dateDay($offset = 0)
{
    return roundStamp(timestamp($offset),'d');
}

function dateYear($offset = 0)
{
    return roundStamp(timestamp(0,0,$offset),'y');
}

/** Convert MySQL timestamp in unix time.
@return int unix time
@param  int mysql timestamp
*/
function makeTime($string)
{
	if (!empty($string))
	    return mktime(substr($string,8,2),substr($string,10,2),substr($string,12,2),
                substr($string,4,2),substr($string,6,2),substr($string,0,4));
	return 0;				
}

/** Convert a string with the format
 yyyymmddhhmmss or yyyy-mm-dd hh:mm:ss
 into a list that can be passed to mktime
 */
function explodeDate($string)
{
    $res = array();
    if (strlen($string) == 14) {
        $res[] = substr($string,8,2);
        $res[] = substr($string,10,2);
        $res[] = substr($string,12,2);
        $res[] = substr($string,4,2);
        $res[] = substr($string,6,2);
        $res[] = substr($string,0,4);
    } elseif (preg_match('/(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)/',$string,$match)) {
        $res[] = $match[4];
        $res[] = $match[5];
        $res[] = $match[6];
        $res[] = $match[2];
        $res[] = $match[3];
        $res[] = $match[1];
    } else {
        $res = false;
    }        
    return $res;
}

/** Check the validity of a date, if not valid try to force it
to be valid.

@return int mysql timestamp or false
@param  string date time
*/
function checkCleanDate($string)
{
    if (false !== ($list = explodeDate($string))) {
        list($dt_h, $dt_i, $dt_s, $dt_m, $dt_d, $dt_y) = $list;

		$dt_y = (string) sprintf('%04d',$dt_y);
		$dt_m = (string) sprintf('%02d',$dt_m);
		$dt_d = (string) sprintf('%02d',$dt_d);
		$dt_h = (string) sprintf('%02d',$dt_h);
		$dt_i = (string) sprintf('%02d',$dt_i);
		$dt_s = (string) sprintf('%02d',$dt_s);
        
		if ($dt_d > 31 || $dt_d < 1) { $dt_d = '01'; }
		if ($dt_h > 23 || $dt_h < 0) { $dt_h = '00'; }
		if ($dt_i > 59 || $dt_i < 0) { $dt_i = '00'; }
		if ($dt_s > 59 || $dt_s < 0) { $dt_s = '00'; }
           
        if (!checkdate($dt_m, $dt_d, $dt_y)) {
            // try to 'clean' the date
            $date = @date('YmdHis', @mktime($dt_h, $dt_i, $dt_s, $dt_m, $dt_d, $dt_y));
            if (14 == strlen($date)) return $date;
            else return false;
        }
        return $dt_y.$dt_m.$dt_d.$dt_h.$dt_i.$dt_s;
    }
    return false;
}

/** Inform if the category is an hidden category or not.
@return bool
@param  category path
*/
function isGhostCat($path='')
{
    return preg_match('#/_#', $path);
}


function strlenCompare($a, $b) 
{
    $a = textRemoveEntities($a);
    $b = textRemoveEntities($b);
    if (strlen($a) == strlen($b)) {
        return 0;
    }
    return (strlen($a) > strlen($b)) ? -1 : 1;
}

/** get the relative url of the website as
    automatically as possible */
function getRelativeUrl() 
{    
    if (preg_match('#(.*)/manager(.*)#i', $_SERVER["SCRIPT_NAME"], $match)) 
        return $match[1];
    if (preg_match('#(.*)/\w+\.php#i', $_SERVER["SCRIPT_NAME"], $match))            
        return $match[1];
    return '';        
}

/**
get the current relative url for redirection (with / at the end)
*/
function getCurrentRelativeUrl()
{
	$s = '';
    if (isset($_SERVER['HTTPS']))
       $s = 's';
	return 'http'.$s.'://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']).'/';
}

/** 
get the current host url no / at the end
*/
function getCurrentHostUrl()
{
	$s = '';
    if (isset($_SERVER['HTTPS']))
       $s = 's';
	return 'http'.$s.'://'.$_SERVER['HTTP_HOST'];
}

/** Give is a file is safe or not
   @return bool
   @param file name
   @param path to the data
*/
function isFileSafe($file_name, $file_data ='')
{
    if (preg_match('/[^A-Za-z0-9\-\_\.]/',$file_name)) return false;
    if (!preg_match('/\.(png|jpg|jpeg|gif|bmp|psd|tif|aiff|asf|avi|bz2|css|doc|eps|gz|htm|mid|mov|mp3|mpg|ogg|pdf|ppt|ps|qt|ra|ram|rm|rtf|sdd|sdw|sit|sxi|sxw|swf|tgz|txt|wav|xls|xml|wmv|zip)$/i',$file_name)) return false;
    return true;
}

function prettySize($size)
{
    $mb = 1024*1024;
    if ( $size > $mb ) {
        $mysize = sprintf('%01.2f', $size/$mb) . ' ' .  __('MB');
    } elseif ($size >= 1024) {
        $mysize = sprintf('%01.2f', $size/1024) . ' ' . __('KB');
    } else {
        $mysize = sprintf('%01.2f', $size/1024) . ' ' . __('bytes');
    }
    return $mysize;
}

function isImage($file_name)
{
    if (preg_match('/\.(png|jpg|gif|jpeg)$/i',$file_name)) return true;
    return false;
}

function getFileExtension($file_name)
{
    if (preg_match('/\.([A-Za-z0-9]{2,4})$/i',$file_name, $match)) return $match[1];
    return 'default';
}

function removeFileExtension($file_name)
{
	return preg_replace('/(\.[A-Za-z0-9]{2,4})$/i', '', $file_name);
}

function getParentDir($current_dir)
{
    if (preg_match('#(.*/)*([^/])+/$#i',$current_dir, $match)) return $match[1];
    return '';
}

function cleanDirname($dir)
{
    return str_replace('\\','/',$dir);
}

/**
 utf 8 encode a string if the global encoding is utf-8
*/
function if_utf8($string)
{
	if (strtolower($GLOBALS['_PX_config']['encoding']) == 'utf-8') return utf8_encode($string);
	return $string;
}

function showDebugInfo()
{
    echo '<!-- '."\n";
	echo 'Loaded locale files';
	if (!empty($GLOBALS['_PX_locale_files'])) {
		echo ' [original encoding]:'."\n";
		foreach($GLOBALS['_PX_locale_files'] as $file => $encoding) {
			echo $file .' ['.$encoding."]\n";
		}
	} else {
		echo ': None loaded.'."\n";
	}
	echo "\n";
	echo 'Untranslated strings:';
	if (!empty($GLOBALS['_PX_debug_data']['untranslated'])) {
		echo "\n";
		foreach($GLOBALS['_PX_debug_data']['untranslated'] as $string) {
			echo '    \''.$string."'\n";
		}
	} else {
		echo ' None.'."\n";
	}
	echo "\n";
	echo 'SQL queries:';
	if (!empty($GLOBALS['_PX_debug_data']['sql_queries'])) {
		echo "\n";
		foreach($GLOBALS['_PX_debug_data']['sql_queries'] as $string) {
			echo '    '.str_replace("\n", '', $string)."\n";
		}
		echo 'Queries starting with * are direct mysql function calls.'."\n";
	} else {
		echo ' None.'."\n";
	}
    echo '-->';
}
?>
