<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Credits:
#   Olivier Meunier. The administration part of PLUME CMS starting 
#   version 1.0RC1 is mainly based on DotClear Weblog by Olivier Meunier. 
#   More details: http://www.dotclear.net/
#   DotClear Weblog is distributed under the License MPL 1.1/GPL 2.0/LGPL 2.1
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

require_once 'path.php';
require_once $_PX_config['manager_path'].'/prepend.php';
auth(PX_AUTH_ADMIN, (!empty($_SESSION['website_id']))?$_SESSION['website_id']:'');

/*=================================================
 Create the manager object, load the libs and class
=================================================*/
$u = new User();
$u->synchronize(PX_USER_SYNCHRO_FROM_SESSION);
$m = new Manager();
$m->setUser($u);
$_px_theme = $m->user->getTheme();

/*=================================================
 Get some info of the current step for the submenu
=================================================*/
if (empty($_REQUEST['site_id']) && empty($_REQUEST['op'])) $display_add_site = true;
else $display_add_site = false;
$is_full_editable = authAsLevel(PX_AUTH_ROOT); //full right to modify the current site and add. Only the root user.
if ($is_full_editable) {
	$px_submenu->addItem( __('New site'),  'sites.php?op=add',  'themes/'.$_px_theme.'/images/ico_new.png', false, $display_add_site);
}
$px_submenu->addItem( __('Back to the list of the sites'),  'sites.php',  'themes/'.$_px_theme.'/images/ico_back.png', false, !$display_add_site);

/*==============================================================================
 Process block
==============================================================================*/
$is_editable = true; //partial rights to modify the current website, see a partial list of websites (only the one with some rights)
$update = false;
$is_new_site = false; //switched to true only if the form to create a new site was submitted and the data in the form were
                      //ok to start the creation. A page with the log of the creation is then displayed.

if (empty($_REQUEST['op']) && empty($_REQUEST['site_id'])) {
	// list the sites
	$px_sites = $m->getSites();
} else {
	// edit a website
	// set default values
	$px_id               = '';
	$px_name             = __('New website name');
	$px_website_address  = '';
	$px_xmedia_name      = 'xmedia';
	$px_website_path     = '';
	$px_description      = __('Description of the __new website__.');
	$px_sitelang         = $m->user->lang;
	$arry_langs = l10n::getIsoCodes(true);

	if ('iso-8859-1' == strtolower($_PX_config['encoding'])) {
		//do not accept languages that can't be supported by the current encoding
		$west_langs = l10n::getIsoWestern();
		reset($arry_langs);
		while (list($key, $val) = each($arry_langs)) {
			if (!in_array($val, $west_langs))
				unset($arry_langs[$key]);
		}
	}

	if (!empty($_REQUEST['site_id']) && empty($_REQUEST['op'])) {
		// see if the website exists or not
		$px_site = $m->getSites($_REQUEST['site_id']);
		if (strlen($px_site->f('website_id')) == 0) {
			// site does not exist error
			$msg =  __('This site is not available.');
			header('Location: sites.php?msg='.urlencode($msg));
			exit;
		}
		if (!authAsLevel(PX_AUTH_ADMIN,$px_site->f('website_id'))) {
			// No rights
			$msg =  __('No rights to edit this website.');
			header('Location: sites.php?msg='.urlencode($msg));
			exit;
		}
		if (file_exists($_PX_config['manager_path'].'/conf/configweb_'.$px_site->f('website_id').'.php')) {
			include($_PX_config['manager_path'].'/conf/configweb_'.$px_site->f('website_id').'.php');
			if (!is_writable($_PX_config['manager_path'].'/conf/configweb_'.$px_site->f('website_id').'.php')) {
				$is_editable = false;
				$m->setError( __('Error: Permissions of the configuration prevent the addition or modification of the sites.'), 500);
			}
		} else {
			$m->setError(sprintf( __('Error: Configuration file of the website(<strong>%s</strong>) not available.'), files::real_path($_PX_config['manager_path'].'/conf/configweb_'.$px_site->f('website_id').'.php')), 500);
			$is_editable = false;
		}

		if ((false === $m->error(true, false)) && $is_editable) {
			$px_id           = $px_site->f('website_id');
			$px_name         = $px_site->f('website_name');
			$px_website_address = 'http'.(($_PX_website_config['secure']) ? 's': '').'://'.$_PX_website_config['domain'].$_PX_website_config['rel_url'];
			$px_xmedia_name  = trim(str_replace('/', '', substr($_PX_website_config['rel_url_files'], strlen($_PX_website_config['rel_url']))));
			$px_description  = $px_site->f('website_description');
			$px_sitelang     = $_PX_website_config['lang'];
			$px_website_path = substr(files::real_path($_PX_website_config['xmedia_root']), 0, (-1 - strlen($px_xmedia_name))); //  $rest = substr("abcdef", 0, -1);  // returns "abcde"
			$update = true;
		}
	}
	if (!$is_full_editable && (!empty($_REQUEST['op']) && 'add' == $_REQUEST['op'])) {
		// No rights
		$msg =  __('No rights to add a website.');
		header('Location: sites.php?msg='.urlencode($msg));
		exit;
	}
	if (!empty($_SESSION['log_new_site']) && (!empty($_REQUEST['op']) && 'log' == $_REQUEST['op'])) {
		$px_new_site = $_SESSION['log_new_site'];
		$is_new_site = true;
	}
}


/*=================================================
 Save the data
=================================================*/
if ($is_editable && (!empty($_POST['save']))) {
	// get data from form
	$px_name         = (!empty($_POST['s_name'])) ? trim($_POST['s_name']) : '';
	$px_description  = (!empty($_POST['s_description'])) ? trim($_POST['s_description']) : '';
	$px_sitelang     = (!empty($_POST['s_sitelang'])) ? trim($_POST['s_sitelang']) : '';
	if ($is_full_editable) {
		$px_website_address = (!empty($_POST['s_website_address'])) ? trim($_POST['s_website_address']) : '';
		$px_website_path    = (!empty($_POST['s_website_path'])) ? trim($_POST['s_website_path']) : '';
		$px_xmedia_name     = (!empty($_POST['s_xmedia_name'])) ? trim($_POST['s_xmedia_name']) : '';
	}
	$px_log_new_site = ''; //updated through a reference
	if (!empty($px_id)) {
		if (false !== $m->saveSite($px_id, $px_name, $px_description, $px_sitelang, $px_website_address, $px_website_path, $px_xmedia_name, $px_log_new_site)) {
			//$px_log_new_site is not updated, but is needed as a reference can't be set as NULL for default :o(
			$msg =  sprintf(__('Site <strong>%s</strong> successfully saved.'), $px_name);
			header('Location: sites.php?msg='.urlencode($msg));
			exit;
		}
	} else {
		if (false !== $m->saveSite($px_id, $px_name, $px_description, $px_sitelang, $px_website_address, $px_website_path, $px_xmedia_name, $px_log_new_site)) {
			$msg =  sprintf(__('Site <strong>%s</strong> successfully added.'), $px_name);
			$_SESSION['log_new_site'] = $px_log_new_site; //Save the log in the session, the header is to prevent a double "addition".
			header('Location: sites.php?op=log&msg='.urlencode($msg));
			exit;
		}
	}
}

if ($is_editable && (!empty($_POST['delete']))) {
	if (false !== $m->delSite($px_id)) {
		$msg =  __('Site successfully deleted. No file were removed from the document root.');
		header('Location: sites.php?msg='.urlencode($msg));
		exit;
	}
}

/*==============================================================================
 Display block
==============================================================================*/
/*=================================================
 Set title of the page, and load common top page
=================================================*/
$px_title =  __('Sites');
include $_PX_config['manager_path'].'/mtemplates/_top.php';

echo '<h1>'.__('Sites')."</h1>\n\n";

if($is_new_site) {
/*=================================================
 Show the log of the creation of a new website
=================================================*/
	echo '<h2>'.__('New site')."</h2>\n\n";
	
	echo '<p>'.__('Log of the installation:').'</p>'."\n\n";

	echo $px_new_site;

	echo '<p>'.__('If you see warnings, check them carefully and look at the online documentation to see the impact. You may need to complete the installation manually.').'</p>'."\n\n";

	echo '<p>'.__('To see the new website in the manager, you need to logout and login again.').'</p>'."\n\n";

	echo '<p>'.__('If you are using the nice url option, you need to configure the webserver to use these url with this new website.').'</p>'."\n\n";

} elseif (empty($_REQUEST['op']) && empty($_REQUEST['site_id'])) {
/*=================================================
 Show the list of websites.
=================================================*/
	while(!$px_sites->EOF()) {
		if (authAsLevel(PX_AUTH_ADMIN, $px_sites->f('website_id'))) {
			echo '<div class="line" id="p'.$px_sites->f('website_id').'"><p>';
			echo '<a href="'.$px_sites->f('website_url').'"><strong>'.$px_sites->f('website_name').'</strong></a>';
			echo ' <span class="small">('.$px_sites->f('website_id').')</span> ';
			echo '[<strong><a href="sites.php?site_id='.$px_sites->f('website_id').'">'. __('edit').'</a></strong>]<br />';
			echo $px_sites->f('website_description');
			echo "</p>\n\n";
			echo '<p>'.sprintf(__('<a href="%s">Manage the subtypes of resources</a>.'), 'subtypes.php').'</p>'."\n\n";
			echo "\n</div>\n\n";
		}
		$px_sites->moveNext();
	}
} elseif (($is_editable && !empty($_REQUEST['site_id'])) or ((!empty($_REQUEST['op']) && 'add' == $_REQUEST['op']) && $is_full_editable)) {
/*=================================================
 Show the page to edit/create a website
=================================================*/
	if (!empty($_REQUEST['op']) && 'add' == $_REQUEST['op']) {
		echo '<h2>'. __('New site')."</h2>\n\n";
	}
?>

<form action="sites.php" method="post" id="formPost">
<?php
	if (empty($_REQUEST['op']) or 'add' != $_REQUEST['op']) {
 ?>
	<p class="field"><label class="float" for="s_site_id" style="display:inline"><strong><?php  echo __('Site id:'); ?></strong></label>
	<strong><?php echo $px_id; ?></strong></p>
<?php
	}
?>
  <p class="field"><label class="float" for="s_name" style="display:inline"><strong><?php  echo __('Site name:'); ?></strong></label>
  <?php echo php_f_textField('s_name', 30, 255, $px_name, '', ''); ?>
  </p>
<?php
// in case of addition of a new website, we ask where the "index.php" folder is
	if ($is_full_editable or (!empty($_REQUEST['op']) && 'add' == $_REQUEST['op'])):  
	?>
  <p class="field"><label class="float" for="s_website_address" style="display:inline"><strong><?php  echo __('Website address:'); ?></strong></label>
  <?php echo php_f_textField('s_website_address', 50, 255, $px_website_address, '', ''); ?><br />
  <?php  echo __('For example put <em>https://www.mydomain.com/mysite/</em> or <em>http://www.mydomain.com/</em>. Do not forget the <em>http(s)</em>.'); ?>
  </p>

  		<p class="field"><label class="float" for="s_website_path" style="display:inline"><strong><?php  echo __('Path to the document root of the website:'); ?></strong></label>
		<?php echo php_f_textField('s_website_path', 30, 255, $px_website_path, '', '');
		
		if (!empty($_REQUEST['op']) && 'add' == $_REQUEST['op']):
			echo '<br />'."\n".__('Path on the server side. For example <em>/home/login/www</em> or <em>c:/http</em>. The document root is the folder where the <em>index.php</em> will be placed.');
		endif;

  	echo "\n".'</p>'."\n";
	endif;
?>

<?php if ($is_full_editable): ?>
  <p class="field"><label class="float" for="s_xmedia_name" style="display:inline"><strong><?php  echo __('Name of the file and image folder:'); ?></strong></label>
  <?php echo php_f_textField('s_xmedia_name', 15, 255, $px_xmedia_name, '', ''); ?><br />
  <?php  echo __('For example put <em>xmedia</em> or <em>documents</em>.'); ?>
  </p>
<?php endif; ?>

  <p class="field"><label class="float" for="s_sitelang" style="display:inline"><strong><?php  echo __('Main language of the website:'); ?></strong></label>
  <?php echo php_f_comboBox('s_sitelang', $arry_langs, $px_sitelang); ?>
  </p>

  <p>
  <label for="s_description"><strong><?php  echo __('Description:'); ?></strong></label>
  <?php
  echo php_f_textArea('s_description', 60, 4, htmlspecialchars($px_description), '', '');
  ?>
  </p>

<?php
if ($update) {
	echo php_f_hidden('site_id',$px_id);
    ?><p> <input name="save" type="submit" class="submit" value="<?php  echo __('Save [s]'); ?>"  accesskey="s" />&nbsp;
    <?php
} else {
	echo php_f_hidden('op','add');
	echo "\n".'<p>'.__('If all the information you provided are valid, the system will copy the required files in the right folders. The complete log of those files will be shown to you. No file already available on the system will be overwritten.').'</p>'."\n\n";
	?> <p> <input name="save" type="submit" class="submit" value="<?php  echo __('Create the new website'); ?>"  accesskey="s" />
	<?php
}
?>
<?php
if ($update && (false === $m->getEarlierDate('m', '', '', $px_id))) {
	echo '&nbsp;<input name="delete" type="submit" class="submit" '.
	'value="'.  __('Delete').'" onclick="return '.
	'window.confirm(\''.addslashes( __('Are you sure you want to delete this site?')).'\')" />';
}
echo "\n".'</p></form>';

}

/*=================================================
 Load common bottom page
=================================================*/
include $_PX_config['manager_path'].'/mtemplates/_bottom.php';

?>
