<?php
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is PLUME CMS.
#
# The Initial Developer of the Original Code is
# loic d'Anterroches.
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Credits:
#   Olivier Meunier. The administration part of PLUME CMS starting 
#   version 1.0RC1 is mainly based on DotClear Weblog by Olivier Meunier. 
#   More details: http://www.dotclear.net/
#   DotClear Weblog is distributed under the License MPL 1.1/GPL 2.0/LGPL 2.1
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****


require_once 'path.php';
require_once $_PX_config['manager_path'].'/prepend.php';
auth(PX_AUTH_ADMIN, (!empty($_SESSION['website_id']))?$_SESSION['website_id']:'');

/*=================================================
 Create the manager object, load the libs and class
=================================================*/
$u = new User();
$u->synchronize(PX_USER_SYNCHRO_FROM_SESSION);
$m = new Manager();
$m->setUser($u);
$_px_theme = $m->user->getTheme();

/*=================================================
 Submenu info
=================================================*/
$px_submenu->addItem( __('Back to the list of resources'), 'index.php', 'themes/'.$_px_theme.'/images/ico_back.png', false);
$px_submenu->addItem( __('List the types'), 'subtypes.php', 'themes/'.$_px_theme.'/images/ico_subtype.png', false);
$px_submenu->addItem( __('New type'), 'subtypes.php?op=add', 'themes/'.$_px_theme.'/images/ico_new.png', false);

/*=================================================
 Set all the default values
=================================================*/
$px_id         = '';
$px_type_id    = '';
$px_name       = '';
$px_website_id = '';
$px_template   = '';
$px_cachetime  = '';
$px_extra1     = '';
$px_extra2     = '';
$px_inuse      = true;

/*==============================================================================
 Process block
==============================================================================*/
// get list of subtypes
if (empty($_REQUEST['op']) && empty($_REQUEST['s_id'])) { 
    $px_subtypes = $m->getSubTypes();
} 

// get data for requested subtype 
if (!empty($_REQUEST['s_id'])) {
    if (false !== ($px_subtype = $m->getSubTypes($_REQUEST['s_id']))) {
        $px_id         = $px_subtype->f('subtype_id');
        $px_type_id    = $px_subtype->f('type_id');
        $px_name       = $px_subtype->f('subtype_name');
        $px_template   = $px_subtype->f('subtype_template');
        $px_cachetime  = $px_subtype->f('subtype_cachetime');
        $px_extra1     = $px_subtype->f('subtype_extra1');
        $px_extra2     = $px_subtype->f('subtype_extra2');
        
        $px_inuse = $m->isSubTypeUsed($px_id);
    }
}

/*=================================================
 Save/Add the type
=================================================*/
if (!empty($_POST['save'])) {

    $px_type_id    = trim($_POST['s_type_id']);
    $px_name       = trim($_POST['s_name']);
    $px_template   = trim($_POST['s_template']);
    $px_cachetime  = trim($_POST['s_cachetime']);
    $px_extra1     = (isset($_POST['s_extra1'])) ? trim($_POST['s_extra1']) : '' ;
    $px_extra2     = (isset($_POST['s_extra2'])) ? trim($_POST['s_extra2']) : '' ;
		
	if (false !== ($id = $m->saveType($px_id, $px_type_id, $px_name, $px_template, $px_cachetime, $px_extra1, $px_extra2))) {
		$msg =  __('The type was successfully saved.');
		header('Location: subtypes.php?msg='.urlencode($msg));
		exit; 
	}	
} 

if (!empty($_POST['delete'])) {
	if (false !== ($id = $m->deleteType($px_id))) {
		$msg =  __('The type of resource was successfully deleted.');
		header('Location: subtypes.php?msg='.urlencode($msg));
		exit; 
	}	
} 

	 
$arry_templates = $m->getTemplates('resource');
/*==============================================================================
 Display block
==============================================================================*/

/*=================================================
 Set title of the page, and load common top page
=================================================*/
$px_title =  __('Resource types');
include $_PX_config['manager_path'].'/mtemplates/_top.php';

echo '<h1>'. __('Resource types')."</h1>\n\n";

/*=================================================
 Add/Edit a subtype
=================================================*/
if (!empty($_REQUEST['s_id']) or !empty($_REQUEST['op'])):
?>

<form action="subtypes.php" method="post" id="formPost">
  <p class="field"><label class="float" for="s_type_id" style="display:inline"><strong><?php  echo __('New type of:'); ?></strong></label>
  <?php echo php_f_combobox('s_type_id',   array(  __('Article') => 'articles',   __('News') => 'news'), $px_type_id,'','', '',"onChange=\"openCloseBlockIf('p_extra1','s_type_id','news',1,-1);\""); ?></span>
  </p>

  <p class="field"><label class="float" for="s_name" style="display:inline"><strong><?php  echo __('Name of the type:'); ?></strong></label>
  <?php echo php_f_textField('s_name', 30, 30, $px_name, '', ''); ?> 
  </p>
  
  <p class="field"><label class="float" for="s_template" style="display:inline"><strong><?php  echo __('Template:'); ?></strong></label>
  <?php echo php_f_combobox('s_template', $arry_templates, $px_template); ?></span>
  </p>

  <p class="field"><label class="float" for="s_cachetime" style="display:inline"><strong><?php  echo __('Cache time in seconds:'); ?></strong></label>
  <?php echo php_f_textField('s_cachetime', 10, 20, $px_cachetime, '', ''); ?>
  </p>  
  
  <?php $p_extra1_style = ($px_type_id == 'news') ? '' : ' style="display: none"'; ?>
  
  <p class="field" id="p_extra1"<?php echo $p_extra1_style; ?>><label class="float" for="s_extra1" style="display:inline"><strong><?php  echo __('Have associated link to news items:'); ?></strong></label>
  <?php echo php_f_combobox('s_extra1', array(  __('Yes') => '1',  __('No') => '0'), $px_extra1); ?></span>
  </p>

  <?php
  if (!empty($px_id)) {
   	 echo php_f_hidden('s_id',$px_id);
  } else { 
	 echo php_f_hidden('op','add');
  }
  ?>
  <p> <input name="save" type="submit" class="submit" value="<?php  echo __('Save [s]'); ?>"
  accesskey="s" />  
  <?php 
    if (!$px_inuse) {
	echo '&nbsp;<input name="delete" type="submit" class="submit" '.
	'value="'.  __('Delete').'" onclick="return '.
	'window.confirm(\''.addslashes( __('Are you sure you want to delete this type of resource?')).'\')" />';
   
  }
  ?>
  </p>
</form>

<?php
endif;

/*=================================================
 Display the list of subtypes
=================================================*/
if (empty($_REQUEST['op']) && empty($_REQUEST['s_id'])):

$px_news_title = '<h2>'. __('Types of news').'</h2>'."\n";
$px_articles_title = '<h2>'. __('Type of articles').'</h2>'."\n";

$current = ''; //$px_subtypes->f('type_id');

if ($px_subtypes->nbRow() == 0) {
	echo '<p>'.sprintf( __('You need to <a href="%s">add 2 types of resource</a>, one for the articles and one for the news.'), 'subtypes.php?op=add').'</p>';
}
    
while (!$px_subtypes->EOF()) {
    if ($current != $px_subtypes->f('type_id')) {
        if ($px_subtypes->getIndex() != 0) {
            echo "</ul>\n";
        }
        $current = $px_subtypes->f('type_id');
        if ($current == 'articles') {
            echo $px_articles_title;
        } else {    
            echo $px_news_title;
        }    
        echo "<ul>\n";
    }
    echo '<li><a href="subtypes.php?s_id='.$px_subtypes->f('subtype_id').'">'.$px_subtypes->f('subtype_name').'</a></li>'."\n";
    
    $px_subtypes->moveNext();
}
echo "</ul>\n";
endif; 

/*=================================================
 Load common bottom page
=================================================*/
include $_PX_config['manager_path'].'/mtemplates/_bottom.php';
?>
